'******************************************************************************
'*  pm80.spin - a recompiling Z80 emulator
'*
'*  (c) Juergen Buchmueller <pullmoll@t-online.de>
'*
'* $Id: pm80.spin,v 1.38 2010-03-09 05:31:19 pm Exp $
'******************************************************************************
CON
	_xinfreq		= 5_000_000		' Quartz is 5MHz
	_clkmode		= xtal1 + pll16x	' System clock is 80MHz

	patch_imm16		=	1<<0		' flag indicates to patch an imm16 value
	patch_imm8		=	1<<1		' flag indicates to patch an imm8 value
	patch_jr_disp8		=	1<<2		' flag indicates to patch a relative jump
 	patch_xy_disp8		=	1<<3		' flag indicates to patch IX/IY disp8 value
	terminate		=	1<<4
'	patch_type5		=	1<<5
'	patch_type6		=	1<<6
'	patch_type7		=	1<<7
	jump_to			=	1<<8

	c_flag			=	%00000001	' carry
	n_flag			=	%00000010	' negative
	p_flag			=	%00000100	' parity (also overflow)
	x_flag			=	%00001000	' undocumented x
	h_flag			=	%00010000	' auxiliaray (half) carry
	y_flag			=	%00100000	' undocumented y
	z_flag			=	%01000000	' zero
	s_flag			=	%10000000	' sign

	bit0			=	%00000001
	bit1			=	%00000010
	bit2			=	%00000100
	bit3			=	%00001000
	bit4			=	%00010000
	bit5			=	%00100000
	bit6			=	%01000000
	bit7			=	%10000000

	OP_FLAGS_SHIFT		=	23
	CYCLES_SHIFT		=	8

	tvPin			=	14

' Uncomment to do cycle counting
' #define DO_CCOUNT

' Uncomment to do refresh register counting
' #define DO_RCOUNT

' Uncomment to have debug code inserted
' #define	DEBUG

VAR
	long  cog

OBJ
	tv	: "Debug_1pinTV"

PUB start(ram, io) : okay

'' Start CPU simulator - starts a cog
'' returns false if no cog available
''
'' ram is a pointer to Z80 RAM space
'' io is a pointer to Z80 I/O space

	stop
        tv.start(tvPin)        'start the Debug 1pinTV driver
        tv.chr(0)              'clear screen
        tv.str(string("PM80 firing up...",13))

	reloc := @entry - $10
	ram_base := ram
	io_base := io
	dbg := tv.RENDEZVOUS
	snippets_00_table := @snippets_00
	snippets_cb_table := @snippets_cb
	snippets_dd_table := @snippets_dd
	snippets_ed_table := @snippets_ed
	snippets_fd_table := @snippets_fd
	okay := cog := COGNEW(@entry, 0) + 1

PUB stop

'' Stop CPU simulator - frees a cog

	if cog
		cogstop(cog~ - 1)

DAT
'---------------------------------------------------------------------------------------------------------

		org	0

'---------------------------------------------------------------------------------------------------------
entry
hubptr					' contains hub RAM address for ea (overwrites jmp)
		jmp	#startup

'******************************************************************************
'
' cog variables
'
reloc		long	0-0		' relocation address for tables + functions
ram_base	long	0-0		' base address of the Z80 RAM
io_base		long	0-0
dbg		long	0-0

#ifdef	DO_CCOUNT
cc		long	0		' cycle counter
#endif
ea		long	0		' effective address
opcode		long	0		' opcode
halt		long	0		' HALT flag, set by opcode 76

alu		long	0		' arithmetic logic unit (sort of :)
tmp		long	0		' generic temporary
aux		long	0		' auxiliary flag temporary

dst_pc		long	0		' compiler output program counter
max_pc		long	0		' max. address for compiler output
snippet		long	0		' start address of current code snippet in cog RAM
count		long	0		' count of longs to copy
adj_06					' re-use this long in DAA
cycles		long	0		' number of cycles for the current opcode
adj_60					' re-use this long in DAA
op_flags	long	0		' opcode flags coming from the snippets_xx_table

low_16bits	long	$0000ffff
add_sub_bit	long	%000001_0000_0000_000000000_000000000

snippets_00_table	long	0
snippets_cb_table	long	0
snippets_dd_table	long	0
snippets_ed_table	long	0
snippets_fd_table	long	0

time		long	80_000_000
t1		long	0
ch		long	0

#ifdef  DO_CCOUNT
_1_000_000	long	1_000_000
#endif

BC		long	0		' BC register pair
DE		long	0		' DE register pair
HL		long	0		' HL register pair
AF		long	z_flag		' A and flags register pair
BC2		long	0		' alternate BC register pair
DE2		long	0		' alternate DE register pair
HL2		long	0		' alternate HL register pair
AF2		long	z_flag		' alternate A and flags register pair
IX		long	$ffff		' IX register pair
IY		long	$ffff		' IY register pair
SP		long	0		' stack pointer
PC		long	0		' program counter
PC0		long	0		' program counter at the start of this #compile
PC1		long	0		' program counter at the start of this instruction
R		long	0		' R register (refresh address)
R2		long	0		' bit #7 of last write to R register
I		long	0		' interrupt vector register

startup
'---------------------------------------------------------------------------------------------------------
' Z80 compiler
'---------------------------------------------------------------------------------------------------------
compile
		mov	dst_pc, #ORIGIN			' start over to compile to ORIGIN++
		mov	max_pc, #$1ef
		mov	PC0, PC				' remember the program counter of this compiled block
#ifdef	DO_CCOUNT
		mov	cc, _1_000_000			' For now give us a million cycles to run
#endif
compile_next
		mov	PC1, PC				' save PC at start of this opcode
		call	#rd_opcode			' get opcode

		mov	tmp, snippets_00_table
compile_op

#ifdef	DEBUG
		mov	hval, PC1
		call	#hexw
		mov	ch, #$3a
		call	#chr
		mov	hval, opcode
		call	#hexb
		call	#spc
#endif
		shl	opcode, #2			' * 4
		add	opcode, tmp			' offset into one of the code snippets tables
		rdlong	ea, opcode			' get the address + op_flags of the code snippet
		mov	op_flags, ea WC			' save a copy in op_flags, set carry for jump_to flag
		and	ea, low_16bits			' mask hub RAM address, or cog RAM address for jump_to
	if_c	jmp	ea				' jump to the prefix handler
		add	ea, reloc
		rdlong	count, ea			' get number of longs to copy
		mov	cycles, count
		and	count, #$ff			' lower byte contains length of the code
		shr	cycles, #CYCLES_SHIFT		' cycles for this opcode

#ifdef	DEBUG
		mov	hval, count
		call	#hexb
		call	#spc
		mov	hval, cycles
		call	#hexb
		call	#cr
		mov	tmp, cnt
		add	tmp, time
		waitcnt	tmp, time
#endif

		' verify that this code snippet still fits in cog RAM
		mov	tmp, dst_pc			' destination
		add	tmp, count			' + count longs
		add	tmp, #$10			' + 16 longs
		cmp	tmp, max_pc WZ, WC		' < last possible cog RAM address?
	if_ae	jmp	#execute			' no, now execute what we compiled

		movd	:dest0, dst_pc
		mov	snippet, dst_pc			' remember the start of this code snippet
:dest0
		mov	0-0, PC1			' store PC of the instruction in the snippet's first long
							' the upper 6 bits are zero, so this is a PASM NOP
		add	dst_pc, #1

#ifdef	DO_RCOUNT
		movd	:dest1, dst_pc			' patch destination PC address 
		add	dst_pc, #1
:dest1		mov	0-0, add_r_1_op			' store a "add R, #1" in the code
#endif

		cmp	count, #0 WZ			' test if count is zero (NOP opcodes)
	if_z	jmp	#compile_next			' yes, copy nothing at all
:copy
		movd	:dest2, dst_pc			' store destination address in rdlong opcode
		add	ea, #4				' next source address (+ 1 long)
		rdlong	tmp, ea
		shr	tmp, #(32-6) WZ, NR		' see if this is a tinted NOP
	if_z	call	#paste_snippet			' yes, paste the corresponding snippet of code
:dest2
	if_nz	mov	0-0, tmp			' store the PASM opcode
	if_nz	add	dst_pc, #1			' next cog RAM address
		djnz	count, #:copy			' copy the code snippet

		shr	op_flags, #OP_FLAGS_SHIFT WZ	' get op_flags to bits 8..0
	if_z	jmp	#compile_next			' no bits set, continue with the next opcode

		test	op_flags, #patch_imm16 WZ
	if_nz	call	#do_patch_imm16

		test	op_flags, #patch_imm8 WZ
	if_nz	call	#do_patch_imm8

		test	op_flags, #patch_jr_disp8 WZ
	if_nz	call	#do_patch_jr_disp8

		test	op_flags, #patch_xy_disp8 WZ
	if_nz	call	#do_patch_xy_disp8

		test	op_flags, #terminate WZ
	if_z	jmp	#compile_next

		' fall through

'*******************************************************
' Prepare execution of the compiled code
'
execute
		' store a "jmp #compile" at the end of the cog RAM at dst_pc
		movd	:dest, dst_pc			' store destination address in the mov opcode
		mov	ea, dst_pc			' save dst_pc in ea
:dest		mov	0-0, jmp_compile_op
		jmp	#ORIGIN				' now execute what we compiled

'*******************************************************
' Paste another snippet of code into the current one
' snippet source address (in cog RAM) is in tmp
'
paste_snippet
		add	tmp, reloc
		rdlong	aux, tmp			' fetch the number of longs to copy
:copy	
		movd	:dest1, dst_pc			' store destination address in rdlong opcode
		add	tmp, #4				' next source address (+ 1 long)
:dest1
		rdlong	0-0, tmp			' read the PASM opcode to cog RAM
		add	dst_pc, #1			' next cog RAM address
		djnz	aux, #:copy			' copy the code snippet
		cmp	tmp, tmp WZ			' make sure the Z flag is set
paste_snippet_ret
		ret

'*******************************************************
' Get opcode snippet for CB xx opcode
'
prefix_cb_xx
		call	#rd_opcode			' get 2nd byte of opcode
		mov	tmp, snippets_cb_table
		jmp	#compile_op

'*******************************************************
' Get opcode snippet for ED xx opcode
'
prefix_ed_xx
		call	#rd_opcode			' get 2nd byte of opcode
		cmp	opcode, #$c0 WC			' opcodes $c0..$ff are invalid
	if_ae	jmp	#invalid_ed_xx
		sub	opcode, #$40 WC			' opcodes $00..$3f are invalid
	if_c	jmp	#invalid_ed_xx
		mov	tmp, snippets_ed_table
		jmp	#compile_op
invalid_ed_xx
		mov	opcode, #$7f			' invalid opcode slot
		jmp	#compile_op

'*******************************************************
' Get opcode snippet for DD xx opcode
'
prefix_dd_xx
		call	#rd_opcode			' get 2nd byte of opcode
		cmp	opcode, #$dd WZ			' repeated prefix?
	if_z	jmp	#prefix_dd_xx
		cmp	opcode, #$fd WZ			' fd prefix?
	if_z	jmp	#prefix_fd_xx
		mov	tmp, snippets_dd_table
		jmp	#compile_op

'*******************************************************
' Get opcode snippet for FD xx opcode
'
prefix_fd_xx
		call	#rd_opcode			' get 2nd byte of opcode
		cmp	opcode, #$fd WZ			' repeated prefix?
	if_z	jmp	#prefix_fd_xx
		cmp	opcode, #$dd WZ			' dd prefix?
	if_z	jmp	#prefix_dd_xx
		mov	tmp, snippets_dd_table
		jmp	#compile_op

#ifdef	DO_CCOUNT
'*******************************************************
' The current timeslice is up.
'
timeslice
		mov	ea, #0-0			' source field is patched by the jmpret to #timeslice
		sub	ea, #2				' subtract for 2 PASM opcodes to point to the
							' original PC in the compiled block
		movs	:fetch_pc, ea			' patch the source address
		nop					' needed because of Prop's prefetch
:fetch_pc
		mov	PC, 0-0				' fetch the original PC from the snippet
#endif
jmp_compile_op
		jmp	#compile			' and jump to the compiler
#ifdef	DO_RCOUNT
add_r_1_op	add	R, #1
#endif

'*******************************************************
' A snippet has modified compiled code. We need to find
' the Z80 PC of that snippet and recompile from there
snippet_pc
		and	tmp, #$1ff			' mask the return address of the modifying snippet
		sub	tmp, #1
:loop
		movs	:padr, tmp
		sub	tmp, #1
:padr
		mov	ea, 0-0				' fetch the PASM opcode
		shr	ea, #23 WZ, NR			' test for a NOP opcode
	if_nz	jmp	#:loop
		mov	PC, ea				' this is the PC where to start recompiling at
		jmp	#compile

'*******************************************************
' Handle a relative jump outside of the compiled space
'
handle_jr
#ifdef	DO_CCOUNT
		sub	cc, #5				' 5 additional cycles
#endif
		mov	PC, aux				' set the new Z80 program counter
		jmp	#compile

'*******************************************************
' Pop a word from the Z80 stack to alu
'
pop_alu
		mov	ea, SP				' pop a word
		add	SP, #2				' off the stack
		and	SP, low_16bits
		call	#rd_memw_alu			' into alu
pop_alu_ret
		ret

'*******************************************************
' Push a word from alu to the Z80 stack
'
push_alu
		sub	SP, #2
		and	SP, low_16bits
		mov	ea, SP
		call	#wr_memw_alu			' from alu
push_alu_ret
		ret


'*******************************************************
' Handle a return from subroutine
'
handle_ret
#ifdef	DO_CCOUNT
		sub	cc, #6				' 6 additional cycles if return is executed
#endif
		call	#pop_alu
		mov	PC, alu				' and set the new Z80 program counter
		jmp	#compile

'*******************************************************
' Handle an absolute jump outside of the compiled space
'
handle_jp
		mov	PC, ea				' set the new Z80 program counter
		jmp	#compile

'*******************************************************
' Handle an absolute call outside of the compiled space
'
handle_call
#ifdef	DO_CCOUNT
		sub	cc, #6				' 6 additional cycles if call is executed
#endif
		mov	alu, PC
		call	#push_alu
		mov	PC, aux				' set the new Z80 program counter
		jmp	#compile

'*******************************************************
' Write a byte from the alu to RAM[DE]
'
wr_memb_de_alu
		mov	ea, DE

'*******************************************************
' Write a byte from the alu to RAM[ea]
'
' If the write modifies the compiled code,
' we need to bail out and recompile from PC
'
wr_memb_alu
		mov	hubptr, ea
		add	hubptr, ram_base
		wrbyte	alu, hubptr
		cmp	ea, PC0	WC, WZ			' writing below PC0 ?
	if_b	jmp	#:ok_to_write
		cmp	ea, PC WC, WZ			' writing PC or beyond, i.e. end of compiled block ?
	if_a	jmp	#:ok_to_write
		mov	tmp, wr_memb_alu_ret		' get the return address in the source field
		jmp	#snippet_pc			' need to recompile from PC
:ok_to_write
wr_memb_alu_ret
wr_memb_de_alu_ret
		ret

'*******************************************************
' Read a byte from RAM[HL] into the alu
'
rd_memb_hl_alu
		mov	ea, HL
'*******************************************************
' Read a byte from RAM[ea] into the alu
'
rd_memb_alu
		mov	hubptr, ea
		add	hubptr, ram_base
		rdbyte	alu, hubptr
rd_memb_alu_ret
rd_memb_hl_alu_ret
		ret

'*******************************************************
' Read a byte from RAM[ea] into the opcode
'
rd_opcode
		mov	hubptr, PC
		add	PC, #1
		and	PC, low_16bits
		add	hubptr, ram_base
		rdbyte	opcode, hubptr
rd_opcode_ret
		ret

'*******************************************************
' Write a word from the alu to RAM[ea]
'
' If the write modifies the compiled code,
' we need to bail out and recompile from PC
'
wr_memw_alu
		mov	hubptr, ea
		add	hubptr, ram_base
		wrbyte	alu, hubptr
		add	hubptr, #1
		ror	alu, #8
		wrbyte	alu, hubptr
		rol	alu, #8
		cmp	ea, PC0	WC, WZ			' writing below PC0 ?
	if_b	jmp	#:ok_to_write
		cmp	ea, PC WC, WZ			' writing PC or beyond, i.e. end of compiled block ?
	if_a	jmp	#:ok_to_write
		mov	tmp, wr_memw_alu_ret		' get the return address in the source field
		jmp	#snippet_pc			' need to recompile from PC
:ok_to_write
wr_memw_alu_ret
		ret

'*******************************************************
' Read a word from RAM[ea] into the alu
'
rd_memw_alu
		mov	hubptr, ea
		add	hubptr, ram_base
		rdbyte	alu, hubptr
		add	hubptr, #1
		ror	alu, #8
		rdbyte	t1, hubptr
		or	alu, t1
		rol	alu, #8
rd_memw_alu_ret
		ret


'*******************************************************
' Write a byte from the alu to an I/O port
'
wr_port_alu
		cmp	ea, #$40 WZ
	if_z	jmp	#wr_debug
		mov	hubptr, ea
		add	hubptr, io_base
		wrbyte	alu, hubptr
wr_port_alu_ret
		ret

wr_debug
		rdlong	t1, dbg
		cmp	t1, #0 WZ
	if_nz	jmp	#wr_debug
		mov	t1, #$100
		or	t1, alu
		wrlong	t1, dbg
		jmp	wr_port_alu_ret

'*******************************************************
' Read a byte from an I/O port into the alu
'
rd_port_alu
		mov	hubptr, ea
		add	hubptr, io_base
		rdbyte	alu, hubptr
rd_port_alu_ret
		ret


'*******************************************************
' Patch an immediate 16 bit (word) load in snippet 
'
do_patch_imm16
#ifdef	DO_RCOUNT
		mov	tmp, #2
#else
		mov	tmp, #1
#endif
		add	tmp, snippet			' add offset to patch
		movd	:padr2, tmp			' set patch address for address in code
		movd	:padr1, max_pc			' set patch address for long at end of cog RAM
		mov	ea, PC
		add	PC, #2
		and	PC, low_16bits
		call	#rd_memw_alu
:padr1		mov	0-0, alu			' store word in long at the end of cog RAM
:padr2		movs	0-0, max_pc			' patch the code with the address of the word
		sub	max_pc, #1			' one less long available
do_patch_imm16_ret
		ret

'*******************************************************
' Patch an immediate 8 bit (byte) load in snippet
'
do_patch_imm8
#ifdef	DO_RCOUNT
		mov	tmp, #3
#else
		mov	tmp, #2
#endif
		add	tmp, snippet			' add offset to patch
		movd	:padr, tmp			' set patch address
		call	#rd_opcode
:padr		movs	0-0, opcode			' patch the code with the byte
do_patch_imm8_ret
		ret

'*******************************************************
' Patch a relative jump address in snippet
'
do_patch_jr_disp8
#ifdef	DO_RCOUNT
		mov	tmp, #3				' 3 longs
#else
		mov	tmp, #2				' 2 longs
#endif
		add	tmp, snippet			' into the snippets code
		mov	:padr0, tmp
		mov	ea, PC
:padr0
		movs	0-0, dst_pc			' patch the code with the next opcode address
		add	PC, #1
		call	#rd_memb_alu			' get the disp8
		shl	alu, #24			' sign extend the byte
		sar	alu, #24			' into a long
		add	alu, PC				' alu = new target address

#ifdef	DO_RCOUNT
		mov	tmp, #4				' 4 longs
#else
		mov	tmp, #3				' 3 longs
#endif
		add	tmp, snippet			' into the snippets code
		mov	:padr1, tmp
		cmp	alu, PC0 WZ, WC
:padr1		movs	0-0, alu			' store new PC in the snippet

	if_b	jmp	#execute			' new PC is below this compiled block
		cmp	alu, PC WZ, WC
	if_a	jmp	#execute			' new PC is beyond the current address
		mov	tmp, dst_pc			' get the end of the compiler output
		sub	tmp, #4
:loop							' and start to search backwards
		movs	:find_addr, tmp
		sub	tmp, #4
:find_addr
		mov	ea, 0-0				' get the code word
		cmp	ea, alu	WZ			' found the expected PC?
	if_z	jmp	#:found				' yes, :found
		cmp	tmp, #ORIGIN WZ, WC		' hit bottom of compiled code?
	if_ae	jmp	#:loop				' not yet
		jmp	#execute			' not found, execute the block
:found
#ifdef	DO_RCOUNT
		mov	tmp, #2				' 2 longs
#else
		mov	tmp, #1				' 1 long
#endif
		add	ea, snippet			' into the snippet's code
		movd	:padr2, ea			' set patch address
		add	tmp, #2				' address after the tinted NOP
:padr2
		movs	0-0, tmp			' patch the code with the found snippet's address
do_patch_jr_disp8_ret
		ret

'*******************************************************
' Patch a IX or IY disp8 into the snippet
'
do_patch_xy_disp8
#ifdef	DO_RCOUNT
		mov	tmp, #2				' 2 longs
#else
		mov	tmp, #1				' 1 longs
#endif
		add	tmp, snippet
		movd	:padr, tmp
		call	#rd_opcode
:padr
		movs	0-0, alu
do_patch_xy_disp8_ret
		ret

'******************************************************************************
'******************************************************************************
'
'	Debug code
'
'******************************************************************************
'******************************************************************************
#ifdef	DEBUG
spc
		mov	ch, #32
		jmp	#chr
cr
		mov	ch, #13
chr
		rdlong	t1, dbg
		cmp	t1, #0 WZ
	if_nz	jmp	#chr
		mov	t1, #$100
		or	t1, ch
		wrlong	t1, dbg
spc_ret
chr_ret
cr_ret
		ret

hcnt		long	0
hval		long	0
hex
		mov	hcnt, #8
hex_loop
		rol	hval, #4
		mov	ch, hval
		and	ch, #15
		cmp	ch, #10 WZ, WC
	if_b	add	ch, #48
	if_ae	add	ch, #55
		call	#chr
		djnz	hcnt, #hex_loop
hex_loop_ret
hex_ret
		ret

hexb
		mov	hcnt, #2
		shl	hval, #24
		call	#hex_loop
hexb_ret
		ret

hexw
		mov	hcnt, #4
		shl	hval, #16
		call	#hex_loop
hexw_ret
		ret
#endif
'******************************************************************************
'
' compiled code goes here
'
ORIGIN					' here is the origin of the compiler output
		res	$1f0 - $
		fit	$1f0

'******************************************************************************
'
' Compute an 8 bit increment and set the flags
'
alu_inc		long	(:end - $ - 1)
		add	alu, #1				' increment alu
		andn	AF, #n_flag			' reset negative flag
		test	alu, #$FF WZ
		muxz	AF, #z_flag
		test	alu, #$80 WZ
		muxz	AF, #s_flag
		cmp	alu, #$80 WZ			' increment 7f -> 80 ?
		muxz	AF, #p_flag			' P flag used to indicate overflow
		test	alu, #$0f WZ			' lower nibble wrapped to 0?
		muxz	AF, #h_flag			' set auxiliary carry
		and	alu, #$ff
:end


'******************************************************************************
'
' Compute an 8 bit decrement and set the flags
'
alu_dec		long	(:end - $ - 1)
		sub	alu, #1				' decrement alu
		or	AF, #n_flag			' set negative flag
		test	alu, #$FF WZ
		muxz	AF, #z_flag
		test	alu, #$80 WZ
		muxz	AF, #s_flag
		cmp	alu, #$7f WZ			' decrement 80 -> 7f ?
		muxz	AF, #p_flag			' P flag used to indicate overflow
		xor	alu, #$0f
		test	alu, #$0f WZ			' lower nibble wrapped to f?
		xor	alu, #$0f
		muxz	AF, #h_flag			' set auxiliary carry
		and	alu, #$ff
:end

'******************************************************************************
'
' Compute an arithmetic or logic alu operation and set the flags
'
alu_add		long	(:end - $ - 1)
		andn	AF, #n_flag			' clear the N flag
		mov	alu, AF				' operate on A
		shr	alu, #8				' A in bits 7..0
		add	alu, tmp			' add the tmp value
		shl	alu, #24 WC, WZ			' to bits 31..24
		muxz	AF, #z_flag			' if zero set Z flag
		muxc	AF, #c_flag			' if bit #8 was set, set C flag
		mov	alu, alu WC			' get sign in prop's carry flag
		muxc	AF, #s_flag			' set S flag
		and	AF, #$ff			' keep flags, clear A
		shr	alu, #16			' result in 15..8
		or	AF, alu
:end

alu_adc		long	(:end - $ - 1)
		andn	AF, #n_flag			' clear the N flag
		mov	alu, AF				' operate on A
		shr	alu, #8				' A in bits 7..0
		test	AF, #c_flag WC			' get Z80 carry into prop's carry
		addx	alu, tmp			' add the tmp value with carry
		shl	alu, #24 WC, WZ			' to bits 31..24
		muxz	AF, #z_flag			' if zero set Z flag
		muxc	AF, #c_flag			' if bit #8 was set, set C flag
		mov	alu, alu WC			' get sign in prop's carry flag
		muxc	AF, #s_flag			' set S flag
		and	AF, #$ff			' keep flags, clear A
		shr	alu, #16			' result in 15..8
		or	AF, alu
:end

alu_sub		long	(:end - $ - 1)
		andn	AF, #n_flag			' clear the N flag
		mov	alu, AF				' operate on A
		shr	alu, #8				' A in bits 7..0
		sub	alu, tmp			' subtract the tmp value
		shl	alu, #24 WC, WZ			' to bits 31..24
		muxz	AF, #z_flag			' if zero set Z flag
		muxc	AF, #c_flag			' if bit #8 was set, set C flag
		mov	alu, alu WC			' get sign in prop's carry flag
		muxc	AF, #s_flag			' set S flag
		and	AF, #$ff			' keep flags, clear A
		shr	alu, #16			' result in 15..8
		or	AF, alu
:end

alu_sbc		long	(:end - $ - 1)
		andn	AF, #n_flag			' clear the N flag
		mov	alu, AF				' operate on A
		shr	alu, #8				' A in bits 7..0
		test	AF, #c_flag WC			' get Z80 carry into prop's carry
		subx	alu, tmp			' subtract the tmp value with carry
		shl	alu, #24 WC, WZ			' to bits 31..24
		muxz	AF, #z_flag			' if zero set Z flag
		muxc	AF, #c_flag			' if bit #8 was set, set C flag
		mov	alu, alu WC			' get sign in prop's carry flag
		muxc	AF, #s_flag			' set S flag
		and	AF, #$ff			' keep flags, clear A
		shr	alu, #16			' result in 15..8
		or	AF, alu
:end

alu_and		long	(:end - $ - 1)
		mov	alu, AF				' operate on A
		and	AF, #(n_flag|c_flag)		' keep N and C flags, clear A
		shr	alu, #8				' A in bits 7..0
		and	alu, tmp WC, WZ			' and with the tmp value
		muxz	AF, #z_flag			' if zero set Z flag
		muxnc	AF, #p_flag			' if even parity set the P flag
		test	alu, #$80 WC			' get sign in prop's carry flag
		muxc	AF, #s_flag			' set S flag
		shl	alu, #8				' result in 15..8
		or	AF, alu
:end

alu_xor		long	(:end - $ - 1)
		mov	alu, AF				' operate on A
		and	AF, #(n_flag|c_flag)		' keep N and C flags, clear A
		shr	alu, #8				' A in bits 7..0
		xor	alu, tmp WC, WZ			' xor with the tmp value
		muxz	AF, #z_flag			' if zero set Z flag
		muxnc	AF, #p_flag			' if even parity set the P flag
		test	alu, #$80 WC			' get sign in prop's carry flag
		muxc	AF, #s_flag			' set S flag
		shl	alu, #8				' result in 15..8
		or	AF, alu
:end

alu_or		long	(:end - $ - 1)
		mov	alu, AF				' operate on A
		and	AF, #(n_flag|c_flag)		' keep N and C flags, clear A
		shr	alu, #8				' A in bits 7..0
		or	alu, tmp WC, WZ			' or with the tmp value
		muxz	AF, #z_flag			' if zero set Z flag
		muxnc	AF, #p_flag			' if even parity set the P flag
		test	alu, #$80 WC			' get sign in prop's carry flag
		muxc	AF, #s_flag			' set S flag
		shl	alu, #8				' result in 15..8
		or	AF, alu
:end

alu_cp		long	(:end - $ - 1)
		mov	alu, AF				' operate on A
		andn	AF, #n_flag			' clear the N flag
		shr	alu, #8				' A in bits 7..0
		sub	alu, tmp			' compare alu with the tmp value
		shl	alu, #24 WC, WZ			' to bits 31..24
		muxz	AF, #z_flag			' if zero set Z flag
		muxc	AF, #c_flag			' if bit #8 was set, set C flag
		mov	alu, alu WC			' get sign in prop's carry flag
		muxc	AF, #s_flag			' set S flag
:end

alu_add16	long	(:end - $ - 1)
		mov	aux, alu
		add	alu, tmp
		shl	alu, #16 WC			' get bit #16 into prop's carry
		muxc	AF, #c_flag			' clear or set carry flag
		shr	alu, #16
		xor	aux, alu
		xor	aux, tmp
		shr	aux, #8				' get MSB to LSB
		test	aux, #h_flag WC
		muxc	AF, #h_flag			' clear or set auxiliary carry flag
:end

'*******************************************************
' 00 No operation
'
z_nop		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
:end

'*******************************************************
' 08 Exchange AF and AF'
'
z_ex_af_af2	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		xor	AF, AF2
		xor	AF2, AF
		xor	AF, AF2
:end

'*******************************************************
' 10 Decrement B and jump if non zero
'
z_djnz		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	ea, #$0		' source is patched with the compiled address
		mov	opcode, #$0	' source is patched with the address of next opcode
		mov	aux, #$0	' source is patched with Z80 PC of the target
		sub	BC, #$100	' decrement B
		shr	BC, #8 WZ, NR	' is B zero now?
	if_z	jmp	opcode		' yes, skip to next opcode (patched with address of :end)
		test	ea, ea WZ	' is ea zero?
	if_nz	jmp	ea		' ea contains the cog RAM address for the rel jump
		jmp	#handle_jr
:end

'*******************************************************
' 18 Jump relative
'
z_jr		long	(:end - $ - 1) | (12 << CYCLES_SHIFT)
		mov	ea, #$0		' source is patched with the compiled address
		mov	opcode, #$0	' source is patched with the address of next opcode (unused)
		mov	aux, #$0	' source is patched with Z80 PC of the target
		test	ea, ea WZ	' is ea zero?
	if_nz	jmp	ea		' nope, ea contains the cog RAM address for the rel jump
		jmp	#handle_jr
:end

'*******************************************************
' 20 Jump relative if Z flag is clear
'
z_jr_nz		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	ea, #$0		' source is patched with the compiled address
		mov	opcode, #$0	' source is patched with the address of next opcode
		mov	aux, #$0	' source is patched with Z80 PC of the target
		test	AF, #z_flag WC	' test if Z flag is set
	if_c	jmp	opcode		' no, Z flag is set: skip to next opcode
		test	ea, ea WZ	' is ea zero?
	if_nz	jmp	ea		' ea contains the cog RAM address for the rel jump
		jmp	#handle_jr
:end

'*******************************************************
' 28 Jump relative if Z flag is set
'
z_jr_z		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	ea, #$0		' source is patched with the compiled address
		mov	opcode, #$0	' source is patched with the address of next opcode
		mov	aux, #$0	' source is patched with Z80 PC of the target
		test	AF, #z_flag WC	' test if Z flag is clear
	if_nc	jmp	opcode		' no, Z flag is set: skip to next opcode
		test	ea, ea WZ	' is ea zero?
	if_nz	jmp	ea		' ea contains the cog RAM address for the rel jump
		jmp	#handle_jr
:end

'*******************************************************
' 30 Jump relative if C flag is clear
'
z_jr_nc		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	ea, #$0		' source is patched with the compiled address
		mov	opcode, #$0	' source is patched with the address of next opcode
		mov	aux, #$0	' source is patched with Z80 PC of the target
		test	AF, #c_flag WC	' test if C flag is clear
	if_c	jmp	opcode		' no, C flag is set: skip to next opcode
		test	ea, ea WZ	' is ea zero?
	if_nz	jmp	ea		' ea contains the cog RAM address for the rel jump
		jmp	#handle_jr
:end

'*******************************************************
' 38 Jump relative if C flag is set
'
z_jr_c		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	ea, #$0		' source is patched with the compiled address
		mov	opcode, #$0	' source is patched with the address of next opcode
		mov	aux, #$0	' source is patched with Z80 PC of the target
		test	AF, #c_flag WC	' test if Z flag is set
	if_nc	jmp	opcode		' no, C flag is clear: skip to next opcode
		test	ea, ea WZ	' is ea zero?
	if_nz	jmp	ea		' ea contains the cog RAM address for the rel jump
		jmp	#handle_jr
:end

'*******************************************************
' 01 Load BC with imm16
'
z_ld_bc_imm16	long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	BC, 0-0		' source is patched with the address of imm16
:end

'*******************************************************
' 11 Load DE with imm16
'
z_ld_de_imm16	long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	DE, 0-0		' source is patched with the address of imm16
:end

'*******************************************************
' 21 Load HL with imm16
'
z_ld_hl_imm16	long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	HL, 0-0		' source is patched with the address of imm16
:end

'*******************************************************
' 31 Load SP with imm16
'
z_ld_sp_imm16	long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	SP, 0-0		' source is patched with the address of imm16
:end

'*******************************************************
' 09 Add BC to HL
'
z_add_hl_bc	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		mov	alu, HL
		mov	tmp, BC
		long	@alu_add16	' paste the add16 code snippet here
		mov	HL, alu
:end

'*******************************************************
' 19 Add DE to HL
'
z_add_hl_de	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		mov	alu, HL
		mov	tmp, DE
		long	@alu_add16	' paste the add16 code snippet here
		mov	HL, alu
:end

'*******************************************************
' 29 Add HL to HL
'
z_add_hl_hl	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		mov	alu, HL
		mov	tmp, HL
		long	@alu_add16	' paste the add16 code snippet here
		mov	HL, alu
:end

'*******************************************************
' 39 Add SP to HL
'
z_add_hl_sp	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		mov	alu, HL
		mov	tmp, SP
		long	@alu_add16	' paste the add16 code snippet here
		mov	HL, alu
:end

'*******************************************************
' 02 Load memory (BC) with accu
'
z_ld_mem_bc_a	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		mov	ea, BC
		mov	alu, AF
		shr	alu, #8
		call	#wr_memb_alu
:end

'*******************************************************
' 0a Load accu from memory (BC)
'
z_ld_a_mem_bc	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		mov	ea, BC
		call	#rd_memb_alu
		shl	alu, #8
		and	AF, #$ff
		or	AF, alu
:end

'*******************************************************
' 12 Load memory (DE) with accu
'
z_ld_mem_de_a	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		mov	alu, AF
		shr	alu, #8
		call	#wr_memb_de_alu
:end

'*******************************************************
' 1a Load accu from memory (DE)
'
z_ld_a_mem_de	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		mov	ea, DE
		call	#rd_memb_alu
		shl	alu, #8
		and	AF, #$ff
		or	AF, alu
:end

'*******************************************************
' 22 Load memory (imm16) with HL
'
z_ld_mem_hl	long	(:end - $ - 1) | (16 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		mov	alu, HL		' store HL
		call	#wr_memw_alu
:end

'*******************************************************
' 2a Load HL from memory (imm16)
'
z_ld_hl_mem	long	(:end - $ - 1) | (16 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		call	#rd_memw_alu
		mov	HL, alu		' to HL
:end

'*******************************************************
' 32 Load memory (imm16) with A
'
z_ld_mem_a	long	(:end - $ - 1) | (13 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		mov	alu, AF		' store AF
		shr	alu, #8		' isolate A
		call	#wr_memb_alu
:end

'*******************************************************
' 3a Load A from memory (imm16)
'
z_ld_a_mem	long	(:end - $ - 1) | (13 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		call	#rd_memb_alu
		and	AF, #$ff	' keep flags
		shl	alu, #8
		or	AF, alu		' paste alu to A
:end

'*******************************************************
' 03 Increment BC
'
z_inc_bc	long	(:end - $ - 1) | (6 << CYCLES_SHIFT)
		add	BC, #1
		and	BC, low_16bits
:end

'*******************************************************
' 0b Decrement BC
'
z_dec_bc	long	(:end - $ - 1) | (6 << CYCLES_SHIFT)
		sub	BC, #1
		and	BC, low_16bits
:end

'*******************************************************
' 13 Increment DE
'
z_inc_de	long	(:end - $ - 1) | (6 << CYCLES_SHIFT)
		add	DE, #1
		and	DE, low_16bits
:end

'*******************************************************
' 1b Decrement DE
'
z_dec_de	long	(:end - $ - 1) | (6 << CYCLES_SHIFT)
		sub	DE, #1
		and	DE, low_16bits
:end

'*******************************************************
' 23 Increment HL
'
z_inc_hl	long	(:end - $ - 1) | (6 << CYCLES_SHIFT)
		add	HL, #1
		and	HL, low_16bits
:end

'*******************************************************
' 2b Decrement HL
'
z_dec_hl	long	(:end - $ - 1) | (6 << CYCLES_SHIFT)
		sub	HL, #1
		and	HL, low_16bits
:end

'*******************************************************
' 33 Increment SP
'
z_inc_sp	long	(:end - $ - 1) | (6 << CYCLES_SHIFT)
		add	SP, #1
		and	SP, low_16bits
:end

'*******************************************************
' 3b Decrement SP
'
z_dec_sp	long	(:end - $ - 1) | (6 << CYCLES_SHIFT)
		sub	SP, #1
		and	SP, low_16bits
:end

'*******************************************************
' 04 Increment B
'
z_inc_b		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	alu, BC
		shr	alu, #8		' isolate B
		and	BC, #$ff	' clear B in BC
		long	@alu_inc
		shl	alu, #8
		or	BC, alu		' paste alu in B
:end

'*******************************************************
' 0c Increment C
'
z_inc_c		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	alu, BC
		and	alu, #$ff	' isolate C
		andn	BC, #$ff	' clear C in BC
		long	@alu_inc
		or	BC, alu		' paste alu in C
:end

'*******************************************************
' 14 Increment D
'
z_inc_d		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	alu, DE
		shr	alu, #8		' isolate D
		and	DE, #$ff	' clear D in DE
		long	@alu_inc
		shl	alu, #8
		or	DE, alu		' paste alu in D
:end


'*******************************************************
' 1c Increment E
'
z_inc_e		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	alu, DE
		and	alu, #$ff	' isolate E
		andn	DE, #$ff	' clear E in DE
		long	@alu_inc
		or	DE, alu		' paste alu in E
:end

'*******************************************************
' 24 Increment H
'
z_inc_h		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	alu, HL
		shr	alu, #8		' isolate H
		and	HL, #$ff	' clear H in HL
		long	@alu_inc
		shl	alu, #8
		or	HL, alu		' paste alu in H
:end


'*******************************************************
' 2c Increment L
'
z_inc_l		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	alu, HL
		and	alu, #$ff	' isolate L
		andn	HL, #$ff	' clear L in HL
		long	@alu_inc
		or	HL, alu		' paste alu in L
:end

'*******************************************************
' 34 Increment memory (HL)
'
z_inc_m		long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		long	@alu_inc
		call	#wr_memb_alu
:end

'*******************************************************
' 3c Increment A
'
z_inc_a		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	alu, AF
		shr	alu, #8		' isolate A
		and	AF, #$ff	' clear A in AF
		long	@alu_inc
		shl	alu, #8
		or	AF, alu		' paste alu in A
:end

'*******************************************************
' 05 Decrement B
'
z_dec_b		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	alu, BC
		shr	alu, #8		' isolate B
		and	BC, #$ff	' clear B in BC
		long	@alu_dec
		shl	alu, #8
		or	BC, alu		' paste alu in B
:end

'*******************************************************
' 0d Decrement C
'
z_dec_c		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	alu, BC
		and	alu, #$ff	' isolate C
		andn	BC, #$ff	' clear C in BC
		long	@alu_dec
		or	BC, alu		' paste alu into C
:end

'*******************************************************
' 15 Decrement D
'
z_dec_d		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	alu, DE
		shr	alu, #8		' isolate D
		and	DE, #$ff	' clear D in DE
		long	@alu_dec
		shl	alu, #8
		or	DE, alu		' paste alu in D
:end

'*******************************************************
' 1d Decrement E
'
z_dec_e		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	alu, DE
		and	alu, #$ff	' isolate E
		andn	DE, #$ff	' clear E in DE
		long	@alu_dec
		or	DE, alu		' paste alu in E
:end

'*******************************************************
' 25 Decrement H
'
z_dec_h		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	alu, HL
		shr	alu, #8		' isolate H
		and	HL, #$ff	' clear H in HL
		long	@alu_dec
		shl	alu, #8
		or	HL, alu		' paste alu in H
:end

'*******************************************************
' 2d Decrement L
'
z_dec_l		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	alu, HL
		and	alu, #$ff	' isolate L
		andn	HL, #$ff	' clear L in HL
		long	@alu_dec
		or	HL, alu		' paste alu in L
:end

'*******************************************************
' 35 Decrement memory (HL)
'
z_dec_m		long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		long	@alu_dec
		call	#wr_memb_alu
:end

'*******************************************************
' 3d Decrement A
'
z_dec_a		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	alu, AF
		shr	alu, #8		' isolate A
		and	AF, #$ff	' clear A in AF
		long	@alu_dec
		shl	alu, #8
		or	AF, alu		' paste alu in A
:end

'*******************************************************
' 06 Load B with imm8
'
z_ld_b_imm8	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		ror	BC, #8
		movs	BC, #0		' source is patched with the imm8
		rol	BC, #8
:end

'*******************************************************
' 0e Load c with imm8
'
z_ld_c_imm8	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		andn	BC, #$ff	' clear C
		or	BC, #0		' source is patched with the imm8
:end

'*******************************************************
' 16 Load D with imm8
'
z_ld_d_imm8	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		ror	DE, #8
		movs	DE, #0		' source is patched with the imm8
		rol	DE, #8
:end

'*******************************************************
' 1e Load E with imm8
'
z_ld_e_imm8	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		andn	DE, #$ff	' clear E
		or	DE, #0		' source is patched with the imm8
:end

'*******************************************************
' 26 Load H with imm8
'
z_ld_h_imm8	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		ror	HL, #8
		movs	HL, #0		' source is patched with the imm8
		rol	HL, #8
:end

'*******************************************************
' 2e Load L with imm8
'
z_ld_l_imm8	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		andn	HL, #$ff	' clear L
		or	HL, #0		' source is patched with the imm8
:end

'*******************************************************
' 36 Load memory (HL) with imm8
'
z_ld_m_imm8	long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	ea, HL
		mov	alu, #0		' source is patched with the imm8
		call	#wr_memb_alu
:end

'*******************************************************
' 3e Load A with imm8
'
z_ld_a_imm8	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		ror	AF, #8
		movs	AF, #0		' source is patched with the imm8
		rol	AF, #8
:end

'*******************************************************
' 07 Rotate left circular accu
'
z_rlca		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	alu, AF
		shr	alu, #8 - 1
		andn	alu, #1		' clear low bit
		test	alu, #$100 WC	' check carry
		muxc	AF, #c_flag	' set carry flag
		muxc	alu, #$01	' set low bit
		shl	alu, #8
		and	AF, #$ff	' keep flags
		or	AF, alu		' paste alu into AF
:end


'*******************************************************
' 0f Rotate right circular accu
'
z_rrca		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	alu, AF
		shr	alu, #8 + 1
		test	alu, #$80 WC	' check carry
		muxc	AF, #c_flag	' set carry flag
		muxc	alu, #$80	' set high bit
		shl	alu, #8
		and	AF, #$ff	' keep flags
		or	AF, alu		' paste alu into AF
:end

'*******************************************************
' 17 Rotate left accu FIXME:
'
z_rla		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	alu, AF
		shr	alu, #8 - 1
		test	alu, #$100 WC	' test former bit #7
		and	alu, #$ff
		muxc	AF, #c_flag	' set carry flag
		muxc	alu, #$01	' set bit #0
		shl	alu, #8		' result to 15..8
		and	AF, #$ff	' keep flags
		or	AF, alu		' paste result into AF
:end

'*******************************************************
' 1f Rotate right accu FIXME:
'
z_rra		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	alu, AF
		shr	alu, #8 + 1
		test	AF, #$100 WC	' check original bit #0
		muxc	AF, #c_flag	' set carry flag
		muxc	alu, #$80	' set bit #7
		shl	alu, #8		' result to 15..8
		and	AF, #$ff	' keep flags
		or	AF, alu		' paste alu into AF
:end

'*******************************************************
' 27 Decimal adjust accumulator
'
z_daa		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	alu, AF
		and	AF, #$ff		' clear A
		shr	alu, #8
		mov	tmp, alu
		mov	aux, alu		' save for auxiliary flag test
		and	tmp, #15		' isolate low nibble of accumulator
		mov	adj_06, #$06
		mov	adj_60, #$60
		test	AF, #n_flag WC		' test negative flag
		negc	adj_06, adj_06
		negc	adj_60, adj_60
		test	AF, #h_flag WZ
	if_nz	cmp	tmp, #$09+1 WC		' aux isn't set; is nibble > $9 ?
	if_ae	add	alu, adj_06		' aux was set or nibble is > $9 => add $06 or -$06
		test	AF, #c_flag WZ
	if_nz	cmp	alu, #$99+1 WC		' carry isn't set; is data > $99 ?
	if_ae	add	alu, adj_60		' carry was set or data is > $99 => add $60 or -$60
		and	AF, #(n_flag|c_flag)	' preserve old CF and NF
		cmp	alu, #$99+1 WC
	if_nc	or	AF, #c_flag		' set carry if result > $99
		xor	aux, alu
		and	aux, #h_flag		' if bit #4 of result changed
		or	AF, aux			' set the auxiliary carry flag
		and	alu, #$FF WZ, WC
		muxz	AF, #z_flag		' set Z80 zero flag from props zero
		muxnc	AF, #p_flag		' set Z80 parity flag from props carry
		test	AF, #$80 WZ
		muxnz	AF, #s_flag
		shl	alu, #8
		or	AF, alu			' paste the result into A
:end

'*******************************************************
' 2f Complement A
'
z_cpl		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		ror	AF, #8
		xor	AF, #$ff
		mov	aux, AF
		rol	AF, #8
		andn	AF, #(x_flag|y_flag)	' clear the X and Y flags
		or	AF, #(h_flag|n_flag)	' set the H and N flags
		and	aux, #(x_flag|y_flag)
		or	AF, aux			' set the X and Y flags
:end

'*******************************************************
' 37 Set carry flag  FIXME: other flags
'
z_scf		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	aux, AF
		shr	aux, #8
		or	AF, #c_flag		' set the C flag
		and	aux, #(x_flag|y_flag)
		or	AF, aux			' or in the X and Y flags (sic!)
:end

'*******************************************************
' 3f Complement carry flag  FIXME: other flags
'
z_ccf		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	aux, AF
		shr	aux, #8
		xor	AF, #c_flag		' toggle the C flag
		and	aux, #(x_flag|y_flag)
		or	AF, aux			' or in the X and Y flags (sic!)
:end


'*******************************************************
' 40 Load B with B (NOP)
'
z_ld_b_b	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
:end

'*******************************************************
' 41 Load B with C
'
z_ld_b_c	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		and	BC, #$ff	' keep C, clear B
		and	tmp, #$ff	' mask C
		shl	tmp, #8
		or	BC, tmp
:end

'*******************************************************
' 42 Load B with D
'
z_ld_b_d	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		and	BC, #$ff	' keep C, clear B
		andn	tmp, #$ff	' mask D
		or	BC, tmp
:end

'*******************************************************
' 43 Load B with E
'
z_ld_b_e	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		and	BC, #$ff	' keep C, clear B
		and	tmp, #$ff	' mask E
		shl	tmp, #8
		or	BC, tmp
:end

'*******************************************************
' 44 Load B with H
'
z_ld_b_h	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		and	BC, #$ff	' keep C, clear B
		andn	tmp, #$ff	' mask H
		or	BC, tmp
:end

'*******************************************************
' 45 Load B with L
'
z_ld_b_l	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		and	BC, #$ff	' keep C, clear B
		and	tmp, #$ff	' mask L
		shl	tmp, #8
		or	BC, tmp
:end

'*******************************************************
' 46 Load B with memory (HL)
'
z_ld_b_m	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		and	BC, #$ff	' keep C, clear B
		shl	alu, #8
		or	BC, alu
:end

'*******************************************************
' 47 Load B with A
'
z_ld_b_a	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, AF
		and	BC, #$ff	' keep C, clear B
		andn	tmp, #$ff	' mask A
		or	BC, tmp
:end

'*******************************************************
' 48 Load C with B
'
z_ld_c_b	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		shr	tmp, #8		' mask B
		andn	BC, #$ff	' keep B, clear C
		or	BC, tmp
:end

'*******************************************************
' 49 Load C with C (NOP)
'
z_ld_c_c	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
:end

'*******************************************************
' 4a Load C with D
'
z_ld_c_d	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		andn	BC, #$ff	' keep B, clear C
		shr	tmp, #8		' mask D
		or	BC, tmp
:end

'*******************************************************
' 4b Load C with E
'
z_ld_c_e	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		andn	BC, #$ff	' keep B, clear C
		and	tmp, #$ff	' mask E
		or	BC, tmp
:end

'*******************************************************
' 4c Load B with H
'
z_ld_c_h	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		andn	BC, #$ff	' keep B, clear C
		shr	tmp, #8		' mask H
		or	BC, tmp
:end

'*******************************************************
' 4d Load C with L
'
z_ld_c_l	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		andn	BC, #$ff	' keep B, clear C
		and	tmp, #$ff	' mask L
		or	BC, tmp
:end

'*******************************************************
' 4e Load C with memory (HL)
'
z_ld_c_m	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		andn	BC, #$ff	' keep B, clear C
		or	BC, alu
:end

'*******************************************************
' 4f Load C with A
'
z_ld_c_a	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, AF
		andn	BC, #$ff	' keep B, clear C
		shr	tmp, #8
		or	BC, tmp
:end

'*******************************************************
' 50 Load D with B
'
z_ld_d_b	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		and	DE, #$ff	' keep E, clear D
		andn	tmp, #$ff	' mask B
		or	DE, tmp
:end

'*******************************************************
' 51 Load D with C
'
z_ld_d_c	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		and	DE, #$ff	' keep E, clear D
		and	tmp, #$ff	' mask C
		shl	tmp, #8
		or	DE, tmp
:end

'*******************************************************
' 52 Load D with D (NOP)
'
z_ld_d_d	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
:end

'*******************************************************
' 53 Load D with E
'
z_ld_d_e	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		and	DE, #$ff	' keep E, clear D
		and	tmp, #$ff	' mask E
		shl	tmp, #8
		or	DE, tmp
:end

'*******************************************************
' 54 Load D with H
'
z_ld_d_h	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		and	DE, #$ff	' keep E, clear D
		andn	tmp, #$ff	' mask H
		or	DE, tmp
:end

'*******************************************************
' 55 Load D with L
'
z_ld_d_l	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		and	DE, #$ff	' keep E, clear D
		and	tmp, #$ff	' mask L
		shl	tmp, #8
		or	DE, tmp
:end

'*******************************************************
' 56 Load D with memory (HL)
'
z_ld_d_m	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		and	DE, #$ff	' keep E, clear D
		shl	alu, #8
		or	DE, alu
:end

'*******************************************************
' 57 Load D with A
'
z_ld_d_a	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, AF
		and	DE, #$ff	' keep E, clear D
		andn	tmp, #$ff	' mask A
		or	DE, tmp
:end

'*******************************************************
' 58 Load E with B
'
z_ld_e_b	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		andn	DE, #$ff	' keep D, clear E
		shr	tmp, #8		' mask B
		or	DE, tmp
:end

'*******************************************************
' 59 Load E with C
'
z_ld_e_c	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		andn	DE, #$ff	' keep D, clear E
		and	tmp, #$ff	' mask C
		or	DE, tmp
:end

'*******************************************************
' 5a Load E with D
'
z_ld_e_d	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		andn	DE, #$ff	' keep D, clear E
		shr	tmp, #8		' mask D
		or	DE, tmp
:end

'*******************************************************
' 5b Load E with E (NOP)
'
z_ld_e_e	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
:end

'*******************************************************
' 5c Load E with H
'
z_ld_e_h	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		andn	DE, #$ff	' keep D, clear E
		shr	tmp, #8		' mask H
		or	DE, tmp
:end

'*******************************************************
' 5d Load E with L
'
z_ld_e_l	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		andn	DE, #$ff	' keep D, clear E
		and	tmp, #$ff	' mask L
		or	DE, tmp
:end

'*******************************************************
' 5e Load E with memory (HL)
'
z_ld_e_m	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		andn	DE, #$ff	' keep D, clear E
		or	DE, alu
:end

'*******************************************************
' 5f Load E with A
'
z_ld_e_a	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, AF
		andn	DE, #$ff	' keep D, clear E
		shr	tmp, #8		' mask A
		or	DE, tmp
:end

'*******************************************************
' 60 Load H with B
'
z_ld_h_b	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		and	HL, #$ff	' keep L, clear H
		andn	tmp, #$ff	' mask B
		or	HL, tmp
:end

'*******************************************************
' 61 Load H with C
'
z_ld_h_c	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		and	HL, #$ff	' keep L, clear H
		and	tmp, #$ff	' mask C
		shl	tmp, #8
		or	HL, tmp
:end

'*******************************************************
' 62 Load H with D
'
z_ld_h_d	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		and	HL, #$ff	' keep L, clear H
		andn	tmp, #$ff	' mask D
		or	HL, tmp
:end

'*******************************************************
' 63 Load H with E
'
z_ld_h_e	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		and	HL, #$ff	' keep L, clear H
		and	tmp, #$ff	' mask E
		shl	tmp, #8
		or	HL, tmp
:end

'*******************************************************
' 64 Load H with H (NOP)
'
z_ld_h_h	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
:end

'*******************************************************
' 65 Load H with L
'
z_ld_h_l	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		and	HL, #$ff	' keep L, clear H
		and	tmp, #$ff	' mask L
		shl	tmp, #8
		or	HL, tmp
:end

'*******************************************************
' 66 Load H with memory (HL)
'
z_ld_h_m	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		and	HL, #$ff	' keep L, clear H
		shl	alu, #8
		or	HL, alu
:end

'*******************************************************
' 67 Load H with A
'
z_ld_h_a	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, AF
		and	HL, #$ff	' keep L, clear H
		andn	tmp, #$ff	' mask A
		or	HL, tmp
:end

'*******************************************************
' 68 Load L with B
'
z_ld_l_b	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		andn	HL, #$ff	' keep H, clear L
		shr	tmp, #8		' mask B
		or	HL, tmp
:end

'*******************************************************
' 69 Load L with C
'
z_ld_l_c	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		andn	HL, #$ff	' keep H, clear L
		and	tmp, #$ff	' mask C
		or	HL, tmp
:end

'*******************************************************
' 6a Load L with D
'
z_ld_l_d	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		andn	HL, #$ff	' keep H, clear L
		shr	tmp, #8		' mask D
		or	HL, tmp
:end

'*******************************************************
' 6b Load L with E
'
z_ld_l_e	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		andn	HL, #$ff	' keep H, clear L
		and	tmp, #$ff	' mask E
		or	HL, tmp
:end

'*******************************************************
' 6c Load L with H
'
z_ld_l_h	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		andn	HL, #$ff	' keep H, clear L
		shr	tmp, #8		' mask H
		or	HL, tmp
:end

'*******************************************************
' 6d Load L with L (NOP)
'
z_ld_l_l	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
:end

'*******************************************************
' 6e Load L with memory (HL)
'
z_ld_l_m	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		andn	HL, #$ff	' keep H, clear L
		or	HL, alu
:end

'*******************************************************
' 6f Load L with A
'
z_ld_l_a	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, AF
		andn	HL, #$ff	' keep H, clear L
		shr	tmp, #8		' mask A
		or	HL, tmp
:end

'*******************************************************
' 70 Load memory (HL) with B
'
z_ld_m_b	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		mov	ea, HL
		mov	alu, BC
		shr	alu, #8		' mask B
		call	#wr_memb_alu
:end

'*******************************************************
' 71 Load memory (HL) with C
'
z_ld_m_c	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		mov	ea, HL
		mov	alu, BC
		and	alu, #$ff	' mask C
		call	#wr_memb_alu
:end

'*******************************************************
' 72 Load memory (HL) with D
'
z_ld_m_d	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		mov	ea, HL
		mov	alu, DE
		shr	alu, #8		' mask D
		call	#wr_memb_alu
:end

'*******************************************************
' 73 Load memory (HL) with E
'
z_ld_m_e	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		mov	ea, HL
		mov	alu, DE
		and	alu, #$ff	' mask E
		call	#wr_memb_alu
:end

'*******************************************************
' 74 Load memory (HL) with H
'
z_ld_m_h	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		mov	ea, HL
		mov	alu, HL
		shr	alu, #8		' mask H
		call	#wr_memb_alu
:end

'*******************************************************
' 75 Load memory (HL) with L
'
z_ld_m_l	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		mov	ea, HL
		mov	alu, HL
		and	alu, #$ff	' mask L
		call	#wr_memb_alu
:end

'*******************************************************
' 76 HALT
'
z_halt		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	halt, #1	' set the halt flag
:end

'*******************************************************
' 77 Load memory (HL) with A
'
z_ld_m_a	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		mov	ea, HL
		mov	alu, AF
		shr	alu, #8		' mask A
		call	#wr_memb_alu
:end

'*******************************************************
' 78 Load A with B
'
z_ld_a_b	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		and	AF, #$ff	' keep F, clear A
		andn	tmp, #$ff	' mask B
		or	AF, tmp
:end

'*******************************************************
' 79 Load A with C
'
z_ld_a_c	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		and	AF, #$ff	' keep F, clear A
		and	tmp, #$ff	' mask C
		shl	tmp, #8
		or	AF, tmp
:end

'*******************************************************
' 7a Load A with D
'
z_ld_a_d	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		and	AF, #$ff	' keep F, clear A
		andn	tmp, #$ff	' mask D
		or	AF, tmp
:end

'*******************************************************
' 7b Load A with E
'
z_ld_a_e	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		and	AF, #$ff	' keep F, clear A
		and	tmp, #$ff	' mask E
		shl	tmp, #8
		or	AF, tmp
:end

'*******************************************************
' 7c Load A with H
'
z_ld_a_h	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		and	AF, #$ff	' keep F, clear A
		andn	tmp, #$ff	' mask H
		or	AF, tmp
:end

'*******************************************************
' 7d Load A with L
'
z_ld_a_l	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		and	AF, #$ff	' keep F, clear A
		and	tmp, #$ff	' mask L
		shl	tmp, #8
		or	AF, tmp
:end

'*******************************************************
' 7e Load A with memory (HL)
'
z_ld_a_m	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		and	AF, #$ff	' keep F, clear A
		shl	alu, #8
		or	AF, alu
:end

'*******************************************************
' 7f Load A with A (NOP)
'
z_ld_a_a	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
:end

'*******************************************************
' 80 Add B to A
'
z_add_a_b	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		shr	tmp, #8
		long	@alu_add	' paste the alu_add code snippet here
:end

'*******************************************************
' 81 Add C to A
'
z_add_a_c	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		and	tmp, #$ff
		long	@alu_add	' paste the alu_add code snippet here
:end

'*******************************************************
' 82 Add D to A
'
z_add_a_d	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		shr	tmp, #8
		long	@alu_add	' paste the alu_add code snippet here
:end

'*******************************************************
' 83 Add E to A
'
z_add_a_e	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		and	tmp, #$ff
		long	@alu_add	' paste the alu_add code snippet here
:end

'*******************************************************
' 84 Add H to A
'
z_add_a_h	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		shr	tmp, #8
		long	@alu_add	' paste the alu_add code snippet here
:end

'*******************************************************
' 85 Add L to A
'
z_add_a_l	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		and	tmp, #$ff
		long	@alu_add		' paste the alu_add code snippet here
:end

'*******************************************************
' 86 Add memory (HL) to A
'
z_add_a_m	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		long	@alu_add		' paste the alu_add code snippet here
:end

'*******************************************************
' 87 Add A to A
'
z_add_a_a	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, AF
		shr	tmp, #8
		long	@alu_add		' paste the alu_add code snippet here
:end

'*******************************************************
' 88 Add with carry B to A
'
z_adc_a_b	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		shr	tmp, #8
		long	@alu_adc		' paste the alu_adc code snippet here
:end

'*******************************************************
' 89 Add with carry C to A
'
z_adc_a_c	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		and	tmp, #$ff
		long	@alu_adc		' paste the alu_adc code snippet here
:end

'*******************************************************
' 8a Add with carry D to A
'
z_adc_a_d	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		shr	tmp, #8
		long	@alu_adc		' paste the alu_adc code snippet here
:end

'*******************************************************
' 8b Add with carry E to A
'
z_adc_a_e	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		and	tmp, #$ff
		long	@alu_adc		' paste the alu_adc code snippet here
:end

'*******************************************************
' 8c Add with carry H to A
'
z_adc_a_h	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		shr	tmp, #8
		long	@alu_adc		' paste the alu_adc code snippet here
:end

'*******************************************************
' 8d Add with carry L to A
'
z_adc_a_l	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		and	tmp, #$ff
		long	@alu_adc		' paste the alu_adc code snippet here
:end

'*******************************************************
' 8e Add with carry memory (HL) to A
'
z_adc_a_m	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		long	@alu_adc		' paste the alu_adc code snippet here
:end

'*******************************************************
' 8f Add with carry A to A
'
z_adc_a_a	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, AF
		shr	tmp, #8
		long	@alu_adc		' paste the alu_adc code snippet here
:end

'*******************************************************
' 90 Subtract B from A 
'
z_sub_a_b	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		shr	tmp, #8
		long	@alu_sub		' paste the alu_sub code snippet here
:end

'*******************************************************
' 91 Subtract C from A 
'
z_sub_a_c	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		and	tmp, #$ff
		long	@alu_sub		' paste the alu_sub code snippet here
:end

'*******************************************************
' 92 Subtract D from A 
'
z_sub_a_d	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		shr	tmp, #8
		long	@alu_sub		' paste the alu_sub code snippet here
:end

'*******************************************************
' 93 Subtract E from A 
'
z_sub_a_e	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		and	tmp, #$ff
		long	@alu_sub		' paste the alu_sub code snippet here
:end

'*******************************************************
' 94 Subtract H from A 
'
z_sub_a_h	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		shr	tmp, #8
		long	@alu_sub		' paste the alu_sub code snippet here
:end

'*******************************************************
' 95 Subtract L from A 
'
z_sub_a_l	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		and	tmp, #$ff
		long	@alu_sub		' paste the alu_sub code snippet here
:end

'*******************************************************
' 96 Subtract memory (HL) from A 
'
z_sub_a_m	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		long	@alu_sub		' paste the alu_sub code snippet here
:end

'*******************************************************
' 97 Subtract A from A 
'
z_sub_a_a	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, AF
		shr	tmp, #8
		long	@alu_sub		' paste the alu_sub code snippet here
:end

'*******************************************************
' 98 Subtract with carry B from A 
'
z_sbc_a_b	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		shr	tmp, #8
		long	@alu_sbc		' paste the alu_sbc code snippet here
:end

'*******************************************************
' 99 Subtract with carry C from A 
'
z_sbc_a_c	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		and	tmp, #$ff
		long	@alu_sbc		' paste the alu_sbc code snippet here
:end

'*******************************************************
' 9a Subtract with carry D from A 
'
z_sbc_a_d	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		shr	tmp, #8
		long	@alu_sbc		' paste the alu_sbc code snippet here
:end

'*******************************************************
' 9b Subtract with carry E from A 
'
z_sbc_a_e	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		and	tmp, #$ff
		long	@alu_sbc		' paste the alu_sbc code snippet here
:end

'*******************************************************
' 9c Subtract with carry H from A 
'
z_sbc_a_h	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		shr	tmp, #8
		long	@alu_sbc		' paste the alu_sbc code snippet here
:end

'*******************************************************
' 9d Subtract with carry L from A 
'
z_sbc_a_l	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		and	tmp, #$ff
		long	@alu_sbc		' paste the alu_sbc code snippet here
:end

'*******************************************************
' 9e Subtract with carry memory (HL) from A 
'
z_sbc_a_m	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		long	@alu_sbc		' paste the alu_sbc code snippet here
:end

'*******************************************************
' 9f Subtract with carry A from A 
'
z_sbc_a_a	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, AF
		shr	tmp, #8
		long	@alu_sbc		' paste the alu_sbc code snippet here
:end

'*******************************************************
' a0 And A with B
'
z_and_a_b	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		shr	tmp, #8
		long	@alu_and		' paste the alu_and code snippet here
:end

'*******************************************************
' a1 And A with C
'
z_and_a_c	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		and	tmp, #$ff
		long	@alu_and		' paste the alu_and code snippet here
:end

'*******************************************************
' a2 And A with D
'
z_and_a_d	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		shr	tmp, #8
		long	@alu_and		' paste the alu_and code snippet here
:end

'*******************************************************
' a3 And A with E
'
z_and_a_e	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		and	tmp, #$ff
		long	@alu_and		' paste the alu_and code snippet here
:end

'*******************************************************
' a4 And A with H
'
z_and_a_h	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		shr	tmp, #8
		long	@alu_and		' paste the alu_and code snippet here
:end

'*******************************************************
' a5 And A with L
'
z_and_a_l	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		and	tmp, #$ff
		long	@alu_and		' paste the alu_and code snippet here
:end

'*******************************************************
' a6 And A with memory (HL)
'
z_and_a_m	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		long	@alu_and		' paste the alu_and code snippet here
:end

'*******************************************************
' a7 And A with A
'
z_and_a_a	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, AF
		shr	tmp, #8
		long	@alu_and		' paste the alu_and code snippet here
:end

'*******************************************************
' a8 Exclusive or A with B
'
z_xor_a_b	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		shr	tmp, #8
		long	@alu_xor		' paste the alu_xor code snippet here
:end

'*******************************************************
' a9 Exclusive or A with C
'
z_xor_a_c	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		and	tmp, #$ff
		long	@alu_xor		' paste the alu_xor code snippet here
:end

'*******************************************************
' aa Exclusive or A with D
'
z_xor_a_d	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		shr	tmp, #8
		long	@alu_xor		' paste the alu_xor code snippet here
:end

'*******************************************************
' ab Exclusive or A with E
'
z_xor_a_e	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		and	tmp, #$ff
		long	@alu_xor		' paste the alu_xor code snippet here
:end

'*******************************************************
' ac Exclusive or A with H
'
z_xor_a_h	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		shr	tmp, #8
		long	@alu_xor		' paste the alu_xor code snippet here
:end

'*******************************************************
' ad Exclusive or A with L
'
z_xor_a_l	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		and	tmp, #$ff
		long	@alu_xor		' paste the alu_xor code snippet here
:end

'*******************************************************
' ae Exclusive or A with memory (HL)
'
z_xor_a_m	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		long	@alu_xor		' paste the alu_xor code snippet here
:end

'*******************************************************
' af Exclusive or A with A
'
z_xor_a_a	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, AF
		shr	tmp, #8
		long	@alu_xor		' paste the alu_xor code snippet here
:end

'*******************************************************
' b0 Or A with B
'
z_or_a_b	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		shr	tmp, #8
		long	@alu_or		' paste the alu_or code snippet here
:end

'*******************************************************
' b1 Or A with C
'
z_or_a_c	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		and	tmp, #$ff
		long	@alu_or		' paste the alu_or code snippet here
:end

'*******************************************************
' b2 Or A with D
'
z_or_a_d	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		shr	tmp, #8
		long	@alu_or		' paste the alu_or code snippet here
:end

'*******************************************************
' b3 Or A with E
'
z_or_a_e	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		and	tmp, #$ff
		long	@alu_or		' paste the alu_or code snippet here
:end

'*******************************************************
' b4 Or A with H
'
z_or_a_h	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		shr	tmp, #8
		long	@alu_or		' paste the alu_or code snippet here
:end

'*******************************************************
' b5 Or A with L
'
z_or_a_l	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		and	tmp, #$ff
		long	@alu_or		' paste the alu_or code snippet here
:end

'*******************************************************
' b6 Or A with memory (HL)
'
z_or_a_m	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		long	@alu_or		' paste the alu_or code snippet here
:end

'*******************************************************
' b7 Or A with A
'
z_or_a_a	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, AF
		shr	tmp, #8
		long	@alu_or		' paste the alu_or code snippet here
:end

'*******************************************************
' b8 Compare A with B
'
z_cp_a_b	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		shr	tmp, #8
		long	@alu_cp		' paste the alu_cp code snippet here
:end

'*******************************************************
' b9 Compare A with C
'
z_cp_a_c	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, BC
		and	tmp, #$ff
		long	@alu_cp		' paste the alu_cp code snippet here
:end

'*******************************************************
' ba Compare A with D
'
z_cp_a_d	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		shr	tmp, #8
		long	@alu_cp		' paste the alu_cp code snippet here
:end

'*******************************************************
' bb Compare A with E
'
z_cp_a_e	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, DE
		and	tmp, #$ff
		long	@alu_cp		' paste the alu_cp code snippet here
:end

'*******************************************************
' bc Compare A with H
'
z_cp_a_h	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		shr	tmp, #8
		long	@alu_cp		' paste the alu_cp code snippet here
:end

'*******************************************************
' bd Compare A with L
'
z_cp_a_l	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, HL
		and	tmp, #$ff
		long	@alu_cp		' paste the alu_cp code snippet here
:end

'*******************************************************
' be Compare A with memory (HL)
'
z_cp_a_m	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		long	@alu_cp		' paste the alu_cp code snippet here
:end

'*******************************************************
' bf Compare A with A
'
z_cp_a_a	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	tmp, AF
		shr	tmp, #8
		long	@alu_cp		' paste the alu_cp code snippet here
:end

'*******************************************************
' c0 Return on Z flag clear (non zero)
'
z_ret_nz	long	(:end - $ - 1) | (5 << CYCLES_SHIFT)
		test	AF, #z_flag WC	' get Z flag into carry
	if_nc	jmp	#handle_ret	' do the return if Z is clear
:end

'*******************************************************
' c8 Return on Z flag set (zero)
'
z_ret_z		long	(:end - $ - 1) | (5 << CYCLES_SHIFT)
		test	AF, #z_flag WC	' get Z flag into carry
	if_c	jmp	#handle_ret	' do the return if Z is set
:end

'*******************************************************
' d0 Return on C flag clear (non carry)
'
z_ret_nc	long	(:end - $ - 1) | (5 << CYCLES_SHIFT)
		test	AF, #c_flag WC	' get C flag into carry
	if_nc	jmp	#handle_ret	' do the return if C is clear
:end

'*******************************************************
' d8 Return on C flag set (carry)
'
z_ret_c		long	(:end - $ - 1) | (5 << CYCLES_SHIFT)
		test	AF, #c_flag WC	' get C flag into carry
	if_c	jmp	#handle_ret	' do the return if C is set
:end

'*******************************************************
' e0 Return on P flag clear (parity odd)
'
z_ret_po	long	(:end - $ - 1) | (5 << CYCLES_SHIFT)
		test	AF, #p_flag WC	' get P flag into carry
	if_nc	jmp	#handle_ret	' do the return if P is clear
:end

'*******************************************************
' e8 Return on P flag set (parity even)
'
z_ret_pe	long	(:end - $ - 1) | (5 << CYCLES_SHIFT)
		test	AF, #p_flag WC	' get P flag into carry
	if_c	jmp	#handle_ret	' do the return if P is set
:end

'*******************************************************
' f0 Return on S flag clear (plus)
'
z_ret_p		long	(:end - $ - 1) | (5 << CYCLES_SHIFT)
		test	AF, #s_flag WC	' get S flag into carry
	if_nc	jmp	#handle_ret	' do the return if S is clear
:end

'*******************************************************
' e8 Return on S flag set (minus)
'
z_ret_m		long	(:end - $ - 1) | (5 << CYCLES_SHIFT)
		test	AF, #s_flag WC	' get S flag into carry
	if_c	jmp	#handle_ret	' do the return if S is set
:end

'*******************************************************
' c1 Pop BC off the stack
'
z_pop_bc	long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		call	#pop_alu
		mov	BC, alu
:end

'*******************************************************
' d1 Pop DE off the stack
'
z_pop_de	long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		call	#pop_alu
		mov	DE, alu
:end

'*******************************************************
' e1 Pop HL off the stack
'
z_pop_hl	long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		call	#pop_alu
		mov	HL, alu
:end

'*******************************************************
' f1 Pop AF off the stack
'
z_pop_af	long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		call	#pop_alu
		mov	AF, alu
:end

'*******************************************************
' c9 Return unconditionally
'
z_ret		long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		jmp	#handle_ret	' do the return, i.e. recompile from new PC
:end

'*******************************************************
' d9 Exchange register sets
'
z_exx		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		xor	BC, BC2
		xor	BC2, BC
		xor	BC, BC2
		xor	DE, DE2
		xor	DE2, DE
		xor	DE, DE2
		xor	HL, HL2
		xor	HL2, HL
		xor	HL, HL2
:end

'*******************************************************
' e9 Jump to address in HL
'
z_jp_hl		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		mov	PC, HL
		jmp	#compile
:end

'*******************************************************
' f9 Load SP with HL
'
z_ld_sp_hl	long	(:end - $ - 1) | (6 << CYCLES_SHIFT)
		mov	SP, HL
:end

'*******************************************************
' c2 Jump on Z flag clear (non zero)
'
z_jp_nz		long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		test	AF, #z_flag WC	' get Z flag into carry
	if_nc	jmp	#handle_jp	' do the jump if Z is clear
:end

'*******************************************************
' ca Jump on Z flag set (zero)
'
z_jp_z		long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		test	AF, #z_flag WC	' get Z flag into carry
	if_c	jmp	#handle_jp	' do the jump if Z is set
:end

'*******************************************************
' d2 Jump on C flag clear (non carry)
'
z_jp_nc		long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		test	AF, #c_flag WC	' get C flag into carry
	if_nc	jmp	#handle_jp	' do the jump if C is clear
:end

'*******************************************************
' da Jump on C flag set (carry)
'
z_jp_c		long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		test	AF, #c_flag WC	' get C flag into carry
	if_c	jmp	#handle_jp	' do the jump if C is set
:end

'*******************************************************
' e2 Jump on P flag clear (parity odd)
'
z_jp_po		long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		test	AF, #p_flag WC	' get P flag into carry
	if_nc	jmp	#handle_jp	' do the jump if P is clear
:end

'*******************************************************
' ea Jump on P flag set (parity even)
'
z_jp_pe		long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		test	AF, #p_flag WC	' get P flag into carry
	if_c	jmp	#handle_jp	' do the jump if P is set
:end

'*******************************************************
' f2 Jump on S flag clear (plus)
'
z_jp_p		long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		test	AF, #s_flag WC	' get S flag into carry
	if_nc	jmp	#handle_jp	' do the jump if S is clear
:end

'*******************************************************
' ca Jump on S flag set (minus)
'
z_jp_m		long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		test	AF, #s_flag WC	' get S flag into carry
	if_c	jmp	#handle_jp	' do the jump if S is set
:end

'*******************************************************
' c3 Jump unconditionally
'
z_jp		long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		jmp	#handle_jp	' do the jump
:end

'*******************************************************
' d3 Output A to port imm8 FIXME: flags
'
z_out_port_a	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		mov	ea, #$0
		mov	ea, #$0		' source is patched with the imm8
		mov	alu, AF
		shr	alu, #8
		call	#wr_port_alu
:end

'*******************************************************
' db Input port imm8 to A FIXME: flags
'
z_in_a_port	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		mov	ea, #$0
		mov	ea, #$0		' source is patched with the imm8
		call	#rd_port_alu
		shl	alu, #8
		and	AF, #$ff
		or	AF, alu
:end

'*******************************************************
' e3 Exchange stack contents with HL
'
z_ex_sp_hl	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		call	#pop_alu
		xor	HL, alu
		xor	alu, HL
		xor	HL, alu
		call	#push_alu
:end

'*******************************************************
' eb Exchange DE with HL
'
z_ex_de_hl	long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		xor	HL, DE
		xor	DE, HL
		xor	HL, HL
:end

'*******************************************************
' f3 Disable interrupts
'
z_di		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		' nothing to do - yet
:end

'*******************************************************
' fb Enable interrupts
'
z_ei		long	(:end - $ - 1) | (4 << CYCLES_SHIFT)
		' nothing to do - yet
:end

'*******************************************************
' c4 Call on Z flag clear (non zero)
'
z_call_nz	long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	aux, 0-0	' source is patched with the address of imm16
		test	AF, #z_flag WC	' get Z flag into carry
	if_nc	jmp	#handle_call	' do the call if Z is clear
:end

'*******************************************************
' cc Call on Z flag set (zero)
'
z_call_z	long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	aux, 0-0	' source is patched with the address of imm16
		test	AF, #z_flag WC	' get Z flag into carry
	if_c	jmp	#handle_call	' do the call if Z is set
:end

'*******************************************************
' d4 Call on C flag clear (non carry)
'
z_call_nc	long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	aux, 0-0	' source is patched with the address of imm16
		test	AF, #c_flag WC	' get C flag into carry
	if_nc	jmp	#handle_call	' do the call if C is clear
:end

'*******************************************************
' dc Call on C flag set (carry)
'
z_call_c	long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	aux, 0-0	' source is patched with the address of imm16
		test	AF, #c_flag WC	' get C flag into carry
	if_c	jmp	#handle_call	' do the call if C is set
:end

'*******************************************************
' e4 Call on P flag clear (odd parity)
'
z_call_po	long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	aux, 0-0	' source is patched with the address of imm16
		test	AF, #p_flag WC	' get P flag into carry
	if_nc	jmp	#handle_call	' do the call if P is clear
:end

'*******************************************************
' cc Call on P flag set (even parity)
'
z_call_pe	long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	aux, 0-0	' source is patched with the address of imm16
		test	AF, #p_flag WC	' get P flag into carry
	if_c	jmp	#handle_call	' do the call if P is set
:end

'*******************************************************
' f4 Call on S flag clear (plus)
'
z_call_p	long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	aux, 0-0	' source is patched with the address of imm16
		test	AF, #s_flag WC	' get S flag into carry
	if_nc	jmp	#handle_call	' do the call if S is clear
:end

'*******************************************************
' fc Call on S flag set (minus)
'
z_call_m	long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	aux, 0-0	' source is patched with the address of imm16
		test	AF, #s_flag WC	' get S flag into carry
	if_c	jmp	#handle_call	' do the call if S is set
:end

'*******************************************************
' c5 Push BC to the stack
'
z_push_bc	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		mov	alu, BC
		call	#push_alu
:end

'*******************************************************
' d5 Push DE to the stack
'
z_push_de	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		mov	alu, DE
		call	#push_alu
:end

'*******************************************************
' e5 Push HL to the stack
'
z_push_hl	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		mov	alu, HL
		call	#push_alu
:end

'*******************************************************
' f5 Push AF to the stack
'
z_push_af	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		mov	alu, AF
		call	#push_alu
:end

'*******************************************************
' cd Call unconditional
'
z_call		long	(:end - $ - 1) | (10 << CYCLES_SHIFT)
		mov	aux, 0-0	' source is patched with the address of imm16
		jmp	#handle_call	' do the call
:end

'*******************************************************
' c6 Add immediate 8bit to A
'
z_add_a_imm8	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		mov	tmp, #0
		movs	tmp, #0		' source is patched with the imm8
		long	@alu_add
:end

'*******************************************************
' ce Add with carry immediate 8bit to A
'
z_adc_a_imm8	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		mov	tmp, #0
		movs	tmp, #0		' source is patched with the imm8
		long	@alu_adc
:end

'*******************************************************
' d6 Subtract immediate 8bit from A
'
z_sub_imm8	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		mov	tmp, #0
		movs	tmp, #0		' source is patched with the imm8
		long	@alu_sub
:end

'*******************************************************
' de Subtract with carry immediate 8bit from A
'
z_sbc_a_imm8	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		mov	tmp, #0
		movs	tmp, #0		' source is patched with the imm8
		long	@alu_sbc
:end

'*******************************************************
' e6 And A with immediate 8bit
'
z_and_imm8	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		mov	tmp, #0
		movs	tmp, #0		' source is patched with the imm8
		long	@alu_and
:end

'*******************************************************
' ee Exclusive or A with immediate 8bit
'
z_xor_imm8	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		mov	tmp, #0
		movs	tmp, #0		' source is patched with the imm8
		long	@alu_xor
:end

'*******************************************************
' f6 Or A with immediate 8bit
'
z_or_imm8	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		mov	tmp, #0
		movs	tmp, #0		' source is patched with the imm8
		long	@alu_or
:end

'*******************************************************
' fe Compare A with immediate 8bit
'
z_cp_imm8	long	(:end - $ - 1) | (7 << CYCLES_SHIFT)
		mov	tmp, #0
		movs	tmp, #0		' source is patched with the imm8
		long	@alu_cp
:end

'*******************************************************
' c7 Restart at vector $00
'
z_rst_00	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		mov	aux, #$00
		jmp	#handle_call
:end

'*******************************************************
' cf Restart at vector $08
'
z_rst_08	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		mov	aux, #$08
		jmp	#handle_call
:end

'*******************************************************
' d7 Restart at vector $10
'
z_rst_10	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		mov	aux, #$10
		jmp	#handle_call
:end

'*******************************************************
' df Restart at vector $18
'
z_rst_18	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		mov	aux, #$18
		jmp	#handle_call
:end

'*******************************************************
' e7 Restart at vector $20
'
z_rst_20	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		mov	aux, #$20
		jmp	#handle_call
:end

'*******************************************************
' ef Restart at vector $28
'
z_rst_28	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		mov	aux, #$28
		jmp	#handle_call
:end

'*******************************************************
' f7 Restart at vector $30
'
z_rst_30	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		mov	aux, #$30
		jmp	#handle_call
:end

'*******************************************************
' ff Restart at vector $38
'
z_rst_38	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		mov	aux, #$38
		jmp	#handle_call
:end

snippets_00
	{00}	long	@z_nop
	{01}	long	@z_ld_bc_imm16	| (patch_imm16 << OP_FLAGS_SHIFT)
	{02}	long	@z_ld_mem_bc_a
	{03}	long	@z_inc_bc
	{04}	long	@z_inc_b
	{05}	long	@z_dec_b
	{06}	long	@z_ld_b_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{07}	long	@z_rlca
	{08}	long	@z_ex_af_af2
	{09}	long	@z_add_hl_bc
	{0a}	long	@z_ld_a_mem_bc
	{0b}	long	@z_dec_bc
	{0c}	long	@z_inc_c
	{0d}	long	@z_dec_c
	{0e}	long	@z_ld_c_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{0f}	long	@z_rrca
	{10}	long	@z_djnz		| (patch_jr_disp8 << OP_FLAGS_SHIFT)
	{11}	long	@z_ld_de_imm16	| (patch_imm16 << OP_FLAGS_SHIFT)
	{12}	long	@z_ld_mem_de_a
	{13}	long	@z_inc_de
	{14}	long	@z_inc_d
	{15}	long	@z_dec_d
	{16}	long	@z_ld_d_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{17}	long	@z_rla
	{18}	long	@z_jr		| (patch_jr_disp8 << OP_FLAGS_SHIFT)
	{19}	long	@z_add_hl_de
	{1a}	long	@z_ld_a_mem_de
	{1b}	long	@z_dec_de
	{1c}	long	@z_inc_e
	{1d}	long	@z_dec_e
	{1e}	long	@z_ld_e_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{1f}	long	@z_rra
	{20}	long	@z_jr_nz	| (patch_jr_disp8 << OP_FLAGS_SHIFT)
	{21}	long	@z_ld_hl_imm16	| (patch_imm16 << OP_FLAGS_SHIFT)
	{22}	long	@z_ld_mem_hl	| (patch_imm16 << OP_FLAGS_SHIFT)
	{23}	long	@z_inc_hl
	{24}	long	@z_inc_h
	{25}	long	@z_dec_h
	{26}	long	@z_ld_h_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{27}	long	@z_daa
	{28}	long	@z_jr_z		| (patch_jr_disp8 << OP_FLAGS_SHIFT)
	{29}	long	@z_add_hl_hl
	{2a}	long	@z_ld_hl_mem	| (patch_imm16 << OP_FLAGS_SHIFT)
	{2b}	long	@z_dec_hl
	{2c}	long	@z_inc_l
	{2d}	long	@z_dec_l
	{2e}	long	@z_ld_l_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{2f}	long	@z_cpl
	{30}	long	@z_jr_nc	| (patch_jr_disp8 << OP_FLAGS_SHIFT)
	{31}	long	@z_ld_sp_imm16	| (patch_imm16 << OP_FLAGS_SHIFT)
	{32}	long	@z_ld_mem_a	| (patch_imm16 << OP_FLAGS_SHIFT)
	{33}	long	@z_inc_sp
	{34}	long	@z_inc_m
	{35}	long	@z_dec_m
	{36}	long	@z_ld_m_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{37}	long	@z_scf
	{38}	long	@z_jr_c		| (patch_jr_disp8 << OP_FLAGS_SHIFT)
	{39}	long	@z_add_hl_sp
	{3a}	long	@z_ld_a_mem	| (patch_imm16 << OP_FLAGS_SHIFT)
	{3b}	long	@z_dec_sp
	{3c}	long	@z_inc_a
	{3d}	long	@z_dec_a
	{3e}	long	@z_ld_a_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{3f}	long	@z_ccf
	{40}	long	@z_ld_b_b
	{41}	long	@z_ld_b_c
	{42}	long	@z_ld_b_d
	{43}	long	@z_ld_b_e
	{44}	long	@z_ld_b_h
	{45}	long	@z_ld_b_l
	{46}	long	@z_ld_b_m
	{47}	long	@z_ld_b_a
	{48}	long	@z_ld_c_b
	{49}	long	@z_ld_c_c
	{4a}	long	@z_ld_c_d
	{4b}	long	@z_ld_c_e
	{4c}	long	@z_ld_c_h
	{4d}	long	@z_ld_c_l
	{4e}	long	@z_ld_c_m
	{4f}	long	@z_ld_c_a
	{50}	long	@z_ld_d_b
	{51}	long	@z_ld_d_c
	{52}	long	@z_ld_d_d
	{53}	long	@z_ld_d_e
	{54}	long	@z_ld_d_h
	{55}	long	@z_ld_d_l
	{56}	long	@z_ld_d_m
	{57}	long	@z_ld_d_a
	{58}	long	@z_ld_e_b
	{59}	long	@z_ld_e_c
	{5a}	long	@z_ld_e_d
	{5b}	long	@z_ld_e_e
	{5c}	long	@z_ld_e_h
	{5d}	long	@z_ld_e_l
	{5e}	long	@z_ld_e_m
	{5f}	long	@z_ld_e_a
	{60}	long	@z_ld_h_b
	{61}	long	@z_ld_h_c
	{62}	long	@z_ld_h_d
	{63}	long	@z_ld_h_e
	{64}	long	@z_ld_h_h
	{65}	long	@z_ld_h_l
	{66}	long	@z_ld_h_m
	{67}	long	@z_ld_h_a
	{68}	long	@z_ld_l_b
	{69}	long	@z_ld_l_c
	{6a}	long	@z_ld_l_d
	{6b}	long	@z_ld_l_e
	{6c}	long	@z_ld_l_h
	{6d}	long	@z_ld_l_l
	{6e}	long	@z_ld_l_m
	{6f}	long	@z_ld_l_a
	{70}	long	@z_ld_m_b
	{71}	long	@z_ld_m_c
	{72}	long	@z_ld_m_d
	{73}	long	@z_ld_m_e
	{74}	long	@z_ld_m_h
	{75}	long	@z_ld_m_l
	{76}	long	@z_halt		| (terminate << OP_FLAGS_SHIFT)
	{77}	long	@z_ld_m_a
	{78}	long	@z_ld_a_b
	{79}	long	@z_ld_a_c
	{7a}	long	@z_ld_a_d
	{7b}	long	@z_ld_a_e
	{7c}	long	@z_ld_a_h
	{7d}	long	@z_ld_a_l
	{7e}	long	@z_ld_a_m
	{7f}	long	@z_ld_a_a
	{80}	long	@z_add_a_b
	{81}	long	@z_add_a_c
	{82}	long	@z_add_a_d
	{83}	long	@z_add_a_e
	{84}	long	@z_add_a_h
	{85}	long	@z_add_a_l
	{86}	long	@z_add_a_m
	{87}	long	@z_add_a_a
	{88}	long	@z_adc_a_b
	{89}	long	@z_adc_a_c
	{8a}	long	@z_adc_a_d
	{8b}	long	@z_adc_a_e
	{8c}	long	@z_adc_a_h
	{8d}	long	@z_adc_a_l
	{8e}	long	@z_adc_a_m
	{8f}	long	@z_adc_a_a
	{90}	long	@z_sub_a_b
	{91}	long	@z_sub_a_c
	{92}	long	@z_sub_a_d
	{93}	long	@z_sub_a_e
	{94}	long	@z_sub_a_h
	{95}	long	@z_sub_a_l
	{96}	long	@z_sub_a_m
	{97}	long	@z_sub_a_a
	{98}	long	@z_sbc_a_b
	{99}	long	@z_sbc_a_c
	{9a}	long	@z_sbc_a_d
	{9b}	long	@z_sbc_a_e
	{9c}	long	@z_sbc_a_h
	{9d}	long	@z_sbc_a_l
	{9e}	long	@z_sbc_a_m
	{9f}	long	@z_sbc_a_a
	{a0}	long	@z_and_a_b
	{a1}	long	@z_and_a_c
	{a2}	long	@z_and_a_d
	{a3}	long	@z_and_a_e
	{a4}	long	@z_and_a_h
	{a5}	long	@z_and_a_l
	{a6}	long	@z_and_a_m
	{a7}	long	@z_and_a_a
	{a8}	long	@z_xor_a_b
	{a9}	long	@z_xor_a_c
	{aa}	long	@z_xor_a_d
	{ab}	long	@z_xor_a_e
	{ac}	long	@z_xor_a_h
	{ad}	long	@z_xor_a_l
	{ae}	long	@z_xor_a_m
	{af}	long	@z_xor_a_a
	{b0}	long	@z_or_a_b
	{b1}	long	@z_or_a_c
	{b2}	long	@z_or_a_d
	{b3}	long	@z_or_a_e
	{b4}	long	@z_or_a_h
	{b5}	long	@z_or_a_l
	{b6}	long	@z_or_a_m
	{b7}	long	@z_or_a_a
	{b8}	long	@z_cp_a_b
	{b9}	long	@z_cp_a_c
	{ba}	long	@z_cp_a_d
	{bb}	long	@z_cp_a_e
	{bc}	long	@z_cp_a_h
	{bd}	long	@z_cp_a_l
	{be}	long	@z_cp_a_m
	{bf}	long	@z_cp_a_a
	{c0}	long	@z_ret_nz
	{c1}	long	@z_pop_bc
	{c2}	long	@z_jp_nz	| (patch_imm16 << OP_FLAGS_SHIFT)
	{c3}	long	@z_jp		| ((patch_imm16 | terminate) << OP_FLAGS_SHIFT)
	{c4}	long	@z_call_nz	| (patch_imm16 << OP_FLAGS_SHIFT)
	{c5}	long	@z_push_bc
	{c6}	long	@z_add_a_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{c7}	long	@z_rst_00	| (terminate << OP_FLAGS_SHIFT)
	{c8}	long	@z_ret_z
	{c9}	long	@z_ret		| (terminate << OP_FLAGS_SHIFT)
	{ca}	long	@z_jp_z		| (patch_imm16 << OP_FLAGS_SHIFT)
	{cb}	long	@prefix_cb_xx	| (jump_to << OP_FLAGS_SHIFT)
	{cc}	long	@z_call_z	| (patch_imm16 << OP_FLAGS_SHIFT)
	{cd}	long	@z_call		| ((patch_imm16 | terminate) << OP_FLAGS_SHIFT)
	{ce}	long	@z_adc_a_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{c7}	long	@z_rst_08	| (terminate << OP_FLAGS_SHIFT)
	{d0}	long	@z_ret_nc
	{d1}	long	@z_pop_de
	{d2}	long	@z_jp_nc	| (patch_imm16 << OP_FLAGS_SHIFT)
	{d3}	long	@z_out_port_a	| (patch_imm8 << OP_FLAGS_SHIFT)
	{d4}	long	@z_call_nc	| (patch_imm16 << OP_FLAGS_SHIFT)
	{d5}	long	@z_push_de
	{d6}	long	@z_sub_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{d7}	long	@z_rst_10	| (terminate << OP_FLAGS_SHIFT)
	{d8}	long	@z_ret_c
	{d9}	long	@z_exx
	{da}	long	@z_jp_c		| (patch_imm16 << OP_FLAGS_SHIFT)
	{db}	long	@z_in_a_port	| (patch_imm8 << OP_FLAGS_SHIFT)
	{dc}	long	@z_call_c	| (patch_imm16 << OP_FLAGS_SHIFT)
	{dd}	long	@prefix_dd_xx	| (jump_to << OP_FLAGS_SHIFT)
	{de}	long	@z_sbc_a_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{df}	long	@z_rst_18	| (terminate << OP_FLAGS_SHIFT)
	{e0}	long	@z_ret_po
	{e1}	long	@z_pop_hl
	{e2}	long	@z_jp_po	| (patch_imm16 << OP_FLAGS_SHIFT)
	{e3}	long	@z_ex_sp_hl
	{e4}	long	@z_call_po	| (patch_imm16 << OP_FLAGS_SHIFT)
	{e5}	long	@z_push_hl
	{e6}	long	@z_and_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{e7}	long	@z_rst_20	| (terminate << OP_FLAGS_SHIFT)
	{e8}	long	@z_ret_pe
	{e9}	long	@z_jp_hl	| (terminate << OP_FLAGS_SHIFT)
	{ea}	long	@z_jp_pe	| (patch_imm16 << OP_FLAGS_SHIFT)
	{eb}	long	@z_ex_de_hl
	{ec}	long	@z_call_pe	| (patch_imm16 << OP_FLAGS_SHIFT)
	{ed}	long	@prefix_ed_xx	| (jump_to << OP_FLAGS_SHIFT)
	{ee}	long	@z_xor_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{ef}	long	@z_rst_28	| (terminate << OP_FLAGS_SHIFT)
	{f0}	long	@z_ret_p
	{f1}	long	@z_pop_af
	{f2}	long	@z_jp_p		| (patch_imm16 << OP_FLAGS_SHIFT)
	{f3}	long	@z_di
	{f4}	long	@z_call_p	| (patch_imm16 << OP_FLAGS_SHIFT)
	{f5}	long	@z_push_af
	{f6}	long	@z_or_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{f7}	long	@z_rst_30	| (terminate << OP_FLAGS_SHIFT)
	{f8}	long	@z_ret_m
	{f9}	long	@z_ld_sp_hl
	{fa}	long	@z_jp_m		| (patch_imm16 << OP_FLAGS_SHIFT)
	{fb}	long	@z_ei
	{fc}	long	@z_call_m	| (patch_imm16 << OP_FLAGS_SHIFT)
	{fd}	long	@prefix_fd_xx	| (jump_to << OP_FLAGS_SHIFT)
	{fe}	long	@z_cp_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{ff}	long	@z_rst_38	| (terminate << OP_FLAGS_SHIFT)

'******************************************************************************
'******************************************************************************
'
'	CB xx opcodes
'
'******************************************************************************
'******************************************************************************

'******************************************************************************
'
' Compute rotate left circular and the flags
'
alu_rlc		long	(:end - $ - 1)
		shl	alu, #25 WC		' get bit #7 into carry
		shr	alu, #25		' bits 7..1 -> 6..0
		rcl	alu, #1			' get carry in bit #0
		muxc	AF, #c_flag		' set C flag
		andn	AF, #(h_flag|n_flag)	' clear H and N flags
:end

'******************************************************************************
'
' Compute rotate right circular and the flags
'
alu_rrc		long	(:end - $ - 1)
		shr	alu, #1 WC
		muxc	alu, #$80		' set bit #7 from carry
		muxc	AF, #c_flag		' set C flag
		andn	AF, #(h_flag|n_flag)	' clear H and N flags
:end

'******************************************************************************
'
' Compute rotate left through carry and the flags
'
alu_rl		long	(:end - $ - 1)
		test	AF, #c_flag WC		' get current C flag in carry
		shl	alu, #1			' alu in bits 8..1
		muxc	alu, #1			' set bit #0 from carry
		test	alu, #$100 WC		' test new carry
		muxc	AF, #c_flag		' set C flag
		andn	AF, #(h_flag|n_flag)	' clear H and N flags
		and	alu, #$ff		' result in 7..0
:end

'******************************************************************************
'
' Compute rotate right through carry and the flags
'
alu_rr		long	(:end - $ - 1)
		test	AF, #c_flag WC		' get current C flag in carry
		ror	alu, #1			' alu in bits 31 and 6..0
		muxc	alu, #$80		' set bit #7 from carry
		mov	alu, alu WC		' get bit #31 in carry
		muxc	AF, #c_flag		' set C flag
		andn	AF, #(h_flag|n_flag)	' clear H and N flags
		and	alu, #$ff		' result in 7..0
:end

'******************************************************************************
'
' Compute shift left arithmetic and the flags
'
alu_sla		long	(:end - $ - 1)
		shl	alu, #25 WC		' alu in bits 31..25
		muxc	AF, #c_flag		' set C flag
		andn	AF, #(h_flag|n_flag)	' clear H and N flags
		shr	alu, #24		' result in 7..0
:end

'******************************************************************************
'
' Compute shift right arithmetic and the flags
'
alu_sra		long	(:end - $ - 1)
		shr	alu, #1 WC		' alu in bits 6..0
		muxc	AF, #c_flag		' set C flag
		test	alu, #$40 WC		' keep sign of alu
		muxc	alu, #$80		' in bit #7
		andn	AF, #(h_flag|n_flag)	' clear H and N flags
:end

'******************************************************************************
'
' Compute shift left inverted and the flags
'
alu_sli		long	(:end - $ - 1)
		shl	alu, #25 WC		' alu in bits 31..25
		muxc	AF, #c_flag		' set C flag
		andn	AF, #(h_flag|n_flag)	' clear H and N flags
		shr	alu, #24		' result in 7..0
		or	alu, #1			' set bit #0
:end

'******************************************************************************
'
' Compute shift right logical and the flags
'
alu_srl		long	(:end - $ - 1)
		shr	alu, #1 WC		' alu in bits 6..0
		muxc	AF, #c_flag		' set C flag
		andn	AF, #(h_flag|n_flag)	' clear H and N flags
:end

'******************************************************************************
'
' Compute bit test flags
'
alu_bit		long	(:end - $ - 1)
		and	alu, tmp WZ	' mask bit
		muxnz	AF, #z_flag	' set Z flag from ~bit
		test	alu, #$80 WC	' test sign bit
		muxc	AF, #s_flag	' set S flag depending on carry
		andn	AF, #n_flag	' clear N flag
		or	AF, #h_flag	' set H flag
:end

'******************************************************************************
'
' Transfer B to alu and clear B
'
b_to_alu	long	(:end - $ - 1)
		mov	alu, BC
		and	BC, #$ff		' clear B, keep C
		shr	alu, #8
:end

'******************************************************************************
'
' Transfer C to alu and clear C
'
c_to_alu	long	(:end - $ - 1)
		mov	alu, BC
		andn	BC, #$ff		' clear C, keep B
		and	alu, #$ff
:end

'*******************************************************
' cb 00 Rotate left circular B
'
z_rlc_b		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		long	@b_to_alu
		long	@alu_rlc
		shl	alu, #8
		or	BC, alu			' paste alu in B
:end

'*******************************************************
' cb 01 Rotate left circular C
'
z_rlc_c		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		long	@c_to_alu
		long	@alu_rlc
		or	BC, alu			' paste alu in C
:end

'*******************************************************
' cb 02 Rotate left circular D
'
z_rlc_d		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		and	DE, #$ff		' clear D, keep E
		shr	alu, #8
		long	@alu_rlc
		shl	alu, #8
		or	DE, alu			' paste alu in D
:end

'*******************************************************
' cb 03 Rotate left circular E
'
z_rlc_e		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		andn	DE, #$ff		' clear E, keep D
		and	alu, #$ff
		long	@alu_rlc
		or	DE, alu			' paste alu in E
:end

'*******************************************************
' cb 04 Rotate left circular H
'
z_rlc_h		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		and	HL, #$ff		' clear H, keep L
		shr	alu, #8
		long	@alu_rlc
		shl	alu, #8
		or	HL, alu			' paste alu in H
:end

'*******************************************************
' cb 05 Rotate left circular L
'
z_rlc_l		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		andn	HL, #$ff		' clear L, keep H
		and	alu, #$ff
		long	@alu_rlc
		or	HL, alu			' paste alu in L
:end

'*******************************************************
' cb 06 Rotate left circular memory (HL)
'
z_rlc_m		long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		long	@alu_rlc
		call	#wr_memb_alu
:end

'*******************************************************
' cb 07 Rotate left circular A
'
z_rlc_a		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, AF
		and	AF, #$ff		' clear A, keep F
		shr	alu, #8
		long	@alu_rlc
		shl	alu, #8
		or	AF, alu			' paste alu in A
:end

'*******************************************************
' cb 08 Rotate right circular B
'
z_rrc_b		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		long	@b_to_alu
		long	@alu_rrc
		shl	alu, #8
		or	BC, alu			' paste alu in B
:end

'*******************************************************
' cb 09 Rotate right circular C
'
z_rrc_c		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		long	@c_to_alu
		long	@alu_rrc
		or	BC, alu			' paste alu in C
:end

'*******************************************************
' cb 0a Rotate right circular D
'
z_rrc_d		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		and	DE, #$ff		' clear D, keep E
		shr	alu, #8
		long	@alu_rrc
		shl	alu, #8
		or	DE, alu			' paste alu in D
:end

'*******************************************************
' cb 0b Rotate right circular E
'
z_rrc_e		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		andn	DE, #$ff		' clear E, keep D
		and	alu, #$ff
		long	@alu_rrc
		or	DE, alu			' paste alu in E
:end

'*******************************************************
' cb 0c Rotate right circular H
'
z_rrc_h		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		and	HL, #$ff		' clear H, keep L
		shr	alu, #8
		long	@alu_rrc
		shl	alu, #8
		or	HL, alu			' paste alu in H
:end

'*******************************************************
' cb 0d Rotate right circular L
'
z_rrc_l		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		andn	HL, #$ff		' clear L, keep H
		and	alu, #$ff
		long	@alu_rrc
		or	HL, alu			' paste alu in L
:end

'*******************************************************
' cb 0e Rotate right circular memory (HL)
'
z_rrc_m		long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		long	@alu_rrc
		call	#wr_memb_alu
:end

'*******************************************************
' cb 0f Rotate right circular A
'
z_rrc_a		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, AF
		and	AF, #$ff		' clear A, keep F
		shr	alu, #8
		long	@alu_rrc
		shl	alu, #8
		or	AF, alu			' paste alu in A
:end

'*******************************************************
' cb 10 Rotate left through carry B
'
z_rl_b		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		long	@b_to_alu
		long	@alu_rl
		shl	alu, #8
		or	BC, alu			' paste alu in B
:end

'*******************************************************
' cb 11 Rotate left through carry C
'
z_rl_c		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		long	@c_to_alu
		long	@alu_rl
		or	BC, alu			' paste alu in C
:end

'*******************************************************
' cb 12 Rotate left through carry D
'
z_rl_d		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		and	DE, #$ff		' clear D, keep E
		shr	alu, #8
		long	@alu_rl
		shl	alu, #8
		or	DE, alu			' paste alu in D
:end

'*******************************************************
' cb 13 Rotate left through carry E
'
z_rl_e		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		andn	DE, #$ff		' clear E, keep D
		and	alu, #$ff
		long	@alu_rl
		or	DE, alu			' paste alu in E
:end

'*******************************************************
' cb 14 Rotate left through carry H
'
z_rl_h		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		and	HL, #$ff		' clear H, keep L
		shr	alu, #8
		long	@alu_rl
		shl	alu, #8
		or	HL, alu			' paste alu in H
:end

'*******************************************************
' cb 15 Rotate left through carry L
'
z_rl_l		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		andn	HL, #$ff		' clear L, keep H
		and	alu, #$ff
		long	@alu_rl
		or	HL, alu			' paste alu in L
:end

'*******************************************************
' cb 16 Rotate left through carry memory (HL)
'
z_rl_m		long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		long	@alu_rl
		call	#wr_memb_alu
:end

'*******************************************************
' cb 17 Rotate left through carry A
'
z_rl_a		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, AF
		and	AF, #$ff		' clear A, keep F
		shr	alu, #8
		long	@alu_rl
		shl	alu, #8
		or	AF, alu			' paste alu in A
:end

'*******************************************************
' cb 18 Rotate right through carry B
'
z_rr_b		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		long	@b_to_alu
		long	@alu_rr
		shl	alu, #8
		or	BC, alu			' paste alu in B
:end

'*******************************************************
' cb 19 Rotate right through carry C
'
z_rr_c		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		long	@c_to_alu
		long	@alu_rr
		or	BC, alu			' paste alu in C
:end

'*******************************************************
' cb 1a Rotate right through carry D
'
z_rr_d		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		and	DE, #$ff		' clear D, keep E
		shr	alu, #8
		long	@alu_rr
		shl	alu, #8
		or	DE, alu			' paste alu in D
:end

'*******************************************************
' cb 1b Rotate right through carry E
'
z_rr_e		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		andn	DE, #$ff		' clear E, keep D
		and	alu, #$ff
		long	@alu_rr
		or	DE, alu			' paste alu in E
:end

'*******************************************************
' cb 1c Rotate right through carry H
'
z_rr_h		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		and	HL, #$ff		' clear H, keep L
		shr	alu, #8
		long	@alu_rr
		shl	alu, #8
		or	HL, alu			' paste alu in H
:end

'*******************************************************
' cb 1d Rotate right through carry L
'
z_rr_l		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		andn	HL, #$ff		' clear L, keep H
		and	alu, #$ff
		long	@alu_rr
		or	HL, alu			' paste alu in L
:end

'*******************************************************
' cb 1e Rotate right through carry memory (HL)
'
z_rr_m		long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		long	@alu_rr
		call	#wr_memb_alu
:end

'*******************************************************
' cb 1f Rotate right through carry A
'
z_rr_a		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, AF
		and	AF, #$ff		' clear A, keep F
		shr	alu, #8
		long	@alu_rr
		shl	alu, #8
		or	AF, alu			' paste alu in A
:end

'*******************************************************
' cb 20 Shift arithmetic left B
'
z_sla_b		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		long	@b_to_alu
		long	@alu_sla
		shl	alu, #8
		or	BC, alu			' paste alu in B
:end

'*******************************************************
' cb 21 Shift arithmetic left C
'
z_sla_c		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		long	@c_to_alu
		long	@alu_sla
		or	BC, alu			' paste alu in C
:end

'*******************************************************
' cb 22 Shift arithmetic left D
'
z_sla_d		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		and	DE, #$ff		' clear D, keep E
		shr	alu, #8
		long	@alu_sla
		shl	alu, #8
		or	DE, alu			' paste alu in D
:end

'*******************************************************
' cb 23 Shift arithmetic left E
'
z_sla_e		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		andn	DE, #$ff		' clear E, keep D
		and	alu, #$ff
		long	@alu_sla
		or	DE, alu			' paste alu in E
:end

'*******************************************************
' cb 24 Shift arithmetic left H
'
z_sla_h		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		and	HL, #$ff		' clear H, keep L
		shr	alu, #8
		long	@alu_sla
		shl	alu, #8
		or	HL, alu			' paste alu in H
:end

'*******************************************************
' cb 25 Shift arithmetic left L
'
z_sla_l		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		andn	HL, #$ff		' clear L, keep H
		and	alu, #$ff
		long	@alu_sla
		or	HL, alu			' paste alu in L
:end

'*******************************************************
' cb 26 Shift arithmetic left memory (HL)
'
z_sla_m		long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		long	@alu_sla
		call	#wr_memb_alu
:end

'*******************************************************
' cb 27 Shift arithmetic left A
'
z_sla_a		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, AF
		and	AF, #$ff		' clear A, keep F
		shr	alu, #8
		long	@alu_sla
		shl	alu, #8
		or	AF, alu			' paste alu in A
:end

'*******************************************************
' cb 28 Shift arithmetic right B
'
z_sra_b		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		long	@b_to_alu
		long	@alu_sra
		shl	alu, #8
		or	BC, alu			' paste alu in B
:end

'*******************************************************
' cb 29 Shift arithmetic right C
'
z_sra_c		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		long	@c_to_alu
		long	@alu_sra
		or	BC, alu			' paste alu in C
:end

'*******************************************************
' cb 2a Shift arithmetic right D
'
z_sra_d		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		and	DE, #$ff		' clear D, keep E
		shr	alu, #8
		long	@alu_sra
		shl	alu, #8
		or	DE, alu			' paste alu in D
:end

'*******************************************************
' cb 2b Shift arithmetic right E
'
z_sra_e		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		andn	DE, #$ff		' clear E, keep D
		and	alu, #$ff
		long	@alu_sra
		or	DE, alu			' paste alu in E
:end

'*******************************************************
' cb 2c Shift arithmetic right H
'
z_sra_h		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		and	HL, #$ff		' clear H, keep L
		shr	alu, #8
		long	@alu_sra
		shl	alu, #8
		or	HL, alu			' paste alu in H
:end

'*******************************************************
' cb 2d Shift arithmetic right L
'
z_sra_l		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		andn	HL, #$ff		' clear L, keep H
		and	alu, #$ff
		long	@alu_sra
		or	HL, alu			' paste alu in L
:end

'*******************************************************
' cb 2e Shift arithmetic right memory (HL)
'
z_sra_m		long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		long	@alu_sra
		call	#wr_memb_alu
:end

'*******************************************************
' cb 2f Shift arithmetic right A
'
z_sra_a		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, AF
		and	AF, #$ff		' clear A, keep F
		shr	alu, #8
		long	@alu_sra
		shl	alu, #8
		or	AF, alu			' paste alu in A
:end
'*******************************************************
' cb 30 Shift left inverted B
'
z_sli_b		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		long	@b_to_alu
		long	@alu_sli
		shl	alu, #8
		or	BC, alu			' paste alu in B
:end

'*******************************************************
' cb 31 Shift left inverted C
'
z_sli_c		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		long	@c_to_alu
		long	@alu_sli
		or	BC, alu			' paste alu in C
:end

'*******************************************************
' cb 32 Shift left inverted D
'
z_sli_d		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		and	DE, #$ff		' clear D, keep E
		shr	alu, #8
		long	@alu_sli
		shl	alu, #8
		or	DE, alu			' paste alu in D
:end

'*******************************************************
' cb 33 Shift left inverted E
'
z_sli_e		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		andn	DE, #$ff		' clear E, keep D
		and	alu, #$ff
		long	@alu_sli
		or	DE, alu			' paste alu in E
:end

'*******************************************************
' cb 34 Shift left inverted H
'
z_sli_h		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		and	HL, #$ff		' clear H, keep L
		shr	alu, #8
		long	@alu_sli
		shl	alu, #8
		or	HL, alu			' paste alu in H
:end

'*******************************************************
' cb 35 Shift left inverted L
'
z_sli_l		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		andn	HL, #$ff		' clear L, keep H
		and	alu, #$ff
		long	@alu_sli
		or	HL, alu			' paste alu in L
:end

'*******************************************************
' cb 36 Shift left inverted memory (HL)
'
z_sli_m		long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		long	@alu_sli
		call	#wr_memb_alu
:end

'*******************************************************
' cb 37 Shift left inverted A
'
z_sli_a		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, AF
		and	AF, #$ff		' clear A, keep F
		shr	alu, #8
		long	@alu_sli
		shl	alu, #8
		or	AF, alu			' paste alu in A
:end

'*******************************************************
' cb 38 Shift right logical B
'
z_srl_b		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		long	@b_to_alu
		long	@alu_srl
		shl	alu, #8
		or	BC, alu			' paste alu in B
:end

'*******************************************************
' cb 39 Shift right logical C
'
z_srl_c		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		long	@c_to_alu
		long	@alu_srl
		or	BC, alu			' paste alu in C
:end

'*******************************************************
' cb 3a Shift right logical D
'
z_srl_d		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		and	DE, #$ff		' clear D, keep E
		shr	alu, #8
		long	@alu_srl
		shl	alu, #8
		or	DE, alu			' paste alu in D
:end

'*******************************************************
' cb 3b Shift right logical E
'
z_srl_e		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		andn	DE, #$ff		' clear E, keep D
		and	alu, #$ff
		long	@alu_srl
		or	DE, alu			' paste alu in E
:end

'*******************************************************
' cb 3c Shift right logical H
'
z_srl_h		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		and	HL, #$ff		' clear H, keep L
		shr	alu, #8
		long	@alu_srl
		shl	alu, #8
		or	HL, alu			' paste alu in H
:end

'*******************************************************
' cb 3d Shift right logical L
'
z_srl_l		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		andn	HL, #$ff		' clear L, keep H
		and	alu, #$ff
		long	@alu_srl
		or	HL, alu			' paste alu in L
:end

'*******************************************************
' cb 3e Shift right logical memory (HL)
'
z_srl_m		long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		long	@alu_srl
		call	#wr_memb_alu
:end

'*******************************************************
' cb 3f Shift right logical A
'
z_srl_a		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, AF
		and	AF, #$ff		' clear A, keep F
		shr	alu, #8
		long	@alu_srl
		shl	alu, #8
		or	AF, alu			' paste alu in A
:end

'*******************************************************
' cb 40 Test bit 0 in B
'
z_bit_0_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, BC
		shr	alu, #8
		mov	tmp, #bit0
		long	@alu_bit
:end

'*******************************************************
' cb 41 Test bit 0 in C
'
z_bit_0_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, BC
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit0
		long	@alu_bit
:end

'*******************************************************
' cb 42 Test bit 0 in D
'
z_bit_0_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		shr	alu, #8
		mov	tmp, #bit0
		long	@alu_bit
:end

'*******************************************************
' cb 43 Test bit 0 in E
'
z_bit_0_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit0
		long	@alu_bit
:end

'*******************************************************
' cb 44 Test bit 0 in H
'
z_bit_0_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		shr	alu, #8
		mov	tmp, #bit0
		long	@alu_bit
:end

'*******************************************************
' cb 45 Test bit 0 in L
'
z_bit_0_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit0
		long	@alu_bit
:end

'*******************************************************
' cb 46 Test bit 0 in memory (HL)
'
z_bit_0_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		mov	tmp, #bit0
		long	@alu_bit
:end

'*******************************************************
' cb 47 Test bit 0 in A
'
z_bit_0_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, AF
		shr	alu, #8
		mov	tmp, #bit0
		long	@alu_bit
:end

'*******************************************************
' cb 48 Test bit 1 in B
'
z_bit_1_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, BC
		shr	alu, #8
		mov	tmp, #bit1
		long	@alu_bit
:end

'*******************************************************
' cb 49 Test bit 1 in C
'
z_bit_1_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, BC
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit1
		long	@alu_bit
:end

'*******************************************************
' cb 4a Test bit 1 in D
'
z_bit_1_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		shr	alu, #8
		mov	tmp, #bit1
		long	@alu_bit
:end

'*******************************************************
' cb 4b Test bit 1 in E
'
z_bit_1_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit1
		long	@alu_bit
:end

'*******************************************************
' cb 4c Test bit 1 in H
'
z_bit_1_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		shr	alu, #8
		mov	tmp, #bit1
		long	@alu_bit
:end

'*******************************************************
' cb 4d Test bit 1 in L
'
z_bit_1_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit1
		long	@alu_bit
:end

'*******************************************************
' cb 4e Test bit 1 in memory (HL)
'
z_bit_1_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		mov	tmp, #bit1
		long	@alu_bit
:end

'*******************************************************
' cb 4f Test bit 1 in A
'
z_bit_1_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, AF
		shr	alu, #8
		mov	tmp, #bit1
		long	@alu_bit
:end

'*******************************************************
' cb 50 Test bit 2 in B
'
z_bit_2_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, BC
		shr	alu, #8
		mov	tmp, #bit2
		long	@alu_bit
:end

'*******************************************************
' cb 51 Test bit 2 in C
'
z_bit_2_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, BC
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit2
		long	@alu_bit
:end

'*******************************************************
' cb 52 Test bit 2 in D
'
z_bit_2_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		shr	alu, #8
		mov	tmp, #bit2
		long	@alu_bit
:end

'*******************************************************
' cb 53 Test bit 2 in E
'
z_bit_2_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit2
		long	@alu_bit
:end

'*******************************************************
' cb 54 Test bit 2 in H
'
z_bit_2_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		shr	alu, #8
		mov	tmp, #bit2
		long	@alu_bit
:end

'*******************************************************
' cb 55 Test bit 2 in L
'
z_bit_2_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit2
		long	@alu_bit
:end

'*******************************************************
' cb 56 Test bit 2 in memory (HL)
'
z_bit_2_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		mov	tmp, #bit2
		long	@alu_bit
:end

'*******************************************************
' cb 57 Test bit 2 in A
'
z_bit_2_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, AF
		shr	alu, #8
		mov	tmp, #bit2
		long	@alu_bit
:end

'*******************************************************
' cb 58 Test bit 3 in B
'
z_bit_3_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, BC
		shr	alu, #8
		mov	tmp, #bit3
		long	@alu_bit
:end

'*******************************************************
' cb 59 Test bit 3 in C
'
z_bit_3_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, BC
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit3
		long	@alu_bit
:end

'*******************************************************
' cb 5a Test bit 3 in D
'
z_bit_3_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		shr	alu, #8
		mov	tmp, #bit3
		long	@alu_bit
:end

'*******************************************************
' cb 5b Test bit 3 in E
'
z_bit_3_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit3
		long	@alu_bit
:end

'*******************************************************
' cb 5c Test bit 3 in H
'
z_bit_3_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		shr	alu, #8
		mov	tmp, #bit3
		long	@alu_bit
:end

'*******************************************************
' cb 5d Test bit 3 in L
'
z_bit_3_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit3
		long	@alu_bit
:end

'*******************************************************
' cb 5e Test bit 3 in memory (HL)
'
z_bit_3_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		mov	tmp, #bit3
		long	@alu_bit
:end

'*******************************************************
' cb 5f Test bit 3 in A
'
z_bit_3_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, AF
		shr	alu, #8
		mov	tmp, #bit3
		long	@alu_bit
:end

'*******************************************************
' cb 60 Test bit 4 in B
'
z_bit_4_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, BC
		shr	alu, #8
		mov	tmp, #bit4
		long	@alu_bit
:end

'*******************************************************
' cb 61 Test bit 4 in C
'
z_bit_4_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, BC
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit4
		long	@alu_bit
:end

'*******************************************************
' cb 62 Test bit 4 in D
'
z_bit_4_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		shr	alu, #8
		mov	tmp, #bit4
		long	@alu_bit
:end

'*******************************************************
' cb 63 Test bit 4 in E
'
z_bit_4_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit4
		long	@alu_bit
:end

'*******************************************************
' cb 64 Test bit 4 in H
'
z_bit_4_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		shr	alu, #8
		mov	tmp, #bit4
		long	@alu_bit
:end

'*******************************************************
' cb 65 Test bit 4 in L
'
z_bit_4_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit4
		long	@alu_bit
:end

'*******************************************************
' cb 66 Test bit 4 in memory (HL)
'
z_bit_4_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		mov	tmp, #bit4
		long	@alu_bit
:end

'*******************************************************
' cb 67 Test bit 4 in A
'
z_bit_4_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, AF
		shr	alu, #8
		mov	tmp, #bit4
		long	@alu_bit
:end

'*******************************************************
' cb 68 Test bit 5 in B
'
z_bit_5_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, BC
		shr	alu, #8
		mov	tmp, #bit5
		long	@alu_bit
:end

'*******************************************************
' cb 69 Test bit 5 in C
'
z_bit_5_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, BC
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit5
		long	@alu_bit
:end

'*******************************************************
' cb 6a Test bit 5 in D
'
z_bit_5_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		shr	alu, #8
		mov	tmp, #bit5
		long	@alu_bit
:end

'*******************************************************
' cb 6b Test bit 5 in E
'
z_bit_5_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit5
		long	@alu_bit
:end

'*******************************************************
' cb 6c Test bit 5 in H
'
z_bit_5_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		shr	alu, #8
		mov	tmp, #bit5
		long	@alu_bit
:end

'*******************************************************
' cb 6d Test bit 5 in L
'
z_bit_5_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit5
		long	@alu_bit
:end

'*******************************************************
' cb 6e Test bit 5 in memory (HL)
'
z_bit_5_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		mov	tmp, #bit5
		long	@alu_bit
:end

'*******************************************************
' cb 6f Test bit 5 in A
'
z_bit_5_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, AF
		shr	alu, #8
		mov	tmp, #bit5
		long	@alu_bit
:end

'*******************************************************
' cb 70 Test bit 6 in B
'
z_bit_6_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, BC
		shr	alu, #8
		mov	tmp, #bit6
		long	@alu_bit
:end

'*******************************************************
' cb 71 Test bit 6 in C
'
z_bit_6_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, BC
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit6
		long	@alu_bit
:end

'*******************************************************
' cb 72 Test bit 6 in D
'
z_bit_6_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		shr	alu, #8
		mov	tmp, #bit6
		long	@alu_bit
:end

'*******************************************************
' cb 73 Test bit 6 in E
'
z_bit_6_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit6
		long	@alu_bit
:end

'*******************************************************
' cb 74 Test bit 6 in H
'
z_bit_6_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		shr	alu, #8
		mov	tmp, #bit6
		long	@alu_bit
:end

'*******************************************************
' cb 75 Test bit 6 in L
'
z_bit_6_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit6
		long	@alu_bit
:end

'*******************************************************
' cb 76 Test bit 6 in memory (HL)
'
z_bit_6_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		mov	tmp, #bit6
		long	@alu_bit
:end

'*******************************************************
' cb 77 Test bit 6 in A
'
z_bit_6_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, AF
		shr	alu, #8
		mov	tmp, #bit6
		long	@alu_bit
:end

'*******************************************************
' cb 78 Test bit 7 in B
'
z_bit_7_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, BC
		shr	alu, #8
		mov	tmp, #bit7
		long	@alu_bit
:end

'*******************************************************
' cb 79 Test bit 7 in C
'
z_bit_7_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, BC
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit7
		long	@alu_bit
:end

'*******************************************************
' cb 7a Test bit 7 in D
'
z_bit_7_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
		shr	alu, #8
		mov	tmp, #bit7
		long	@alu_bit
:end

'*******************************************************
' cb 7b Test bit 7 in E
'
z_bit_7_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, DE
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit7
		long	@alu_bit
:end

'*******************************************************
' cb 7c Test bit 7 in H
'
z_bit_7_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
		shr	alu, #8
		mov	tmp, #bit7
		long	@alu_bit
:end

'*******************************************************
' cb 7d Test bit 7 in L
'
z_bit_7_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, HL
'		and	alu, #$ff		' not really necessary
		mov	tmp, #bit7
		long	@alu_bit
:end

'*******************************************************
' cb 7e Test bit 7 in memory (HL)
'
z_bit_7_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		mov	tmp, #bit7
		long	@alu_bit
:end

'*******************************************************
' cb 7f Test bit 7 in A
'
z_bit_7_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, AF
		shr	alu, #8
		mov	tmp, #bit7
		long	@alu_bit
:end

'*******************************************************
' cb 80 Reset bit 0 in B
'
z_res_0_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	BC, #8
		andn	BC, #bit0
		rol	BC, #8
:end

'*******************************************************
' cb 81 Reset bit 0 in C
'
z_res_0_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	BC, #bit0
:end

'*******************************************************
' cb 82 Reset bit 0 in D
'
z_res_0_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	DE, #8
		andn	DE, #bit0
		rol	DE, #8
:end

'*******************************************************
' cb 83 Reset bit 0 in E
'
z_res_0_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	DE, #bit0
:end

'*******************************************************
' cb 84 Reset bit 0 in H
'
z_res_0_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	HL, #8
		andn	HL, #bit0
		rol	HL, #8
:end

'*******************************************************
' cb 85 Reset bit 0 in L
'
z_res_0_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	HL, #bit0
:end

'*******************************************************
' cb 86 Reset bit 0 in memory (HL)
'
z_res_0_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		andn	alu, #bit0
		call	#wr_memb_alu
:end

'*******************************************************
' cb 87 Reset bit 0 in A
'
z_res_0_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	AF, #8
		andn	AF, #bit0
		rol	AF, #8
:end

'*******************************************************
' cb 88 Reset bit 1 in B
'
z_res_1_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	BC, #8
		andn	BC, #bit1
		rol	BC, #8
:end

'*******************************************************
' cb 89 Reset bit 1 in C
'
z_res_1_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	BC, #bit1
:end

'*******************************************************
' cb 8a Reset bit 1 in D
'
z_res_1_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	DE, #8
		andn	DE, #bit1
		rol	DE, #8
:end

'*******************************************************
' cb 8b Reset bit 1 in E
'
z_res_1_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	DE, #bit1
:end

'*******************************************************
' cb 8c Reset bit 1 in H
'
z_res_1_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	HL, #8
		andn	HL, #bit1
		rol	HL, #8
:end

'*******************************************************
' cb 8d Reset bit 1 in L
'
z_res_1_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	HL, #bit1
:end

'*******************************************************
' cb 8e Reset bit 1 in memory (HL)
'
z_res_1_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		andn	alu, #bit1
		call	#wr_memb_alu
:end

'*******************************************************
' cb 8f Reset bit 1 in A
'
z_res_1_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	AF, #8
		andn	AF, #bit1
		rol	AF, #8
:end

'*******************************************************
' cb 90 Reset bit 2 in B
'
z_res_2_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	BC, #8
		andn	BC, #bit2
		rol	BC, #8
:end

'*******************************************************
' cb 91 Reset bit 2 in C
'
z_res_2_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	BC, #bit2
:end

'*******************************************************
' cb 92 Reset bit 2 in D
'
z_res_2_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	DE, #8
		andn	DE, #bit2
		rol	DE, #8
:end

'*******************************************************
' cb 93 Reset bit 2 in E
'
z_res_2_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	DE, #bit2
:end

'*******************************************************
' cb 94 Reset bit 2 in H
'
z_res_2_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	HL, #8
		andn	HL, #bit2
		rol	HL, #8
:end

'*******************************************************
' cb 95 Reset bit 2 in L
'
z_res_2_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	HL, #bit2
:end

'*******************************************************
' cb 96 Reset bit 2 in memory (HL)
'
z_res_2_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		andn	alu, #bit2
		call	#wr_memb_alu
:end

'*******************************************************
' cb 97 Reset bit 2 in A
'
z_res_2_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	AF, #8
		andn	AF, #bit2
		rol	AF, #8
:end

'*******************************************************
' cb 98 Reset bit 3 in B
'
z_res_3_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	BC, #8
		andn	BC, #bit3
		rol	BC, #8
:end

'*******************************************************
' cb 99 Reset bit 3 in C
'
z_res_3_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	BC, #bit3
:end

'*******************************************************
' cb 9a Reset bit 3 in D
'
z_res_3_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	DE, #8
		andn	DE, #bit3
		rol	DE, #8
:end

'*******************************************************
' cb 9b Reset bit 3 in E
'
z_res_3_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	DE, #bit3
:end

'*******************************************************
' cb 9c Reset bit 3 in H
'
z_res_3_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	HL, #8
		andn	HL, #bit3
		rol	HL, #8
:end

'*******************************************************
' cb 9d Reset bit 3 in L
'
z_res_3_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	HL, #bit3
:end

'*******************************************************
' cb 9e Reset bit 3 in memory (HL)
'
z_res_3_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		andn	alu, #bit3
		call	#wr_memb_alu
:end

'*******************************************************
' cb 9f Reset bit 3 in A
'
z_res_3_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	AF, #8
		andn	AF, #bit3
		rol	AF, #8
:end

'*******************************************************
' cb a0 Reset bit 4 in B
'
z_res_4_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	BC, #8
		andn	BC, #bit4
		rol	BC, #8
:end

'*******************************************************
' cb a1 Reset bit 4 in C
'
z_res_4_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	BC, #bit4
:end

'*******************************************************
' cb a2 Reset bit 4 in D
'
z_res_4_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	DE, #8
		andn	DE, #bit4
		rol	DE, #8
:end

'*******************************************************
' cb a3 Reset bit 4 in E
'
z_res_4_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	DE, #bit4
:end

'*******************************************************
' cb a4 Reset bit 4 in H
'
z_res_4_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	HL, #8
		andn	HL, #bit4
		rol	HL, #8
:end

'*******************************************************
' cb a5 Reset bit 4 in L
'
z_res_4_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	HL, #bit4
:end

'*******************************************************
' cb a6 Reset bit 4 in memory (HL)
'
z_res_4_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		andn	alu, #bit4
		call	#wr_memb_alu
:end

'*******************************************************
' cb a7 Reset bit 4 in A
'
z_res_4_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	AF, #8
		andn	AF, #bit4
		rol	AF, #8
:end

'*******************************************************
' cb a8 Reset bit 5 in B
'
z_res_5_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	BC, #8
		andn	BC, #bit5
		rol	BC, #8
:end

'*******************************************************
' cb a9 Reset bit 5 in C
'
z_res_5_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	BC, #bit5
:end

'*******************************************************
' cb aa Reset bit 5 in D
'
z_res_5_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	DE, #8
		andn	DE, #bit5
		rol	DE, #8
:end

'*******************************************************
' cb ab Reset bit 5 in E
'
z_res_5_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	DE, #bit5
:end

'*******************************************************
' cb ac Reset bit 5 in H
'
z_res_5_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	HL, #8
		andn	HL, #bit5
		rol	HL, #8
:end

'*******************************************************
' cb ad Reset bit 5 in L
'
z_res_5_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	HL, #bit5
:end

'*******************************************************
' cb ae Reset bit 5 in memory (HL)
'
z_res_5_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		andn	alu, #bit5
		call	#wr_memb_alu
:end

'*******************************************************
' cb af Reset bit 5 in A
'
z_res_5_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	AF, #8
		andn	AF, #bit5
		rol	AF, #8
:end

'*******************************************************
' cb b0 Reset bit 6 in B
'
z_res_6_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	BC, #8
		andn	BC, #bit6
		rol	BC, #8
:end

'*******************************************************
' cb b1 Reset bit 6 in C
'
z_res_6_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	BC, #bit6
:end

'*******************************************************
' cb b2 Reset bit 6 in D
'
z_res_6_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	DE, #8
		andn	DE, #bit6
		rol	DE, #8
:end

'*******************************************************
' cb b3 Reset bit 6 in E
'
z_res_6_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	DE, #bit6
:end

'*******************************************************
' cb b4 Reset bit 6 in H
'
z_res_6_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	HL, #8
		andn	HL, #bit6
		rol	HL, #8
:end

'*******************************************************
' cb b5 Reset bit 6 in L
'
z_res_6_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	HL, #bit6
:end

'*******************************************************
' cb b6 Reset bit 6 in memory (HL)
'
z_res_6_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		andn	alu, #bit6
		call	#wr_memb_alu
:end

'*******************************************************
' cb b7 Reset bit 6 in A
'
z_res_6_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	AF, #8
		andn	AF, #bit6
		rol	AF, #8
:end

'*******************************************************
' cb b8 Reset bit 7 in B
'
z_res_7_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	BC, #8
		andn	BC, #bit7
		rol	BC, #8
:end

'*******************************************************
' cb b9 Reset bit 7 in C
'
z_res_7_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	BC, #bit7
:end

'*******************************************************
' cb ba Reset bit 7 in D
'
z_res_7_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	DE, #8
		andn	DE, #bit7
		rol	DE, #8
:end

'*******************************************************
' cb bb Reset bit 7 in E
'
z_res_7_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	DE, #bit7
:end

'*******************************************************
' cb bc Reset bit 7 in H
'
z_res_7_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	HL, #8
		andn	HL, #bit7
		rol	HL, #8
:end

'*******************************************************
' cb bd Reset bit 7 in L
'
z_res_7_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		andn	HL, #bit7
:end

'*******************************************************
' cb be Reset bit 7 in memory (HL)
'
z_res_7_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		andn	alu, #bit7
		call	#wr_memb_alu
:end

'*******************************************************
' cb bf Reset bit 7 in A
'
z_res_7_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	AF, #8
		andn	AF, #bit7
		rol	AF, #8
:end
'*******************************************************
' cb c0 Set bit 0 in B
'
z_set_0_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	BC, #8
		or	BC, #bit0
		rol	BC, #8
:end

'*******************************************************
' cb c1 Set bit 0 in C
'
z_set_0_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	BC, #bit0
:end

'*******************************************************
' cb c2 Set bit 0 in D
'
z_set_0_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	DE, #8
		or	DE, #bit0
		rol	DE, #8
:end

'*******************************************************
' cb c3 Set bit 0 in E
'
z_set_0_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	DE, #bit0
:end

'*******************************************************
' cb c4 Set bit 0 in H
'
z_set_0_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	HL, #8
		or	HL, #bit0
		rol	HL, #8
:end

'*******************************************************
' cb c5 Set bit 0 in L
'
z_set_0_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	HL, #bit0
:end

'*******************************************************
' cb c6 Set bit 0 in memory (HL)
'
z_set_0_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		or	alu, #bit0
		call	#wr_memb_alu
:end

'*******************************************************
' cb c7 Set bit 0 in A
'
z_set_0_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	AF, #8
		or	AF, #bit0
		rol	AF, #8
:end

'*******************************************************
' cb c8 Set bit 1 in B
'
z_set_1_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	BC, #8
		or	BC, #bit1
		rol	BC, #8
:end

'*******************************************************
' cb c9 Set bit 1 in C
'
z_set_1_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	BC, #bit1
:end

'*******************************************************
' cb ca Set bit 1 in D
'
z_set_1_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	DE, #8
		or	DE, #bit1
		rol	DE, #8
:end

'*******************************************************
' cb cb Set bit 1 in E
'
z_set_1_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	DE, #bit1
:end

'*******************************************************
' cb cc Set bit 1 in H
'
z_set_1_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	HL, #8
		or	HL, #bit1
		rol	HL, #8
:end

'*******************************************************
' cb cd Set bit 1 in L
'
z_set_1_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	HL, #bit1
:end

'*******************************************************
' cb ce Set bit 1 in memory (HL)
'
z_set_1_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		or	alu, #bit1
		call	#wr_memb_alu
:end

'*******************************************************
' cb cf Set bit 1 in A
'
z_set_1_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	AF, #8
		or	AF, #bit1
		rol	AF, #8
:end

'*******************************************************
' cb d0 Set bit 2 in B
'
z_set_2_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	BC, #8
		or	BC, #bit2
		rol	BC, #8
:end

'*******************************************************
' cb d1 Set bit 2 in C
'
z_set_2_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	BC, #bit2
:end

'*******************************************************
' cb d2 Set bit 2 in D
'
z_set_2_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	DE, #8
		or	DE, #bit2
		rol	DE, #8
:end

'*******************************************************
' cb d3 Set bit 2 in E
'
z_set_2_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	DE, #bit2
:end

'*******************************************************
' cb d4 Set bit 2 in H
'
z_set_2_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	HL, #8
		or	HL, #bit2
		rol	HL, #8
:end

'*******************************************************
' cb d5 Set bit 2 in L
'
z_set_2_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	HL, #bit2
:end

'*******************************************************
' cb d6 Set bit 2 in memory (HL)
'
z_set_2_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		or	alu, #bit2
		call	#wr_memb_alu
:end

'*******************************************************
' cb d7 Set bit 2 in A
'
z_set_2_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	AF, #8
		or	AF, #bit2
		rol	AF, #8
:end

'*******************************************************
' cb d8 Set bit 3 in B
'
z_set_3_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	BC, #8
		or	BC, #bit3
		rol	BC, #8
:end

'*******************************************************
' cb d9 Set bit 3 in C
'
z_set_3_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	BC, #bit3
:end

'*******************************************************
' cb da Set bit 3 in D
'
z_set_3_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	DE, #8
		or	DE, #bit3
		rol	DE, #8
:end

'*******************************************************
' cb db Set bit 3 in E
'
z_set_3_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	DE, #bit3
:end

'*******************************************************
' cb dc Set bit 3 in H
'
z_set_3_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	HL, #8
		or	HL, #bit3
		rol	HL, #8
:end

'*******************************************************
' cb dd Set bit 3 in L
'
z_set_3_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	HL, #bit3
:end

'*******************************************************
' cb de Set bit 3 in memory (HL)
'
z_set_3_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		or	alu, #bit3
		call	#wr_memb_alu
:end

'*******************************************************
' cb df Set bit 3 in A
'
z_set_3_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	AF, #8
		or	AF, #bit3
		rol	AF, #8
:end

'*******************************************************
' cb e0 Set bit 4 in B
'
z_set_4_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	BC, #8
		or	BC, #bit4
		rol	BC, #8
:end

'*******************************************************
' cb e1 Set bit 4 in C
'
z_set_4_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	BC, #bit4
:end

'*******************************************************
' cb e2 Set bit 4 in D
'
z_set_4_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	DE, #8
		or	DE, #bit4
		rol	DE, #8
:end

'*******************************************************
' cb e3 Set bit 4 in E
'
z_set_4_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	DE, #bit4
:end

'*******************************************************
' cb e4 Set bit 4 in H
'
z_set_4_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	HL, #8
		or	HL, #bit4
		rol	HL, #8
:end

'*******************************************************
' cb e5 Set bit 4 in L
'
z_set_4_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	HL, #bit4
:end

'*******************************************************
' cb e6 Set bit 4 in memory (HL)
'
z_set_4_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		or	alu, #bit4
		call	#wr_memb_alu
:end

'*******************************************************
' cb e7 Set bit 4 in A
'
z_set_4_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	AF, #8
		or	AF, #bit4
		rol	AF, #8
:end

'*******************************************************
' cb e8 Set bit 5 in B
'
z_set_5_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	BC, #8
		or	BC, #bit5
		rol	BC, #8
:end

'*******************************************************
' cb e9 Set bit 5 in C
'
z_set_5_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	BC, #bit5
:end

'*******************************************************
' cb ea Set bit 5 in D
'
z_set_5_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	DE, #8
		or	DE, #bit5
		rol	DE, #8
:end

'*******************************************************
' cb eb Set bit 5 in E
'
z_set_5_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	DE, #bit5
:end

'*******************************************************
' cb ec Set bit 5 in H
'
z_set_5_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	HL, #8
		or	HL, #bit5
		rol	HL, #8
:end

'*******************************************************
' cb ed Set bit 5 in L
'
z_set_5_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	HL, #bit5
:end

'*******************************************************
' cb ee Set bit 5 in memory (HL)
'
z_set_5_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		or	alu, #bit5
		call	#wr_memb_alu
:end

'*******************************************************
' cb ef Set bit 5 in A
'
z_set_5_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	AF, #8
		or	AF, #bit5
		rol	AF, #8
:end

'*******************************************************
' cb f0 Set bit 6 in B
'
z_set_6_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	BC, #8
		or	BC, #bit6
		rol	BC, #8
:end

'*******************************************************
' cb f1 Set bit 6 in C
'
z_set_6_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	BC, #bit6
:end

'*******************************************************
' cb f2 Set bit 6 in D
'
z_set_6_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	DE, #8
		or	DE, #bit6
		rol	DE, #8
:end

'*******************************************************
' cb f3 Set bit 6 in E
'
z_set_6_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	DE, #bit6
:end

'*******************************************************
' cb f4 Set bit 6 in H
'
z_set_6_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	HL, #8
		or	HL, #bit6
		rol	HL, #8
:end

'*******************************************************
' cb f5 Set bit 6 in L
'
z_set_6_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	HL, #bit6
:end

'*******************************************************
' cb f6 Set bit 6 in memory (HL)
'
z_set_6_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		or	alu, #bit6
		call	#wr_memb_alu
:end

'*******************************************************
' cb f7 Set bit 6 in A
'
z_set_6_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	AF, #8
		or	AF, #bit6
		rol	AF, #8
:end

'*******************************************************
' cb f8 Set bit 7 in B
'
z_set_7_b	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	BC, #8
		or	BC, #bit7
		rol	BC, #8
:end

'*******************************************************
' cb f9 Set bit 7 in C
'
z_set_7_c	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	BC, #bit7
:end

'*******************************************************
' cb fa Set bit 7 in D
'
z_set_7_d	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	DE, #8
		or	DE, #bit7
		rol	DE, #8
:end

'*******************************************************
' cb fb Set bit 7 in E
'
z_set_7_e	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	DE, #bit7
:end

'*******************************************************
' cb fc Set bit 7 in H
'
z_set_7_h	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	HL, #8
		or	HL, #bit7
		rol	HL, #8
:end

'*******************************************************
' cb fd Set bit 7 in L
'
z_set_7_l	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		or	HL, #bit7
:end

'*******************************************************
' cb fe Set bit 7 in memory (HL)
'
z_set_7_m	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		or	alu, #bit7
		call	#wr_memb_alu
:end

'*******************************************************
' cb ff Set bit 7 in A
'
z_set_7_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		ror	AF, #8
		or	AF, #bit7
		rol	AF, #8
:end

snippets_cb
	{00}	long	@z_rlc_b
		long	@z_rlc_c
		long	@z_rlc_d
		long	@z_rlc_e
		long	@z_rlc_h
		long	@z_rlc_l
		long	@z_rlc_m
		long	@z_rlc_a
	{08}	long	@z_rrc_b
		long	@z_rrc_c
		long	@z_rrc_d
		long	@z_rrc_e
		long	@z_rrc_h
		long	@z_rrc_l
		long	@z_rrc_m
		long	@z_rrc_a
	{10}	long	@z_rl_b
		long	@z_rl_c
		long	@z_rl_d
		long	@z_rl_e
		long	@z_rl_h
		long	@z_rl_l
		long	@z_rl_m
		long	@z_rl_a
	{18}	long	@z_rr_b
		long	@z_rr_c
		long	@z_rr_d
		long	@z_rr_e
		long	@z_rr_h
		long	@z_rr_l
		long	@z_rr_m
		long	@z_rr_a
	{20}	long	@z_sla_b
		long	@z_sla_c
		long	@z_sla_d
		long	@z_sla_e
		long	@z_sla_h
		long	@z_sla_l
		long	@z_sla_m
		long	@z_sla_a
	{28}	long	@z_sra_b
		long	@z_sra_c
		long	@z_sra_d
		long	@z_sra_e
		long	@z_sra_h
		long	@z_sra_l
		long	@z_sra_m
		long	@z_sra_a
	{30}	long	@z_sli_b
		long	@z_sli_c
		long	@z_sli_d
		long	@z_sli_e
		long	@z_sli_h
		long	@z_sli_l
		long	@z_sli_m
		long	@z_sli_a
	{38}	long	@z_srl_b
		long	@z_srl_c
		long	@z_srl_d
		long	@z_srl_e
		long	@z_srl_h
		long	@z_srl_l
		long	@z_srl_m
		long	@z_srl_a
	{40}	long	@z_bit_0_b
		long	@z_bit_0_c
		long	@z_bit_0_d
		long	@z_bit_0_e
		long	@z_bit_0_h
		long	@z_bit_0_l
		long	@z_bit_0_m
		long	@z_bit_0_a
	{48}	long	@z_bit_1_b
		long	@z_bit_1_c
		long	@z_bit_1_d
		long	@z_bit_1_e
		long	@z_bit_1_h
		long	@z_bit_1_l
		long	@z_bit_1_m
		long	@z_bit_1_a
	{50}	long	@z_bit_2_b
		long	@z_bit_2_c
		long	@z_bit_2_d
		long	@z_bit_2_e
		long	@z_bit_2_h
		long	@z_bit_2_l
		long	@z_bit_2_m
		long	@z_bit_2_a
	{58}	long	@z_bit_3_b
		long	@z_bit_3_c
		long	@z_bit_3_d
		long	@z_bit_3_e
		long	@z_bit_3_h
		long	@z_bit_3_l
		long	@z_bit_3_m
		long	@z_bit_3_a
	{60}	long	@z_bit_4_b
		long	@z_bit_4_c
		long	@z_bit_4_d
		long	@z_bit_4_e
		long	@z_bit_4_h
		long	@z_bit_4_l
		long	@z_bit_4_m
		long	@z_bit_4_a
	{68}	long	@z_bit_5_b
		long	@z_bit_5_c
		long	@z_bit_5_d
		long	@z_bit_5_e
		long	@z_bit_5_h
		long	@z_bit_5_l
		long	@z_bit_5_m
		long	@z_bit_5_a
	{70}	long	@z_bit_6_b
		long	@z_bit_6_c
		long	@z_bit_6_d
		long	@z_bit_6_e
		long	@z_bit_6_h
		long	@z_bit_6_l
		long	@z_bit_6_m
		long	@z_bit_6_a
	{78}	long	@z_bit_7_b
		long	@z_bit_7_c
		long	@z_bit_7_d
		long	@z_bit_7_e
		long	@z_bit_7_h
		long	@z_bit_7_l
		long	@z_bit_7_m
		long	@z_bit_7_a
	{80}	long	@z_res_0_b
		long	@z_res_0_c
		long	@z_res_0_d
		long	@z_res_0_e
		long	@z_res_0_h
		long	@z_res_0_l
		long	@z_res_0_m
		long	@z_res_0_a
	{88}	long	@z_res_1_b
		long	@z_res_1_c
		long	@z_res_1_d
		long	@z_res_1_e
		long	@z_res_1_h
		long	@z_res_1_l
		long	@z_res_1_m
		long	@z_res_1_a
	{90}	long	@z_res_2_b
		long	@z_res_2_c
		long	@z_res_2_d
		long	@z_res_2_e
		long	@z_res_2_h
		long	@z_res_2_l
		long	@z_res_2_m
		long	@z_res_2_a
	{98}	long	@z_res_3_b
		long	@z_res_3_c
		long	@z_res_3_d
		long	@z_res_3_e
		long	@z_res_3_h
		long	@z_res_3_l
		long	@z_res_3_m
		long	@z_res_3_a
	{a0}	long	@z_res_4_b
		long	@z_res_4_c
		long	@z_res_4_d
		long	@z_res_4_e
		long	@z_res_4_h
		long	@z_res_4_l
		long	@z_res_4_m
		long	@z_res_4_a
	{a8}	long	@z_res_5_b
		long	@z_res_5_c
		long	@z_res_5_d
		long	@z_res_5_e
		long	@z_res_5_h
		long	@z_res_5_l
		long	@z_res_5_m
		long	@z_res_5_a
	{b0}	long	@z_res_6_b
		long	@z_res_6_c
		long	@z_res_6_d
		long	@z_res_6_e
		long	@z_res_6_h
		long	@z_res_6_l
		long	@z_res_6_m
		long	@z_res_6_a
	{b8}	long	@z_res_7_b
		long	@z_res_7_c
		long	@z_res_7_d
		long	@z_res_7_e
		long	@z_res_7_h
		long	@z_res_7_l
		long	@z_res_7_m
		long	@z_res_7_a
	{c0}	long	@z_set_0_b
		long	@z_set_0_c
		long	@z_set_0_d
		long	@z_set_0_e
		long	@z_set_0_h
		long	@z_set_0_l
		long	@z_set_0_m
		long	@z_set_0_a
	{c8}	long	@z_set_1_b
		long	@z_set_1_c
		long	@z_set_1_d
		long	@z_set_1_e
		long	@z_set_1_h
		long	@z_set_1_l
		long	@z_set_1_m
		long	@z_set_1_a
	{d0}	long	@z_set_2_b
		long	@z_set_2_c
		long	@z_set_2_d
		long	@z_set_2_e
		long	@z_set_2_h
		long	@z_set_2_l
		long	@z_set_2_m
		long	@z_set_2_a
	{d8}	long	@z_set_3_b
		long	@z_set_3_c
		long	@z_set_3_d
		long	@z_set_3_e
		long	@z_set_3_h
		long	@z_set_3_l
		long	@z_set_3_m
		long	@z_set_3_a
	{e0}	long	@z_set_4_b
		long	@z_set_4_c
		long	@z_set_4_d
		long	@z_set_4_e
		long	@z_set_4_h
		long	@z_set_4_l
		long	@z_set_4_m
		long	@z_set_4_a
	{e8}	long	@z_set_5_b
		long	@z_set_5_c
		long	@z_set_5_d
		long	@z_set_5_e
		long	@z_set_5_h
		long	@z_set_5_l
		long	@z_set_5_m
		long	@z_set_5_a
	{f0}	long	@z_set_6_b
		long	@z_set_6_c
		long	@z_set_6_d
		long	@z_set_6_e
		long	@z_set_6_h
		long	@z_set_6_l
		long	@z_set_6_m
		long	@z_set_6_a
	{f8}	long	@z_set_7_b
		long	@z_set_7_c
		long	@z_set_7_d
		long	@z_set_7_e
		long	@z_set_7_h
		long	@z_set_7_l
		long	@z_set_7_m
		long	@z_set_7_a

'******************************************************************************
'******************************************************************************
'
'	ED xx opcodes
'
'******************************************************************************
'******************************************************************************

alu_adc16	long	(:end - $ - 1)
		test	AF, #c_flag WC
		mov	aux, HL
		mov	alu, HL
		addx	alu, tmp
		shl	alu, #16 WZ, WC
		muxc	AF, #c_flag
		muxz	AF, #z_flag
		shr	alu, #16
		xor	aux, alu
		xor	aux, HL
		shr	aux, #8
		and	aux, #h_flag
		or	AF, aux
		mov	HL, alu
:end

alu_sbc16	long	(:end - $ - 1)
		test	AF, #c_flag WC
		mov	aux, HL
		mov	alu, HL
		subx	alu, tmp
		shl	alu, #16 WZ, WC
		muxc	AF, #c_flag
		muxz	AF, #z_flag
		shr	alu, #16
		xor	aux, alu
		xor	aux, HL
		shr	aux, #8
		and	aux, #h_flag
		or	AF, aux
		mov	HL, alu
:end

'*******************************************************
' ed 40 Input B from port (BC)
'
z_inp_b_bc	long	(:end - $ - 1) | (12 << CYCLES_SHIFT)
		mov	ea, BC
		call	#rd_port_alu
		and	BC, #$ff
		shl	alu, #8
		or	BC, alu
:end

'*******************************************************
' ed 48 Input C from port (BC)
'
z_inp_c_bc	long	(:end - $ - 1) | (12 << CYCLES_SHIFT)
		mov	ea, BC
		call	#rd_port_alu
		andn	BC, #$ff
		or	BC, alu
:end

'*******************************************************
' ed 50 Input D from port (BC)
'
z_inp_d_bc	long	(:end - $ - 1) | (12 << CYCLES_SHIFT)
		mov	ea, BC
		call	#rd_port_alu
		and	DE, #$ff
		shl	alu, #8
		or	DE, alu
:end

'*******************************************************
' ed 58 Input E from port (BC)
'
z_inp_e_bc	long	(:end - $ - 1) | (12 << CYCLES_SHIFT)
		mov	ea, BC
		call	#rd_port_alu
		andn	DE, #$ff
		or	DE, alu
:end

'*******************************************************
' ed 60 Input H from port (BC)
'
z_inp_h_bc	long	(:end - $ - 1) | (12 << CYCLES_SHIFT)
		mov	ea, BC
		call	#rd_port_alu
		and	HL, #$ff
		shl	alu, #8
		or	HL, alu
:end

'*******************************************************
' ed 68 Input L from port (BC)
'
z_inp_l_bc	long	(:end - $ - 1) | (12 << CYCLES_SHIFT)
		mov	ea, BC
		call	#rd_port_alu
		andn	HL, #$ff
		or	HL, alu
:end

'*******************************************************
' ed 70 Input from port (BC)
'
z_inp_0_bc	long	(:end - $ - 1) | (12 << CYCLES_SHIFT)
		mov	ea, BC
		call	#rd_port_alu
:end

'*******************************************************
' ed 78 Input A from port (BC)
'
z_inp_a_bc	long	(:end - $ - 1) | (12 << CYCLES_SHIFT)
		mov	ea, BC
		call	#rd_port_alu
		and	AF, #$ff
		shl	alu, #8
		or	AF, alu
:end

'*******************************************************
' ed 41 Output B to port (BC)
'
z_out_bc_b	long	(:end - $ - 1) | (12 << CYCLES_SHIFT)
		mov	ea, BC
		mov	alu, BC
		shr	alu, #8		' isolate B
		call	#wr_port_alu
:end

'*******************************************************
' ed 49 Output C to port (BC)
'
z_out_bc_c	long	(:end - $ - 1) | (12 << CYCLES_SHIFT)
		mov	ea, BC
		mov	alu, BC
		and	alu, #$ff	' isolate C
		call	#wr_port_alu
:end

'*******************************************************
' ed 51 Output B to port (BC)
'
z_out_bc_d	long	(:end - $ - 1) | (12 << CYCLES_SHIFT)
		mov	ea, BC
		mov	alu, DE
		shr	alu, #8		' isolate D
		call	#wr_port_alu
:end

'*******************************************************
' ed 59 Output C to port (BC)
'
z_out_bc_e	long	(:end - $ - 1) | (12 << CYCLES_SHIFT)
		mov	ea, BC
		mov	alu, DE
		and	alu, #$ff	' isolate E
		call	#wr_port_alu
:end

'*******************************************************
' ed 61 Output H to port (BC)
'
z_out_bc_h	long	(:end - $ - 1) | (12 << CYCLES_SHIFT)
		mov	ea, BC
		mov	alu, HL
		shr	alu, #8		' isolate H
		call	#wr_port_alu
:end

'*******************************************************
' ed 69 Output L to port (BC)
'
z_out_bc_l	long	(:end - $ - 1) | (12 << CYCLES_SHIFT)
		mov	ea, BC
		mov	alu, HL
		and	alu, #$ff	' isolate L
		call	#wr_port_alu
:end

'*******************************************************
' ed 71 Output 0 to port (BC)
'
z_out_bc_0	long	(:end - $ - 1) | (12 << CYCLES_SHIFT)
		mov	ea, BC
		mov	alu, #0
		call	#wr_port_alu
:end

'*******************************************************
' ed 79 Output A to port (BC)
'
z_out_bc_a	long	(:end - $ - 1) | (12 << CYCLES_SHIFT)
		mov	ea, BC
		mov	alu, AF
		shr	alu, #8		' isolate A
		call	#wr_port_alu
:end

'*******************************************************
' ed 42 Subtract with carry BC from HL
'
z_sbc_hl_bc	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	tmp, BC
		long	@alu_sbc16
:end

'*******************************************************
' ed 4a Add with carry BC to HL
'
z_adc_hl_bc	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	tmp, BC
		long	@alu_adc16
:end

'*******************************************************
' ed 52 Subtract with carry DE from HL
'
z_sbc_hl_de	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	tmp, DE
		long	@alu_sbc16
:end

'*******************************************************
' ed 5a Add with carry DE to HL
'
z_adc_hl_de	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	tmp, DE
		long	@alu_adc16
:end

'*******************************************************
' ed 62 Subtract with carry HL from HL
'
z_sbc_hl_hl	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	tmp, HL
		long	@alu_sbc16
:end

'*******************************************************
' ed 6a Add with carry HL to HL
'
z_adc_hl_hl	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	tmp, HL
		long	@alu_adc16
:end

'*******************************************************
' ed 72 Subtract with carry SP from HL
'
z_sbc_hl_sp	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	tmp, SP
		long	@alu_sbc16
:end

'*******************************************************
' ed 7a Add with carry SP to HL
'
z_adc_hl_sp	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	tmp, SP
		long	@alu_adc16
:end

'*******************************************************
' ed 43 Load memory (imm16) with BC
'
z_ld_mem_bc	long	(:end - $ - 1) | (20 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		mov	alu, BC		' store BC
		call	#wr_memw_alu
:end

'*******************************************************
' ed 4b Load BC from memory (imm16)
'
z_ld_bc_mem	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		call	#rd_memw_alu
		mov	BC, alu		' to BC
:end

'*******************************************************
' ed 53 Load memory (imm16) with DE
'
z_ld_mem_de	long	(:end - $ - 1) | (20 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		mov	alu, DE		' store DE
		call	#wr_memw_alu
:end

'*******************************************************
' ed 5b Load DE from memory (imm16)
'
z_ld_de_mem	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		call	#rd_memw_alu
		mov	DE, alu		' to DE
:end

'*******************************************************
' ed 63 Load memory (imm16) with HL
'
z_ld_mem_hl_2	long	(:end - $ - 1) | (20 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		mov	alu, HL		' store HL
		call	#wr_memw_alu
:end

'*******************************************************
' ed 6b Load HL from memory (imm16)
'
z_ld_hl_mem_2	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		call	#rd_memw_alu
		mov	HL, alu		' to HL
:end

'*******************************************************
' ed 73 Load memory (imm16) with SP
'
z_ld_mem_sp	long	(:end - $ - 1) | (20 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		mov	alu, SP		' store SP
		call	#wr_memw_alu
:end

'*******************************************************
' ed 7b Load SP from memory (imm16)
'
z_ld_sp_mem	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	ea, 0-0		' source is patched with the address of imm16
		call	#rd_memw_alu
		mov	SP, alu		' to SP
:end

'*******************************************************
' ed 44 Negate accu
'
z_neg		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	alu, #0
		mov	tmp, AF
		shr	tmp, #8
		long	@alu_sub
:end

'*******************************************************
' ed 45 Return from non maskable interrupt
'
z_retn		long	(:end - $ - 1) | (14 << CYCLES_SHIFT)
		jmp	#handle_ret
:end

'*******************************************************
' ed 5d Return from interrupt
'
z_reti		long	(:end - $ - 1) | (14 << CYCLES_SHIFT)
		jmp	#handle_ret
:end

'*******************************************************
' ed 46 Set interrupt mode 0
'
z_im_0		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
:end

'*******************************************************
' ed 56 Set interrupt mode 1
'
z_im_1		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
:end

'*******************************************************
' ed 5e Set interrupt mode 2
'
z_im_2		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
:end

'*******************************************************
' ed 47 Load I with A
'
z_ld_i_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	tmp, AF
		shr	tmp, #8
		mov	I, tmp
:end

'*******************************************************
' ed 4f Load R with A
'
z_ld_r_a	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	tmp, AF
		shr	tmp, #8
		mov	R, tmp
		and	tmp, #$80
		mov	R2, tmp
:end

'*******************************************************
' ed 57 Load A with I
'
z_ld_a_i	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	tmp, I
		shl	tmp, #8
		and	AF, #$ff
		or	AF, tmp
		' FIXME: flags handling
:end

'*******************************************************
' ed 5f Load A with R
'
z_ld_a_r	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	tmp, R
		and	tmp, #$7f
		or	tmp, R2
		shl	tmp, #8
		and	AF, #$ff
		or	AF, tmp
		' FIXME: flags handling
:end

'*******************************************************
' ed 67 Rotate right digit memory (HL)
'
' A:[7][6][5][4][3][2][1][0] M:[7][6][5][4][3][2][1][0]
' before         x  x  x  x     y  y  y  y  z  z  z  z
' after          z  z  z  z     x  x  x  x  y  y  y  y
'
z_rrd_m		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		mov	tmp, alu
		shr	tmp, #4
		mov	aux, AF
		shr	aux, #4
		and	aux, #$f0
		or	tmp, aux
		and	alu, #$0f
		ror	AF, #8
		andn	AF, #$0f
		or	AF, alu
		rol	AF, #8
		mov	alu, tmp
		call	#wr_memb_alu		
:end

'*******************************************************
' ed 6f Rotate left digit memory (HL)
'
' A:[7][6][5][4][3][2][1][0] M:[7][6][5][4][3][2][1][0]
' before         x  x  x  x     y  y  y  y  z  z  z  z
' after          y  y  y  y     z  z  z  z  x  x  x  x
'
z_rld_m		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		mov	tmp, alu
		shl	tmp, #4
		mov	aux, AF
		shr	aux, #8
		and	aux, #$0f
		or	tmp, aux
		shr	alu, #4
		ror	AF, #8
		andn	AF, #$0f
		or	AF, alu
		rol	AF, #8
		mov	alu, tmp
		and	alu, #$ff
		call	#wr_memb_alu		
:end

'*******************************************************
' ed xx Illegal ED opcode
'
z_illegal	long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		' FIXME: do anything?
:end

'*******************************************************
' ed a0 Load memory (DE) with memory (HL) and increment
'
z_ldi		long	(:end - $ - 1) | (16 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		call	#wr_memb_de_alu
		add	HL, #1
		and	HL, low_16bits
		add	DE, #1
		and	DE, low_16bits
		sub	BC, #1 WZ
		and	BC, low_16bits
		muxnz	AF, #p_flag
:end

'*******************************************************
' ed a8 Load memory (DE) with memory (HL) and decrement
'
z_ldd		long	(:end - $ - 1) | (16 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		call	#wr_memb_de_alu
		call	#wr_memb_alu
		sub	HL, #1
		and	HL, low_16bits
		sub	DE, #1
		and	DE, low_16bits
		sub	BC, #1 WZ
		and	BC, low_16bits
		muxnz	AF, #p_flag
:end

'*******************************************************
' ed b0 Load memory (DE) with memory (HL), increment
' and repeat until BC == 0
'
z_ldir		long	(:end - $ - 1) | (16 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		call	#wr_memb_de_alu
		call	#wr_memb_alu
		add	HL, #1
		and	HL, low_16bits
		add	DE, #1
		and	DE, low_16bits
		sub	BC, #1 WZ
		and	BC, low_16bits
		muxnz	AF, #p_flag
		' FIXME: how to loop?
:end

'*******************************************************
' ed b8 Load memory (DE) with memory (HL), decrement
' and repeat until BC == 0
'
z_lddr		long	(:end - $ - 1) | (16 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		call	#wr_memb_de_alu
		call	#wr_memb_alu
		sub	HL, #1
		and	HL, low_16bits
		sub	DE, #1
		and	DE, low_16bits
		sub	BC, #1 WZ
		and	BC, low_16bits
		muxnz	AF, #p_flag
		' FIXME: how to loop?
:end

'*******************************************************
' ed a1 Compare A with memory (HL) and increment
'
z_cpi		long	(:end - $ - 1) | (16 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		mov	tmp, AF
		shr	tmp, #8
		mov	aux, alu
		sub	alu, tmp WZ, WC, NR
		muxc	AF, #s_flag
		muxz	AF, #z_flag
		xor	aux, alu
		xor	aux, tmp
		and	aux, #h_flag
		or	AF, aux
		add	HL, #1
		and	HL, low_16bits
		sub	BC, #1 WZ
		and	BC, low_16bits
		muxnz	AF, #p_flag
:end

'*******************************************************
' ed a9 Compare A with memory (HL) and decrement
'
z_cpd		long	(:end - $ - 1) | (16 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		mov	tmp, AF
		shr	tmp, #8
		mov	aux, alu
		sub	alu, tmp WZ, WC, NR
		muxc	AF, #s_flag
		muxz	AF, #z_flag
		xor	aux, alu
		xor	aux, tmp
		and	aux, #h_flag
		or	AF, aux
		sub	HL, #1
		and	HL, low_16bits
		sub	BC, #1 WZ
		and	BC, low_16bits
		muxnz	AF, #p_flag
:end

'*******************************************************
' ed b1 Compare A with memory (HL) and increment
' and repeat until Z flag set or BC == 0
'
z_cpir		long	(:end - $ - 1) | (16 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		mov	tmp, AF
		shr	tmp, #8
		mov	aux, alu
		sub	alu, tmp WZ, WC, NR
		muxc	AF, #s_flag
		muxz	AF, #z_flag
		xor	aux, alu
		xor	aux, tmp
		and	aux, #h_flag
		or	AF, aux
		add	HL, #1
		and	HL, low_16bits
		sub	BC, #1 WZ
		and	BC, low_16bits
		muxnz	AF, #p_flag
		' FIXME: how to loop?
:end

'*******************************************************
' ed a9 Compare A with memory (HL) and decrement
' and repeat until Z flag set or BC == 0
'
z_cpdr		long	(:end - $ - 1) | (16 << CYCLES_SHIFT)
		call	#rd_memb_hl_alu
		mov	tmp, AF
		shr	tmp, #8
		mov	aux, alu
		sub	alu, tmp WZ, WC, NR
		muxc	AF, #s_flag
		muxz	AF, #z_flag
		xor	aux, alu
		xor	aux, tmp
		and	aux, #h_flag
		or	AF, aux
		sub	HL, #1
		and	HL, low_16bits
		sub	BC, #1 WZ
		and	BC, low_16bits
		muxnz	AF, #p_flag
		' FIXME: how to loop?
:end

snippets_ed
	{40}	long	@z_inp_b_bc
	{41}	long	@z_out_bc_b
	{42}	long	@z_sbc_hl_bc
	{43}	long	@z_ld_mem_bc	| (patch_imm16 << OP_FLAGS_SHIFT)
	{44}	long	@z_neg
	{45}	long	@z_retn		| (terminate << OP_FLAGS_SHIFT)
	{46}	long	@z_im_0
	{47}	long	@z_ld_i_a
	{48}	long	@z_inp_c_bc
	{49}	long	@z_out_bc_c
	{4a}	long	@z_adc_hl_bc
	{4b}	long	@z_ld_bc_mem	| (patch_imm16 << OP_FLAGS_SHIFT)
	{4c}	long	@z_neg
	{4d}	long	@z_reti		| (terminate << OP_FLAGS_SHIFT)
	{4e}	long	@z_im_0
	{4f}	long	@z_ld_r_a
	{50}	long	@z_inp_d_bc
	{51}	long	@z_out_bc_d
	{52}	long	@z_sbc_hl_de
	{53}	long	@z_ld_mem_de	| (patch_imm16 << OP_FLAGS_SHIFT)
	{54}	long	@z_neg
	{55}	long	@z_retn		| (terminate << OP_FLAGS_SHIFT)
	{56}	long	@z_im_1
	{57}	long	@z_ld_a_i
	{58}	long	@z_inp_e_bc
	{59}	long	@z_out_bc_e
	{5a}	long	@z_adc_hl_de
	{5b}	long	@z_ld_de_mem	| (patch_imm16 << OP_FLAGS_SHIFT)
	{5c}	long	@z_neg
	{5d}	long	@z_reti		| (terminate << OP_FLAGS_SHIFT)
	{5e}	long	@z_im_2
	{5f}	long	@z_ld_a_r
	{60}	long	@z_inp_h_bc
	{61}	long	@z_out_bc_h
	{62}	long	@z_sbc_hl_hl
	{63}	long	@z_ld_mem_hl_2	| (patch_imm16 << OP_FLAGS_SHIFT)
	{64}	long	@z_neg
	{65}	long	@z_retn		| (terminate << OP_FLAGS_SHIFT)
	{66}	long	@z_im_0
	{67}	long	@z_rrd_m
	{68}	long	@z_inp_l_bc
	{69}	long	@z_out_bc_l
	{6a}	long	@z_adc_hl_hl
	{6b}	long	@z_ld_hl_mem_2	| (patch_imm16 << OP_FLAGS_SHIFT)
	{6c}	long	@z_neg
	{6d}	long	@z_reti		| (terminate << OP_FLAGS_SHIFT)
	{6e}	long	@z_im_0
	{6f}	long	@z_rld_m
	{70}	long	@z_inp_0_bc
	{71}	long	@z_out_bc_0
	{72}	long	@z_sbc_hl_sp
	{73}	long	@z_ld_mem_sp	| (patch_imm16 << OP_FLAGS_SHIFT)
	{74}	long	@z_neg
	{75}	long	@z_retn		| (terminate << OP_FLAGS_SHIFT)
	{76}	long	@z_im_1
	{77}	long	@z_illegal
	{78}	long	@z_inp_a_bc
	{79}	long	@z_out_bc_a
	{7a}	long	@z_adc_hl_sp
	{7b}	long	@z_ld_sp_mem	| (patch_imm16 << OP_FLAGS_SHIFT)
	{7c}	long	@z_neg
	{7d}	long	@z_reti		| (terminate << OP_FLAGS_SHIFT)
	{7e}	long	@z_im_2
	{7f}	long	@z_illegal
	{80}	long	@z_illegal
	{81}	long	@z_illegal
	{82}	long	@z_illegal
	{83}	long	@z_illegal
	{84}	long	@z_illegal
	{85}	long	@z_illegal
	{86}	long	@z_illegal
	{87}	long	@z_illegal
	{88}	long	@z_illegal
	{89}	long	@z_illegal
	{8a}	long	@z_illegal
	{8b}	long	@z_illegal
	{8c}	long	@z_illegal
	{8d}	long	@z_illegal
	{8e}	long	@z_illegal
	{8f}	long	@z_illegal
	{90}	long	@z_illegal
	{91}	long	@z_illegal
	{92}	long	@z_illegal
	{93}	long	@z_illegal
	{94}	long	@z_illegal
	{95}	long	@z_illegal
	{96}	long	@z_illegal
	{97}	long	@z_illegal
	{98}	long	@z_illegal
	{99}	long	@z_illegal
	{9a}	long	@z_illegal
	{9b}	long	@z_illegal
	{9c}	long	@z_illegal
	{9d}	long	@z_illegal
	{9e}	long	@z_illegal
	{9f}	long	@z_illegal
	{a0}	long	@z_ldi
	{a1}	long	@z_cpi
	{a2}	long	@z_illegal	' z_ini
	{a3}	long	@z_illegal	' z_outi
	{a4}	long	@z_illegal
	{a5}	long	@z_illegal
	{a6}	long	@z_illegal
	{a7}	long	@z_illegal
	{a8}	long	@z_ldd
	{a9}	long	@z_cpd
	{aa}	long	@z_illegal	' z_ind
	{ab}	long	@z_illegal	' z_outd
	{ac}	long	@z_illegal
	{ad}	long	@z_illegal
	{ae}	long	@z_illegal
	{af}	long	@z_illegal
	{b0}	long	@z_ldir
	{b1}	long	@z_cpir
	{b2}	long	@z_illegal	' z_inir
	{b3}	long	@z_illegal	' z_otir
	{b4}	long	@z_illegal
	{b5}	long	@z_illegal
	{b6}	long	@z_illegal
	{b7}	long	@z_illegal
	{b8}	long	@z_lddr
	{b9}	long	@z_cpdr
	{ba}	long	@z_illegal	' z_indr
	{bb}	long	@z_illegal	' z_otdr
	{bc}	long	@z_illegal
	{bd}	long	@z_illegal
	{be}	long	@z_illegal
	{bf}	long	@z_illegal

'******************************************************************************
'******************************************************************************
'
'	DD xx opcodes
'
'******************************************************************************
'******************************************************************************

'*******************************************************
' Build the address IX + disp8 in ea; ea contains disp8
'
mem_ix		long	(:end - $ - 1)
		shl	ea, #24
		sar	ea, #24
		add	ea, IX
		and	ea, low_16bits
:end

'*******************************************************
' dd 09 Add BC to IX
'
z_add_ix_bc	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	alu, IX
		mov	tmp, BC
		long	@alu_add16	' paste the add16 code snippet here
		mov	IX, alu
:end

'*******************************************************
' dd 19 Add DE to IX
'
z_add_ix_de	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	alu, IX
		mov	tmp, DE
		long	@alu_add16	' paste the add16 code snippet here
		mov	IX, alu
:end

'*******************************************************
' dd 29 Add IX to IX
'
z_add_ix_ix	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	alu, IX
		mov	tmp, IX
		long	@alu_add16	' paste the add16 code snippet here
		mov	IX, alu
:end

'*******************************************************
' dd 39 Add SP to IX
'
z_add_ix_sp	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	alu, IX
		mov	tmp, SP
		long	@alu_add16	' paste the add16 code snippet here
		mov	IX, alu
:end

'*******************************************************
' 21 Load IX with imm16
'
z_ld_ix_imm16	long	(:end - $ - 1) | (14 << CYCLES_SHIFT)
		mov	IX, 0-0		' source is patched with the address of imm16
:end

'*******************************************************
' dd 22 Load memory (imm16) with IX
'
z_ld_mem_ix	long	(:end - $ - 1)
		mov	ea, 0-0		' source is patched with the address of imm16
		mov	alu, IX		' store IX
		call	#wr_memw_alu
:end

'*******************************************************
' 2a Load IX from memory (imm16)
'
z_ld_ix_mem	long	(:end - $ - 1)
		mov	ea, 0-0		' source is patched with the address of imm16
		call	#rd_memw_alu
		mov	IX, alu		' to IX
:end

'*******************************************************
' dd 23 Increment IX
'
z_inc_ix	long	(:end - $ - 1)
		add	IX, #1
		and	IX, low_16bits
:end

'*******************************************************
' dd 2b Decrement IX
'
z_dec_ix	long	(:end - $ - 1)
		sub	IX, #1
		and	IX, low_16bits
:end


'*******************************************************
' dd 24 Increment HX
'
z_inc_hx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	alu, IX
		shr	alu, #8		' isolate HX
		and	IX, #$ff	' clear HX in IX
		long	@alu_inc
		shl	alu, #8
		or	IX, alu		' paste alu in HX
:end


'*******************************************************
' dd 2c Increment LX
'
z_inc_lx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	alu, IX
		and	alu, #$ff	' isolate LY
		andn	IX, #$ff	' clear LY in IX
		long	@alu_inc
		or	IX, alu		' paste alu in LY
:end


'*******************************************************
' dd 25 Decrement HX
'
z_dec_hx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	alu, IX
		shr	alu, #8		' isolate HX
		and	IX, #$ff	' clear HX in IX
		long	@alu_dec
		shl	alu, #8
		or	IX, alu		' paste alu in HX
:end

'*******************************************************
' dd 2d Decrement LX
'
z_dec_lx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	alu, IX
		and	alu, #$ff	' isolate LY
		andn	IX, #$ff	' clear LY in IX
		long	@alu_dec
		or	IX, alu		' paste alu in LY
:end


'*******************************************************
' dd 26 Load HX with imm8
'
z_ld_hx_imm8	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		ror	IX, #8
		movs	IX, #0		' source is patched with the imm8
		rol	IX, #8
:end

'*******************************************************
' dd 2e Load LX with imm8
'
z_ld_lx_imm8	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		andn	IX, #$ff	' clear LX
		or	IX, #0		' source is patched with the imm8
:end

'*******************************************************
' dd 34 Increment memory (IX+offs)
'
z_inc_mx	long	(:end - $ - 1) | (23 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		call	#rd_memb_alu
		long	@alu_inc
		call	#wr_memb_alu
:end

'*******************************************************
' dd 35 Decrement memory (IX+offs)
'
z_dec_mx	long	(:end - $ - 1) | (23 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		call	#rd_memb_alu
		long	@alu_dec
		call	#wr_memb_alu
:end

'*******************************************************
' dd 36 Load memory (IX+offs) with imm8
'
z_ld_mx_imm8	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		mov	alu, #0		' source is patched with the imm8
		long	@mem_ix		' paste the mem_ix code snippet here
		call	#wr_memb_alu
:end

'*******************************************************
' dd 44 Load B with HX
'
z_ld_b_hx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		and	BC, #$ff	' keep C, clear B
		andn	tmp, #$ff	' mask HX
		or	BC, tmp
:end

'*******************************************************
' dd 45 Load B with LX
'
z_ld_b_lx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		and	BC, #$ff	' keep C, clear B
		and	tmp, #$ff	' mask LY
		shl	tmp, #8
		or	BC, tmp
:end

'*******************************************************
' dd 46 Load B with memory (IX+disp8)
'
z_ld_b_mx	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		call	#rd_memb_alu
		and	BC, #$ff	' keep C, clear B
		shl	alu, #8
		or	BC, alu
:end

'*******************************************************
' dd 4c Load C with HX
'
z_ld_c_hx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		andn	BC, #$ff	' keep B, clear C
		shr	tmp, #8		' mask HX
		or	BC, tmp
:end

'*******************************************************
' dd 4d Load C with LX
'
z_ld_c_lx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		andn	BC, #$ff	' keep B, clear C
		and	tmp, #$ff	' mask LY
		or	BC, tmp
:end

'*******************************************************
' dd 4e Load C with memory (IX+disp8)
'
z_ld_c_mx	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		call	#rd_memb_alu
		andn	BC, #$ff	' keep B, clear C
		or	BC, alu
:end

'*******************************************************
' dd 54 Load D with HX
'
z_ld_d_hx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		and	DE, #$ff	' keep E, clear D
		andn	tmp, #$ff	' mask HX
		or	DE, tmp
:end

'*******************************************************
' dd 55 Load D with LX
'
z_ld_d_lx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		and	DE, #$ff	' keep E, clear D
		and	tmp, #$ff	' mask LY
		shl	tmp, #8
		or	DE, tmp
:end

'*******************************************************
' dd 56 Load D with memory (IX+disp8)
'
z_ld_d_mx	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		call	#rd_memb_alu
		and	DE, #$ff	' keep E, clear D
		shl	alu, #8
		or	DE, alu
:end

'*******************************************************
' dd 5c Load E with HX
'
z_ld_e_hx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		andn	DE, #$ff	' keep D, clear E
		shr	tmp, #8		' mask HX
		or	DE, tmp
:end

'*******************************************************
' dd 5d Load E with LX
'
z_ld_e_lx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		andn	DE, #$ff	' keep D, clear E
		and	tmp, #$ff	' mask LY
		or	DE, tmp
:end

'*******************************************************
' dd 5e Load E with memory (IX+disp8)
'
z_ld_e_mx	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		call	#rd_memb_alu
		andn	DE, #$ff	' keep D, clear E
		or	DE, alu
:end


'*******************************************************
' dd 60 Load HX with B
'
z_ld_hx_b	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, BC
		and	IX, #$ff	' keep LY, clear HX
		andn	tmp, #$ff	' mask B
		or	IX, tmp
:end

'*******************************************************
' dd 61 Load HX with C
'
z_ld_hx_c	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, BC
		and	IX, #$ff	' keep LY, clear HX
		and	tmp, #$ff	' mask C
		shl	tmp, #8
		or	IX, tmp
:end

'*******************************************************
' dd 62 Load HX with D
'
z_ld_hx_d	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, DE
		and	IX, #$ff	' keep LX, clear HX
		andn	tmp, #$ff	' mask D
		or	IX, tmp
:end

'*******************************************************
' dd 63 Load HX with E
'
z_ld_hx_e	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, DE
		and	IX, #$ff	' keep LX, clear HX
		and	tmp, #$ff	' mask E
		shl	tmp, #8
		or	IX, tmp
:end

'*******************************************************
' dd 64 Load HX with HX (NOP)
'
z_ld_hx_hx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
:end

'*******************************************************
' dd 65 Load HX with LX
'
z_ld_hx_lx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		and	IX, #$ff	' keep LX, clear HX
		and	tmp, #$ff	' mask LX
		shl	tmp, #8
		or	IX, tmp
:end

'*******************************************************
' dd 66 Load H with memory (IX+disp8)
'
z_ld_h_mx	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		call	#rd_memb_alu
		and	HL, #$ff	' keep L, clear H
		shl	alu, #8
		or	HL, alu
:end

'*******************************************************
' dd 67 Load HX with A
'
z_ld_hx_a	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, AF
		and	IX, #$ff	' keep LX, clear HX
		andn	tmp, #$ff	' mask A
		or	IX, tmp
:end

'*******************************************************
' dd 68 Load LX with B
'
z_ld_lx_b	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, BC
		andn	IX, #$ff	' keep HX, clear LX
		shr	tmp, #8		' mask B
		or	IX, tmp
:end

'*******************************************************
' dd 69 Load LX with C
'
z_ld_lx_c	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, BC
		andn	IX, #$ff	' keep HX, clear LX
		and	tmp, #$ff	' mask C
		or	IX, tmp
:end

'*******************************************************
' dd 6a Load LX with D
'
z_ld_lx_d	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, DE
		andn	IX, #$ff	' keep HX, clear LX
		shr	tmp, #8		' mask D
		or	IX, tmp
:end

'*******************************************************
' dd 6b Load LX with E
'
z_ld_lx_e	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, DE
		andn	IX, #$ff	' keep HX, clear LX
		and	tmp, #$ff	' mask E
		or	IX, tmp
:end

'*******************************************************
' dd 6c Load LX with HX
'
z_ld_lx_hx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		andn	IX, #$ff	' keep HX, clear LX
		shr	tmp, #8		' mask HX
		or	IX, tmp
:end

'*******************************************************
' dd 6d Load LX with LX (NOP)
'
z_ld_lx_lx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
:end

'*******************************************************
' dd 6e Load L with memory (IX+disp8)
'
z_ld_l_mx	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		call	#rd_memb_alu
		andn	HL, #$ff	' keep H, clear L
		or	HL, alu
:end

'*******************************************************
' dd 6f Load L with A
'
z_ld_lx_a	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, AF
		andn	IX, #$ff	' keep HX, clear LX
		shr	tmp, #8		' mask A
		or	IX, tmp
:end

'*******************************************************
' dd 70 Load memory (IX+disp8) with B
'
z_ld_mx_b	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		mov	alu, BC
		shr	alu, #8		' mask B
		call	#wr_memb_alu
:end

'*******************************************************
' dd 71 Load memory (IX+disp8) with C
'
z_ld_mx_c	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		mov	alu, BC
		and	alu, #$ff	' mask C
		call	#wr_memb_alu
:end

'*******************************************************
' dd 72 Load memory (IX+disp8) with D
'
z_ld_mx_d	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		mov	alu, DE
		shr	alu, #8		' mask D
		call	#wr_memb_alu
:end

'*******************************************************
' dd 73 Load memory (IX+disp8) with E
'
z_ld_mx_e	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		mov	alu, DE
		and	alu, #$ff	' mask E
		call	#wr_memb_alu
:end

'*******************************************************
' dd 74 Load memory (IX+disp8) with H
'
z_ld_mx_h	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		mov	alu, HL
		shr	alu, #8		' mask H
		call	#wr_memb_alu
:end

'*******************************************************
' dd 75 Load memory (IX+disp8) with L
'
z_ld_mx_l	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		mov	alu, HL
		and	alu, #$ff	' mask L
		call	#wr_memb_alu
:end

'*******************************************************
' dd 77 Load memory (IX+disp8) with A
'
z_ld_mx_a	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		mov	alu, AF
		shr	alu, #8		' mask A
		call	#wr_memb_alu
:end

'*******************************************************
' dd 7c Load A with HX
'
z_ld_a_hx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		and	AF, #$ff	' keep F, clear A
		andn	tmp, #$ff	' mask HX
		or	AF, tmp
:end

'*******************************************************
' dd 7d Load A with LX
'
z_ld_a_lx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		and	AF, #$ff	' keep F, clear A
		and	tmp, #$ff	' mask LX
		shl	tmp, #8
		or	AF, tmp
:end

'*******************************************************
' dd 7e Load A with memory (IX+disp8)
'
z_ld_a_mx	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		call	#rd_memb_alu
		and	AF, #$ff	' keep F, clear A
		shl	alu, #8
		or	AF, alu
:end

'*******************************************************
' dd 84 Add HX to A
'
z_add_a_hx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		shr	tmp, #8
		long	@alu_add
:end

'*******************************************************
' dd 85 Add LX to A
'
z_add_a_lx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		and	tmp, #$ff
		long	@alu_add
:end

'*******************************************************
' dd 86 Add memory (IX+disp8) to A
'
z_add_a_mx	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		call	#rd_memb_alu
		long	@alu_add
:end

'*******************************************************
' dd 8c Add with carry HX to A
'
z_adc_a_hx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		shr	tmp, #8
		long	@alu_adc
:end

'*******************************************************
' dd 8d Add with carry LX to A
'
z_adc_a_lx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		and	tmp, #$ff
		long	@alu_adc
:end

'*******************************************************
' dd 8e Add with carry memory (IX+disp8) to A
'
z_adc_a_mx	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		call	#rd_memb_alu
		long	@alu_adc
:end

'*******************************************************
' dd 94 Subtract HX from A 
'
z_sub_a_hx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		shr	tmp, #8
		long	@alu_sub
:end

'*******************************************************
' dd 95 Subtract LX from A 
'
z_sub_a_lx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		and	tmp, #$ff
		long	@alu_sub
:end

'*******************************************************
' dd 96 Subtract memory (IX+disp8) from A 
'
z_sub_a_mx	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		call	#rd_memb_alu
		long	@alu_sub
:end

'*******************************************************
' dd 9c Subtract with carry HX from A 
'
z_sbc_a_hx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		shr	tmp, #8
		long	@alu_sbc
:end

'*******************************************************
' dd 9d Subtract with carry LX from A 
'
z_sbc_a_lx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		and	tmp, #$ff
		long	@alu_sbc
:end

'*******************************************************
' dd 9e Subtract with carry memory (IX+disp8) from A 
'
z_sbc_a_mx	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		call	#rd_memb_alu
		long	@alu_sbc
:end

'*******************************************************
' dd a4 And A with HX
'
z_and_a_hx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		shr	tmp, #8
		long	@alu_and
:end

'*******************************************************
' dd a5 And A with LX
'
z_and_a_lx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		and	tmp, #$ff
		long	@alu_and
:end

'*******************************************************
' dd a6 And A with memory (IX+disp8)
'
z_and_a_mx	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		call	#rd_memb_alu
		long	@alu_and
:end

'*******************************************************
' dd ac Exclusive or A with HX
'
z_xor_a_hx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		shr	tmp, #8
		long	@alu_xor
:end

'*******************************************************
' dd ad Exclusive or A with LX
'
z_xor_a_lx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		and	tmp, #$ff
		long	@alu_xor
:end

'*******************************************************
' dd ae Exclusive or A with memory (IX+disp8)
'
z_xor_a_mx	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		call	#rd_memb_alu
		long	@alu_xor
:end

'*******************************************************
' dd b4 Or A with HX
'
z_or_a_hx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		shr	tmp, #8
		long	@alu_or
:end

'*******************************************************
' dd b5 Or A with LX
'
z_or_a_lx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		and	tmp, #$ff
		long	@alu_or
:end

'*******************************************************
' dd b6 Or A with memory (IX+disp8)
'
z_or_a_mx	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		call	#rd_memb_alu
		long	@alu_or
:end

'*******************************************************
' dd bc Compare A with HX
'
z_cp_a_hx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		shr	tmp, #8
		long	@alu_cp
:end

'*******************************************************
' dd bd Compare A with LX
'
z_cp_a_lx	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IX
		and	tmp, #$ff
		long	@alu_cp
:end

'*******************************************************
' dd be Compare A with memory (IX+disp8)
'
z_cp_a_mx	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IX offset
		long	@mem_ix		' paste the mem_ix code snippet here
		call	#rd_memb_alu
		long	@alu_cp
:end

'*******************************************************
' dd e1 Pop IX off the stack
'
z_pop_ix	long	(:end - $ - 1) | (14 << CYCLES_SHIFT)
		call	#pop_alu
		mov	IX, alu
:end

'*******************************************************
' dd e3 Exchange stack contents with IX
'
z_ex_sp_ix	long	(:end - $ - 1) | (23 << CYCLES_SHIFT)
		call	#pop_alu
		xor	IX, alu
		xor	alu, IX
		xor	IX, alu
		call	#push_alu
:end

'*******************************************************
' dd e5 Push IX to the stack
'
z_push_ix	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	alu, IX
		call	#push_alu
:end

'*******************************************************
' dd e9 Jump to address in IX
'
z_jp_ix		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	PC, IX
		jmp	#compile
:end

'*******************************************************
' dd f9 Load SP with IX
'
z_ld_sp_ix	long	(:end - $ - 1)
		mov	SP, IX
:end

snippets_dd
	{00}	long	@z_nop
	{01}	long	@z_ld_bc_imm16	| (patch_imm16 << OP_FLAGS_SHIFT)
	{02}	long	@z_ld_mem_bc_a
	{03}	long	@z_inc_bc
	{04}	long	@z_inc_b
	{05}	long	@z_dec_b
	{06}	long	@z_ld_b_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{07}	long	@z_rlca
	{08}	long	@z_ex_af_af2
	{09}	long	@z_add_ix_bc
	{0a}	long	@z_ld_a_mem_bc
	{0b}	long	@z_dec_bc
	{0c}	long	@z_inc_c
	{0d}	long	@z_dec_c
	{0e}	long	@z_ld_c_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{0f}	long	@z_rrca
	{10}	long	@z_djnz		| (patch_jr_disp8 << OP_FLAGS_SHIFT)
	{11}	long	@z_ld_de_imm16	| (patch_imm16 << OP_FLAGS_SHIFT)
	{12}	long	@z_ld_mem_de_a
	{13}	long	@z_inc_de
	{14}	long	@z_inc_d
	{15}	long	@z_dec_d
	{16}	long	@z_ld_d_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{17}	long	@z_rla
	{18}	long	@z_jr		| (patch_jr_disp8 << OP_FLAGS_SHIFT)
	{19}	long	@z_add_ix_de
	{1a}	long	@z_ld_a_mem_de
	{1b}	long	@z_dec_de
	{1c}	long	@z_inc_e
	{1d}	long	@z_dec_e
	{1e}	long	@z_ld_e_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{1f}	long	@z_rra
	{20}	long	@z_jr_nz	| (patch_jr_disp8 << OP_FLAGS_SHIFT)
	{21}	long	@z_ld_ix_imm16	| (patch_imm16 << OP_FLAGS_SHIFT)
	{22}	long	@z_ld_mem_ix	| (patch_imm16 << OP_FLAGS_SHIFT)
	{23}	long	@z_inc_ix
	{24}	long	@z_inc_hx
	{25}	long	@z_dec_hx
	{26}	long	@z_ld_hx_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{27}	long	@z_daa
	{28}	long	@z_jr_z		| (patch_jr_disp8 << OP_FLAGS_SHIFT)
	{29}	long	@z_add_ix_ix
	{2a}	long	@z_ld_ix_mem	| (patch_imm16 << OP_FLAGS_SHIFT)
	{2b}	long	@z_dec_ix
	{2c}	long	@z_inc_lx
	{2d}	long	@z_dec_lx
	{2e}	long	@z_ld_lx_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{2f}	long	@z_cpl
	{30}	long	@z_jr_nc	| (patch_jr_disp8 << OP_FLAGS_SHIFT)
	{31}	long	@z_ld_sp_imm16	| (patch_imm16 << OP_FLAGS_SHIFT)
	{32}	long	@z_ld_mem_a	| (patch_imm16 << OP_FLAGS_SHIFT)
	{33}	long	@z_inc_sp
	{34}	long	@z_inc_mx	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{35}	long	@z_dec_mx	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{36}	long	@z_ld_mx_imm8	| ((patch_xy_disp8 | patch_imm8) << OP_FLAGS_SHIFT)
	{37}	long	@z_scf
	{38}	long	@z_jr_c		| (patch_jr_disp8 << OP_FLAGS_SHIFT)
	{39}	long	@z_add_ix_sp
	{3a}	long	@z_ld_a_mem	| (patch_imm16 << OP_FLAGS_SHIFT)
	{3b}	long	@z_dec_sp
	{3c}	long	@z_inc_a
	{3d}	long	@z_dec_a
	{3e}	long	@z_ld_a_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{3f}	long	@z_ccf
	{40}	long	@z_ld_b_b
	{41}	long	@z_ld_b_c
	{42}	long	@z_ld_b_d
	{43}	long	@z_ld_b_e
	{44}	long	@z_ld_b_hx
	{45}	long	@z_ld_b_lx
	{46}	long	@z_ld_b_mx	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{47}	long	@z_ld_b_a
	{48}	long	@z_ld_c_b
	{49}	long	@z_ld_c_c
	{4a}	long	@z_ld_c_d
	{4b}	long	@z_ld_c_e
	{4c}	long	@z_ld_c_hx
	{4d}	long	@z_ld_c_lx
	{4e}	long	@z_ld_c_mx	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{4f}	long	@z_ld_c_a
	{50}	long	@z_ld_d_b
	{51}	long	@z_ld_d_c
	{52}	long	@z_ld_d_d
	{53}	long	@z_ld_d_e
	{54}	long	@z_ld_d_hx
	{55}	long	@z_ld_d_lx
	{56}	long	@z_ld_d_mx	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{57}	long	@z_ld_d_a
	{58}	long	@z_ld_e_b
	{59}	long	@z_ld_e_c
	{5a}	long	@z_ld_e_d
	{5b}	long	@z_ld_e_e
	{5c}	long	@z_ld_e_hx
	{5d}	long	@z_ld_e_lx
	{5e}	long	@z_ld_e_mx	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{5f}	long	@z_ld_e_a
	{60}	long	@z_ld_hx_b
	{61}	long	@z_ld_hx_c
	{62}	long	@z_ld_hx_d
	{63}	long	@z_ld_hx_e
	{64}	long	@z_ld_hx_hx
	{65}	long	@z_ld_hx_lx
	{66}	long	@z_ld_h_mx	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{67}	long	@z_ld_hx_a
	{68}	long	@z_ld_lx_b
	{69}	long	@z_ld_lx_c
	{6a}	long	@z_ld_lx_d
	{6b}	long	@z_ld_lx_e
	{6c}	long	@z_ld_lx_hx
	{6d}	long	@z_ld_lx_lx
	{6e}	long	@z_ld_l_mx	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{6f}	long	@z_ld_lx_a
	{70}	long	@z_ld_mx_b	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{71}	long	@z_ld_mx_c	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{72}	long	@z_ld_mx_d	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{73}	long	@z_ld_mx_e	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{74}	long	@z_ld_mx_h	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{75}	long	@z_ld_mx_l	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{76}	long	@z_halt		| (terminate << OP_FLAGS_SHIFT)
	{77}	long	@z_ld_mx_a	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{78}	long	@z_ld_a_b
	{79}	long	@z_ld_a_c
	{7a}	long	@z_ld_a_d
	{7b}	long	@z_ld_a_e
	{7c}	long	@z_ld_a_hx
	{7d}	long	@z_ld_a_lx
	{7e}	long	@z_ld_a_mx	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{7f}	long	@z_ld_a_a
	{80}	long	@z_add_a_b
	{81}	long	@z_add_a_c
	{82}	long	@z_add_a_d
	{83}	long	@z_add_a_e
	{84}	long	@z_add_a_hx
	{85}	long	@z_add_a_lx
	{86}	long	@z_add_a_mx	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{87}	long	@z_add_a_a
	{88}	long	@z_adc_a_b
	{89}	long	@z_adc_a_c
	{8a}	long	@z_adc_a_d
	{8b}	long	@z_adc_a_e
	{8c}	long	@z_adc_a_hx
	{8d}	long	@z_adc_a_lx
	{8e}	long	@z_adc_a_mx	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{8f}	long	@z_adc_a_a
	{90}	long	@z_sub_a_b
	{91}	long	@z_sub_a_c
	{92}	long	@z_sub_a_d
	{93}	long	@z_sub_a_e
	{94}	long	@z_sub_a_hx
	{95}	long	@z_sub_a_lx
	{96}	long	@z_sub_a_mx	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{97}	long	@z_sub_a_a
	{98}	long	@z_sbc_a_b
	{99}	long	@z_sbc_a_c
	{9a}	long	@z_sbc_a_d
	{9b}	long	@z_sbc_a_e
	{9c}	long	@z_sbc_a_hx
	{9d}	long	@z_sbc_a_lx
	{9e}	long	@z_sbc_a_mx	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{9f}	long	@z_sbc_a_a
	{a0}	long	@z_and_a_b
	{a1}	long	@z_and_a_c
	{a2}	long	@z_and_a_d
	{a3}	long	@z_and_a_e
	{a4}	long	@z_and_a_hx
	{a5}	long	@z_and_a_lx
	{a6}	long	@z_and_a_mx	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{a7}	long	@z_and_a_a
	{a8}	long	@z_xor_a_b
	{a9}	long	@z_xor_a_c
	{aa}	long	@z_xor_a_d
	{ab}	long	@z_xor_a_e
	{ac}	long	@z_xor_a_hx
	{ad}	long	@z_xor_a_lx
	{ae}	long	@z_xor_a_mx	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{af}	long	@z_xor_a_a
	{b0}	long	@z_or_a_b
	{b1}	long	@z_or_a_c
	{b2}	long	@z_or_a_d
	{b3}	long	@z_or_a_e
	{b4}	long	@z_or_a_hx
	{b5}	long	@z_or_a_lx
	{b6}	long	@z_or_a_mx	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{b7}	long	@z_or_a_a
	{b8}	long	@z_cp_a_b
	{b9}	long	@z_cp_a_c
	{ba}	long	@z_cp_a_d
	{bb}	long	@z_cp_a_e
	{bc}	long	@z_cp_a_hx
	{bd}	long	@z_cp_a_lx
	{be}	long	@z_cp_a_mx	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{bf}	long	@z_cp_a_a
	{c0}	long	@z_ret_nz
	{c1}	long	@z_pop_bc
	{c2}	long	@z_jp_nz	| (patch_imm16 << OP_FLAGS_SHIFT)
	{c3}	long	@z_jp		| ((patch_imm16 | terminate) << OP_FLAGS_SHIFT)
	{c4}	long	@z_call_nz	| (patch_imm16 << OP_FLAGS_SHIFT)
	{c5}	long	@z_push_bc
	{c6}	long	@z_add_a_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{c7}	long	@z_rst_00	| (terminate << OP_FLAGS_SHIFT)
	{c8}	long	@z_ret_z
	{c9}	long	@z_ret		| (terminate << OP_FLAGS_SHIFT)
	{ca}	long	@z_jp_z		| (patch_imm16 << OP_FLAGS_SHIFT)
	{cb}	long	@prefix_cb_xx	| (jump_to << OP_FLAGS_SHIFT)
	{cc}	long	@z_call_z	| (patch_imm16 << OP_FLAGS_SHIFT)
	{cd}	long	@z_call		| ((patch_imm16 | terminate) << OP_FLAGS_SHIFT)
	{ce}	long	@z_adc_a_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{c7}	long	@z_rst_08	| (terminate << OP_FLAGS_SHIFT)
	{d0}	long	@z_ret_nc
	{d1}	long	@z_pop_de
	{d2}	long	@z_jp_nc	| (patch_imm16 << OP_FLAGS_SHIFT)
	{d3}	long	@z_out_port_a
	{d4}	long	@z_call_nc	| (patch_imm16 << OP_FLAGS_SHIFT)
	{d5}	long	@z_push_de
	{d6}	long	@z_sub_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{d7}	long	@z_rst_10	| (terminate << OP_FLAGS_SHIFT)
	{d8}	long	@z_ret_c
	{d9}	long	@z_exx
	{da}	long	@z_jp_c		| (patch_imm16 << OP_FLAGS_SHIFT)
	{db}	long	@z_in_a_port
	{dc}	long	@z_call_c	| (patch_imm16 << OP_FLAGS_SHIFT)
	{dd}	long	@prefix_dd_xx	| (jump_to << OP_FLAGS_SHIFT)
	{de}	long	@z_sbc_a_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{df}	long	@z_rst_18	| (terminate << OP_FLAGS_SHIFT)
	{e0}	long	@z_ret_po
	{e1}	long	@z_pop_ix
	{e2}	long	@z_jp_po	| (patch_imm16 << OP_FLAGS_SHIFT)
	{e3}	long	@z_ex_sp_ix
	{e4}	long	@z_call_po	| (patch_imm16 << OP_FLAGS_SHIFT)
	{e5}	long	@z_push_ix
	{e6}	long	@z_and_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{e7}	long	@z_rst_20	| (terminate << OP_FLAGS_SHIFT)
	{e8}	long	@z_ret_pe
	{e9}	long	@z_jp_ix	| (terminate << OP_FLAGS_SHIFT)
	{ea}	long	@z_jp_pe	| (patch_imm16 << OP_FLAGS_SHIFT)
	{eb}	long	@z_ex_de_hl
	{ec}	long	@z_call_pe	| (patch_imm16 << OP_FLAGS_SHIFT)
	{ed}	long	@prefix_ed_xx	| (jump_to << OP_FLAGS_SHIFT)
	{ee}	long	@z_xor_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{ef}	long	@z_rst_28	| (terminate << OP_FLAGS_SHIFT)
	{f0}	long	@z_ret_p
	{f1}	long	@z_pop_af
	{f2}	long	@z_jp_p		| (patch_imm16 << OP_FLAGS_SHIFT)
	{f3}	long	@z_di
	{f4}	long	@z_call_p	| (patch_imm16 << OP_FLAGS_SHIFT)
	{f5}	long	@z_push_af
	{f6}	long	@z_or_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{f7}	long	@z_rst_30	| (terminate << OP_FLAGS_SHIFT)
	{f8}	long	@z_ret_m
	{f9}	long	@z_ld_sp_ix
	{fa}	long	@z_jp_m		| (patch_imm16 << OP_FLAGS_SHIFT)
	{fb}	long	@z_ei
	{fc}	long	@z_call_m	| (patch_imm16 << OP_FLAGS_SHIFT)
	{fd}	long	@prefix_fd_xx	| (jump_to << OP_FLAGS_SHIFT)
	{fe}	long	@z_cp_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{ff}	long	@z_rst_38	| (terminate << OP_FLAGS_SHIFT)

'******************************************************************************
'******************************************************************************
'
'	FD xx opcodes
'
'******************************************************************************
'******************************************************************************

'*******************************************************
' Build the address IY + disp8 in ea; ea contains disp8
'
mem_iy		long	(:end - $ - 1)
		shl	ea, #24
		sar	ea, #24
		add	ea, IY
		and	ea, low_16bits
:end

'*******************************************************
' fd 09 Add BC to IY
'
z_add_iy_bc	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	alu, IY
		mov	tmp, BC
		long	@alu_add16	' paste the add16 code snippet here
		mov	IY, alu
:end

'*******************************************************
' fd 19 Add DE to IY
'
z_add_iy_de	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	alu, IY
		mov	tmp, DE
		long	@alu_add16	' paste the add16 code snippet here
		mov	IY, alu
:end

'*******************************************************
' fd 29 Add IY to IY
'
z_add_iy_iy	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	alu, IY
		mov	tmp, IY
		long	@alu_add16	' paste the add16 code snippet here
		mov	IY, alu
:end

'*******************************************************
' fd 39 Add SP to IY
'
z_add_iy_sp	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	alu, IY
		mov	tmp, SP
		long	@alu_add16	' paste the add16 code snippet here
		mov	IY, alu
:end

'*******************************************************
' 21 Load IY with imm16
'
z_ld_iy_imm16	long	(:end - $ - 1) | (14 << CYCLES_SHIFT)
		mov	IY, 0-0		' source is patched with the address of imm16
:end

'*******************************************************
' fd 22 Load memory (imm16) with IY
'
z_ld_mem_iy	long	(:end - $ - 1)
		mov	ea, 0-0		' source is patched with the address of imm16
		mov	alu, IY		' store IY
		call	#wr_memw_alu
:end

'*******************************************************
' 2a Load IY from memory (imm16)
'
z_ld_iy_mem	long	(:end - $ - 1)
		mov	ea, 0-0		' source is patched with the address of imm16
		call	#rd_memw_alu
		mov	IY, alu		' to IY
:end

'*******************************************************
' fd 23 Increment IY
'
z_inc_iy	long	(:end - $ - 1)
		add	IY, #1
		and	IY, low_16bits
:end

'*******************************************************
' fd 2b Decrement IY
'
z_dec_iy	long	(:end - $ - 1)
		sub	IY, #1
		and	IY, low_16bits
:end


'*******************************************************
' fd 24 Increment HX
'
z_inc_hy	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	alu, IY
		shr	alu, #8		' isolate HX
		and	IY, #$ff	' clear HX in IY
		long	@alu_inc
		shl	alu, #8
		or	IY, alu		' paste alu in HX
:end


'*******************************************************
' fd 2c Increment LX
'
z_inc_ly	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	alu, IY
		and	alu, #$ff	' isolate LY
		andn	IY, #$ff	' clear LY in IY
		long	@alu_inc
		or	IY, alu		' paste alu in LY
:end


'*******************************************************
' fd 25 Decrement HX
'
z_dec_hy	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	alu, IY
		shr	alu, #8		' isolate HX
		and	IY, #$ff	' clear HX in IY
		long	@alu_dec
		shl	alu, #8
		or	IY, alu		' paste alu in HX
:end

'*******************************************************
' fd 2d Decrement LX
'
z_dec_ly	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	alu, IY
		and	alu, #$ff	' isolate LY
		andn	IY, #$ff	' clear LY in IY
		long	@alu_dec
		or	IY, alu		' paste alu in LY
:end


'*******************************************************
' fd 26 Load HX with imm8
'
z_ld_hy_imm8	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		ror	IY, #8
		movs	IY, #0		' source is patched with the imm8
		rol	IY, #8
:end

'*******************************************************
' fd 2e Load LX with imm8
'
z_ld_ly_imm8	long	(:end - $ - 1) | (11 << CYCLES_SHIFT)
		andn	IY, #$ff	' clear LY
		or	IY, #0		' source is patched with the imm8
:end

'*******************************************************
' fd 34 Increment memory (IY+offs)
'
z_inc_my	long	(:end - $ - 1) | (23 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		call	#rd_memb_alu
		long	@alu_inc
		call	#wr_memb_alu
:end

'*******************************************************
' fd 35 Decrement memory (IY+offs)
'
z_dec_my	long	(:end - $ - 1) | (23 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		call	#rd_memb_alu
		long	@alu_dec
		call	#wr_memb_alu
:end

'*******************************************************
' fd 36 Load memory (IY+offs) with imm8
'
z_ld_my_imm8	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		mov	alu, #0		' source is patched with the imm8
		long	@mem_iy		' paste the mem_iy code snippet here
		call	#wr_memb_alu
:end

'*******************************************************
' fd 44 Load B with HX
'
z_ld_b_hy	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		and	BC, #$ff	' keep C, clear B
		andn	tmp, #$ff	' mask HX
		or	BC, tmp
:end

'*******************************************************
' fd 45 Load B with LX
'
z_ld_b_ly	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		and	BC, #$ff	' keep C, clear B
		and	tmp, #$ff	' mask LY
		shl	tmp, #8
		or	BC, tmp
:end

'*******************************************************
' fd 46 Load B with memory (IY+disp8)
'
z_ld_b_my	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		call	#rd_memb_alu
		and	BC, #$ff	' keep C, clear B
		shl	alu, #8
		or	BC, alu
:end

'*******************************************************
' fd 4c Load C with HX
'
z_ld_c_hy	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		andn	BC, #$ff	' keep B, clear C
		shr	tmp, #8		' mask HX
		or	BC, tmp
:end

'*******************************************************
' fd 4d Load C with LX
'
z_ld_c_ly	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		andn	BC, #$ff	' keep B, clear C
		and	tmp, #$ff	' mask LY
		or	BC, tmp
:end

'*******************************************************
' fd 4e Load C with memory (IY+disp8)
'
z_ld_c_my	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		call	#rd_memb_alu
		andn	BC, #$ff	' keep B, clear C
		or	BC, alu
:end

'*******************************************************
' fd 54 Load D with HX
'
z_ld_d_hy	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		and	DE, #$ff	' keep E, clear D
		andn	tmp, #$ff	' mask HX
		or	DE, tmp
:end

'*******************************************************
' fd 55 Load D with LX
'
z_ld_d_ly	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		and	DE, #$ff	' keep E, clear D
		and	tmp, #$ff	' mask LY
		shl	tmp, #8
		or	DE, tmp
:end

'*******************************************************
' fd 56 Load D with memory (IY+disp8)
'
z_ld_d_my	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		call	#rd_memb_alu
		and	DE, #$ff	' keep E, clear D
		shl	alu, #8
		or	DE, alu
:end

'*******************************************************
' fd 5c Load E with HX
'
z_ld_e_hy	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		andn	DE, #$ff	' keep D, clear E
		shr	tmp, #8		' mask HX
		or	DE, tmp
:end

'*******************************************************
' fd 5d Load E with LX
'
z_ld_e_ly	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		andn	DE, #$ff	' keep D, clear E
		and	tmp, #$ff	' mask LY
		or	DE, tmp
:end

'*******************************************************
' fd 5e Load E with memory (IY+disp8)
'
z_ld_e_my	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		call	#rd_memb_alu
		andn	DE, #$ff	' keep D, clear E
		or	DE, alu
:end


'*******************************************************
' fd 60 Load HX with B
'
z_ld_hy_b	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, BC
		and	IY, #$ff	' keep LY, clear HX
		andn	tmp, #$ff	' mask B
		or	IY, tmp
:end

'*******************************************************
' fd 61 Load HX with C
'
z_ld_hy_c	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, BC
		and	IY, #$ff	' keep LY, clear HX
		and	tmp, #$ff	' mask C
		shl	tmp, #8
		or	IY, tmp
:end

'*******************************************************
' fd 62 Load HX with D
'
z_ld_hy_d	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, DE
		and	IY, #$ff	' keep LX, clear HX
		andn	tmp, #$ff	' mask D
		or	IY, tmp
:end

'*******************************************************
' fd 63 Load HX with E
'
z_ld_hy_e	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, DE
		and	IY, #$ff	' keep LX, clear HX
		and	tmp, #$ff	' mask E
		shl	tmp, #8
		or	IY, tmp
:end

'*******************************************************
' fd 64 Load HX with HX (NOP)
'
z_ld_hy_hy	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
:end

'*******************************************************
' fd 65 Load HX with LX
'
z_ld_hy_ly	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		and	IY, #$ff	' keep LX, clear HX
		and	tmp, #$ff	' mask LX
		shl	tmp, #8
		or	IY, tmp
:end

'*******************************************************
' fd 66 Load H with memory (IY+disp8)
'
z_ld_h_my	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		call	#rd_memb_alu
		and	HL, #$ff	' keep L, clear H
		shl	alu, #8
		or	HL, alu
:end

'*******************************************************
' fd 67 Load HX with A
'
z_ld_hy_a	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, AF
		and	IY, #$ff	' keep LX, clear HX
		andn	tmp, #$ff	' mask A
		or	IY, tmp
:end

'*******************************************************
' fd 68 Load LX with B
'
z_ld_ly_b	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, BC
		andn	IY, #$ff	' keep HX, clear LX
		shr	tmp, #8		' mask B
		or	IY, tmp
:end

'*******************************************************
' fd 69 Load LX with C
'
z_ld_ly_c	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, BC
		andn	IY, #$ff	' keep HX, clear LX
		and	tmp, #$ff	' mask C
		or	IY, tmp
:end

'*******************************************************
' fd 6a Load LX with D
'
z_ld_ly_d	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, DE
		andn	IY, #$ff	' keep HX, clear LX
		shr	tmp, #8		' mask D
		or	IY, tmp
:end

'*******************************************************
' fd 6b Load LX with E
'
z_ld_ly_e	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, DE
		andn	IY, #$ff	' keep HX, clear LX
		and	tmp, #$ff	' mask E
		or	IY, tmp
:end

'*******************************************************
' fd 6c Load LX with HX
'
z_ld_ly_hy	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		andn	IY, #$ff	' keep HX, clear LX
		shr	tmp, #8		' mask HX
		or	IY, tmp
:end

'*******************************************************
' fd 6d Load LX with LX (NOP)
'
z_ld_ly_ly	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
:end

'*******************************************************
' fd 6e Load L with memory (IY+disp8)
'
z_ld_l_my	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		call	#rd_memb_alu
		andn	HL, #$ff	' keep H, clear L
		or	HL, alu
:end

'*******************************************************
' fd 6f Load L with A
'
z_ld_ly_a	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, AF
		andn	IY, #$ff	' keep HX, clear LX
		shr	tmp, #8		' mask A
		or	IY, tmp
:end

'*******************************************************
' fd 70 Load memory (IY+disp8) with B
'
z_ld_my_b	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		mov	alu, BC
		shr	alu, #8		' mask B
		call	#wr_memb_alu
:end

'*******************************************************
' fd 71 Load memory (IY+disp8) with C
'
z_ld_my_c	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		mov	alu, BC
		and	alu, #$ff	' mask C
		call	#wr_memb_alu
:end

'*******************************************************
' fd 72 Load memory (IY+disp8) with D
'
z_ld_my_d	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		mov	alu, DE
		shr	alu, #8		' mask D
		call	#wr_memb_alu
:end

'*******************************************************
' fd 73 Load memory (IY+disp8) with E
'
z_ld_my_e	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		mov	alu, DE
		and	alu, #$ff	' mask E
		call	#wr_memb_alu
:end

'*******************************************************
' fd 74 Load memory (IY+disp8) with H
'
z_ld_my_h	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		mov	alu, HL
		shr	alu, #8		' mask H
		call	#wr_memb_alu
:end

'*******************************************************
' fd 75 Load memory (IY+disp8) with L
'
z_ld_my_l	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		mov	alu, HL
		and	alu, #$ff	' mask L
		call	#wr_memb_alu
:end

'*******************************************************
' fd 77 Load memory (IY+disp8) with A
'
z_ld_my_a	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		mov	alu, AF
		shr	alu, #8		' mask A
		call	#wr_memb_alu
:end

'*******************************************************
' fd 7c Load A with HX
'
z_ld_a_hy	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		and	AF, #$ff	' keep F, clear A
		andn	tmp, #$ff	' mask HX
		or	AF, tmp
:end

'*******************************************************
' fd 7d Load A with LX
'
z_ld_a_ly	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		and	AF, #$ff	' keep F, clear A
		and	tmp, #$ff	' mask LX
		shl	tmp, #8
		or	AF, tmp
:end

'*******************************************************
' fd 7e Load A with memory (IY+disp8)
'
z_ld_a_my	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		call	#rd_memb_alu
		and	AF, #$ff	' keep F, clear A
		shl	alu, #8
		or	AF, alu
:end

'*******************************************************
' fd 84 Add HX to A
'
z_add_a_hy	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		shr	tmp, #8
		long	@alu_add
:end

'*******************************************************
' fd 85 Add LX to A
'
z_add_a_ly	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		and	tmp, #$ff
		long	@alu_add
:end

'*******************************************************
' fd 86 Add memory (IY+disp8) to A
'
z_add_a_my	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		call	#rd_memb_alu
		long	@alu_add
:end

'*******************************************************
' fd 8c Add with carry HX to A
'
z_adc_a_hy	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		shr	tmp, #8
		long	@alu_adc
:end

'*******************************************************
' fd 8d Add with carry LX to A
'
z_adc_a_ly	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		and	tmp, #$ff
		long	@alu_adc
:end

'*******************************************************
' fd 8e Add with carry memory (IY+disp8) to A
'
z_adc_a_my	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		call	#rd_memb_alu
		long	@alu_adc
:end

'*******************************************************
' fd 94 Subtract HX from A 
'
z_sub_a_hy	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		shr	tmp, #8
		long	@alu_sub
:end

'*******************************************************
' fd 95 Subtract LX from A 
'
z_sub_a_ly	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		and	tmp, #$ff
		long	@alu_sub
:end

'*******************************************************
' fd 96 Subtract memory (IY+disp8) from A 
'
z_sub_a_my	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		call	#rd_memb_alu
		long	@alu_sub
:end

'*******************************************************
' fd 9c Subtract with carry HX from A 
'
z_sbc_a_hy	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		shr	tmp, #8
		long	@alu_sbc
:end

'*******************************************************
' fd 9d Subtract with carry LX from A 
'
z_sbc_a_ly	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		and	tmp, #$ff
		long	@alu_sbc
:end

'*******************************************************
' fd 9e Subtract with carry memory (IY+disp8) from A 
'
z_sbc_a_my	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		call	#rd_memb_alu
		long	@alu_sbc
:end

'*******************************************************
' fd a4 And A with HX
'
z_and_a_hy	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		shr	tmp, #8
		long	@alu_and
:end

'*******************************************************
' fd a5 And A with LX
'
z_and_a_ly	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		and	tmp, #$ff
		long	@alu_and
:end

'*******************************************************
' fd a6 And A with memory (IY+disp8)
'
z_and_a_my	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		call	#rd_memb_alu
		long	@alu_and
:end

'*******************************************************
' fd ac Exclusive or A with HX
'
z_xor_a_hy	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		shr	tmp, #8
		long	@alu_xor
:end

'*******************************************************
' fd ad Exclusive or A with LX
'
z_xor_a_ly	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		and	tmp, #$ff
		long	@alu_xor
:end

'*******************************************************
' fd ae Exclusive or A with memory (IY+disp8)
'
z_xor_a_my	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		call	#rd_memb_alu
		long	@alu_xor
:end

'*******************************************************
' fd b4 Or A with HX
'
z_or_a_hy	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		shr	tmp, #8
		long	@alu_or
:end

'*******************************************************
' fd b5 Or A with LX
'
z_or_a_ly	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		and	tmp, #$ff
		long	@alu_or
:end

'*******************************************************
' fd b6 Or A with memory (IY+disp8)
'
z_or_a_my	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		call	#rd_memb_alu
		long	@alu_or
:end

'*******************************************************
' fd bc Compare A with HX
'
z_cp_a_hy	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		shr	tmp, #8
		long	@alu_cp
:end

'*******************************************************
' fd bd Compare A with LX
'
z_cp_a_ly	long	(:end - $ - 1) | (9 << CYCLES_SHIFT)
		mov	tmp, IY
		and	tmp, #$ff
		long	@alu_cp
:end

'*******************************************************
' fd be Compare A with memory (IY+disp8)
'
z_cp_a_my	long	(:end - $ - 1) | (19 << CYCLES_SHIFT)
		mov	ea, #0		' source is patched with the IY offset
		long	@mem_iy		' paste the mem_iy code snippet here
		call	#rd_memb_alu
		long	@alu_cp
:end

'*******************************************************
' fd e1 Pop IY off the stack
'
z_pop_iy	long	(:end - $ - 1) | (14 << CYCLES_SHIFT)
		call	#pop_alu
		mov	IY, alu
:end

'*******************************************************
' fd e3 Exchange stack contents with IY
'
z_ex_sp_iy	long	(:end - $ - 1) | (23 << CYCLES_SHIFT)
		call	#pop_alu
		xor	IY, alu
		xor	alu, IY
		xor	IY, alu
		call	#push_alu
:end

'*******************************************************
' fd e5 Push IY to the stack
'
z_push_iy	long	(:end - $ - 1) | (15 << CYCLES_SHIFT)
		mov	alu, IY
		call	#push_alu
:end

'*******************************************************
' fd e9 Jump to address in IY
'
z_jp_iy		long	(:end - $ - 1) | (8 << CYCLES_SHIFT)
		mov	PC, IY
		jmp	#compile
:end

'*******************************************************
' fd f9 Load SP with IY
'
z_ld_sp_iy	long	(:end - $ - 1)
		mov	SP, IY
:end

snippets_fd
	{00}	long	@z_nop
	{01}	long	@z_ld_bc_imm16	| (patch_imm16 << OP_FLAGS_SHIFT)
	{02}	long	@z_ld_mem_bc_a
	{03}	long	@z_inc_bc
	{04}	long	@z_inc_b
	{05}	long	@z_dec_b
	{06}	long	@z_ld_b_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{07}	long	@z_rlca
	{08}	long	@z_ex_af_af2
	{09}	long	@z_add_iy_bc
	{0a}	long	@z_ld_a_mem_bc
	{0b}	long	@z_dec_bc
	{0c}	long	@z_inc_c
	{0d}	long	@z_dec_c
	{0e}	long	@z_ld_c_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{0f}	long	@z_rrca
	{10}	long	@z_djnz		| (patch_jr_disp8 << OP_FLAGS_SHIFT)
	{11}	long	@z_ld_de_imm16	| (patch_imm16 << OP_FLAGS_SHIFT)
	{12}	long	@z_ld_mem_de_a
	{13}	long	@z_inc_de
	{14}	long	@z_inc_d
	{15}	long	@z_dec_d
	{16}	long	@z_ld_d_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{17}	long	@z_rla
	{18}	long	@z_jr		| (patch_jr_disp8 << OP_FLAGS_SHIFT)
	{19}	long	@z_add_iy_de
	{1a}	long	@z_ld_a_mem_de
	{1b}	long	@z_dec_de
	{1c}	long	@z_inc_e
	{1d}	long	@z_dec_e
	{1e}	long	@z_ld_e_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{1f}	long	@z_rra
	{20}	long	@z_jr_nz		| (patch_jr_disp8 << OP_FLAGS_SHIFT)
	{21}	long	@z_ld_iy_imm16	| (patch_imm16 << OP_FLAGS_SHIFT)
	{22}	long	@z_ld_mem_iy	| (patch_imm16 << OP_FLAGS_SHIFT)
	{23}	long	@z_inc_iy
	{24}	long	@z_inc_hy
	{25}	long	@z_dec_hy
	{26}	long	@z_ld_hy_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{27}	long	@z_daa
	{28}	long	@z_jr_z		| (patch_jr_disp8 << OP_FLAGS_SHIFT)
	{29}	long	@z_add_iy_iy
	{2a}	long	@z_ld_iy_mem	| (patch_imm16 << OP_FLAGS_SHIFT)
	{2b}	long	@z_dec_iy
	{2c}	long	@z_inc_ly
	{2d}	long	@z_dec_ly
	{2e}	long	@z_ld_ly_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{2f}	long	@z_cpl
	{30}	long	@z_jr_nc		| (patch_jr_disp8 << OP_FLAGS_SHIFT)
	{31}	long	@z_ld_sp_imm16	| (patch_imm16 << OP_FLAGS_SHIFT)
	{32}	long	@z_ld_mem_a	| (patch_imm16 << OP_FLAGS_SHIFT)
	{33}	long	@z_inc_sp
	{34}	long	@z_inc_my	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{35}	long	@z_dec_my	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{36}	long	@z_ld_my_imm8	| ((patch_xy_disp8 | patch_imm8) << OP_FLAGS_SHIFT)
	{37}	long	@z_scf
	{38}	long	@z_jr_c		| (patch_jr_disp8 << OP_FLAGS_SHIFT)
	{39}	long	@z_add_iy_sp
	{3a}	long	@z_ld_a_mem	| (patch_imm16 << OP_FLAGS_SHIFT)
	{3b}	long	@z_dec_sp
	{3c}	long	@z_inc_a
	{3d}	long	@z_dec_a
	{3e}	long	@z_ld_a_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{3f}	long	@z_ccf
	{40}	long	@z_ld_b_b
	{41}	long	@z_ld_b_c
	{42}	long	@z_ld_b_d
	{43}	long	@z_ld_b_e
	{44}	long	@z_ld_b_hy
	{45}	long	@z_ld_b_ly
	{46}	long	@z_ld_b_my	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{47}	long	@z_ld_b_a
	{48}	long	@z_ld_c_b
	{49}	long	@z_ld_c_c
	{4a}	long	@z_ld_c_d
	{4b}	long	@z_ld_c_e
	{4c}	long	@z_ld_c_hy
	{4d}	long	@z_ld_c_ly
	{4e}	long	@z_ld_c_my	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{4f}	long	@z_ld_c_a
	{50}	long	@z_ld_d_b
	{51}	long	@z_ld_d_c
	{52}	long	@z_ld_d_d
	{53}	long	@z_ld_d_e
	{54}	long	@z_ld_d_hy
	{55}	long	@z_ld_d_ly
	{56}	long	@z_ld_d_my	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{57}	long	@z_ld_d_a
	{58}	long	@z_ld_e_b
	{59}	long	@z_ld_e_c
	{5a}	long	@z_ld_e_d
	{5b}	long	@z_ld_e_e
	{5c}	long	@z_ld_e_hy
	{5d}	long	@z_ld_e_ly
	{5e}	long	@z_ld_e_my	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{5f}	long	@z_ld_e_a
	{60}	long	@z_ld_hy_b
	{61}	long	@z_ld_hy_c
	{62}	long	@z_ld_hy_d
	{63}	long	@z_ld_hy_e
	{64}	long	@z_ld_hy_hy
	{65}	long	@z_ld_hy_ly
	{66}	long	@z_ld_h_my	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{67}	long	@z_ld_hy_a
	{68}	long	@z_ld_ly_b
	{69}	long	@z_ld_ly_c
	{6a}	long	@z_ld_ly_d
	{6b}	long	@z_ld_ly_e
	{6c}	long	@z_ld_ly_hy
	{6d}	long	@z_ld_ly_ly
	{6e}	long	@z_ld_l_my	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{6f}	long	@z_ld_ly_a
	{70}	long	@z_ld_my_b	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{71}	long	@z_ld_my_c	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{72}	long	@z_ld_my_d	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{73}	long	@z_ld_my_e	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{74}	long	@z_ld_my_h	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{75}	long	@z_ld_my_l	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{76}	long	@z_halt		| (terminate << OP_FLAGS_SHIFT)
	{77}	long	@z_ld_my_a	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{78}	long	@z_ld_a_b
	{79}	long	@z_ld_a_c
	{7a}	long	@z_ld_a_d
	{7b}	long	@z_ld_a_e
	{7c}	long	@z_ld_a_hy
	{7d}	long	@z_ld_a_ly
	{7e}	long	@z_ld_a_my	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{7f}	long	@z_ld_a_a
	{80}	long	@z_add_a_b
	{81}	long	@z_add_a_c
	{82}	long	@z_add_a_d
	{83}	long	@z_add_a_e
	{84}	long	@z_add_a_hy
	{85}	long	@z_add_a_ly
	{86}	long	@z_add_a_my	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{87}	long	@z_add_a_a
	{88}	long	@z_adc_a_b
	{89}	long	@z_adc_a_c
	{8a}	long	@z_adc_a_d
	{8b}	long	@z_adc_a_e
	{8c}	long	@z_adc_a_hy
	{8d}	long	@z_adc_a_ly
	{8e}	long	@z_adc_a_my	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{8f}	long	@z_adc_a_a
	{90}	long	@z_sub_a_b
	{91}	long	@z_sub_a_c
	{92}	long	@z_sub_a_d
	{93}	long	@z_sub_a_e
	{94}	long	@z_sub_a_hy
	{95}	long	@z_sub_a_ly
	{96}	long	@z_sub_a_my	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{97}	long	@z_sub_a_a
	{98}	long	@z_sbc_a_b
	{99}	long	@z_sbc_a_c
	{9a}	long	@z_sbc_a_d
	{9b}	long	@z_sbc_a_e
	{9c}	long	@z_sbc_a_hy
	{9d}	long	@z_sbc_a_ly
	{9e}	long	@z_sbc_a_my	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{9f}	long	@z_sbc_a_a
	{a0}	long	@z_and_a_b
	{a1}	long	@z_and_a_c
	{a2}	long	@z_and_a_d
	{a3}	long	@z_and_a_e
	{a4}	long	@z_and_a_hy
	{a5}	long	@z_and_a_ly
	{a6}	long	@z_and_a_my	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{a7}	long	@z_and_a_a
	{a8}	long	@z_xor_a_b
	{a9}	long	@z_xor_a_c
	{aa}	long	@z_xor_a_d
	{ab}	long	@z_xor_a_e
	{ac}	long	@z_xor_a_hy
	{ad}	long	@z_xor_a_ly
	{ae}	long	@z_xor_a_my	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{af}	long	@z_xor_a_a
	{b0}	long	@z_or_a_b
	{b1}	long	@z_or_a_c
	{b2}	long	@z_or_a_d
	{b3}	long	@z_or_a_e
	{b4}	long	@z_or_a_hy
	{b5}	long	@z_or_a_ly
	{b6}	long	@z_or_a_my	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{b7}	long	@z_or_a_a
	{b8}	long	@z_cp_a_b
	{b9}	long	@z_cp_a_c
	{ba}	long	@z_cp_a_d
	{bb}	long	@z_cp_a_e
	{bc}	long	@z_cp_a_hy
	{bd}	long	@z_cp_a_ly
	{be}	long	@z_cp_a_my	| (patch_xy_disp8 << OP_FLAGS_SHIFT)
	{bf}	long	@z_cp_a_a
	{c0}	long	@z_ret_nz
	{c1}	long	@z_pop_bc
	{c2}	long	@z_jp_nz	| (patch_imm16 << OP_FLAGS_SHIFT)
	{c3}	long	@z_jp		| ((patch_imm16 | terminate) << OP_FLAGS_SHIFT)
	{c4}	long	@z_call_nz	| (patch_imm16 << OP_FLAGS_SHIFT)
	{c5}	long	@z_push_bc
	{c6}	long	@z_add_a_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{c7}	long	@z_rst_00	| (terminate << OP_FLAGS_SHIFT)
	{c8}	long	@z_ret_z
	{c9}	long	@z_ret		| (terminate << OP_FLAGS_SHIFT)
	{ca}	long	@z_jp_z		| (patch_imm16 << OP_FLAGS_SHIFT)
	{cb}	long	@prefix_cb_xx	| (jump_to << OP_FLAGS_SHIFT)
	{cc}	long	@z_call_z	| (patch_imm16 << OP_FLAGS_SHIFT)
	{cd}	long	@z_call		| ((patch_imm16 | terminate) << OP_FLAGS_SHIFT)
	{ce}	long	@z_adc_a_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{c7}	long	@z_rst_08	| (terminate << OP_FLAGS_SHIFT)
	{d0}	long	@z_ret_nc
	{d1}	long	@z_pop_de
	{d2}	long	@z_jp_nc	| (patch_imm16 << OP_FLAGS_SHIFT)
	{d3}	long	@z_out_port_a
	{d4}	long	@z_call_nc	| (patch_imm16 << OP_FLAGS_SHIFT)
	{d5}	long	@z_push_de
	{d6}	long	@z_sub_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{d7}	long	@z_rst_10	| (terminate << OP_FLAGS_SHIFT)
	{d8}	long	@z_ret_c
	{d9}	long	@z_exx
	{da}	long	@z_jp_c		| (patch_imm16 << OP_FLAGS_SHIFT)
	{db}	long	@z_in_a_port
	{dc}	long	@z_call_c	| (patch_imm16 << OP_FLAGS_SHIFT)
	{dd}	long	@prefix_dd_xx	| (jump_to << OP_FLAGS_SHIFT)
	{de}	long	@z_sbc_a_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{df}	long	@z_rst_18	| (terminate << OP_FLAGS_SHIFT)
	{e0}	long	@z_ret_po
	{e1}	long	@z_pop_iy
	{e2}	long	@z_jp_po	| (patch_imm16 << OP_FLAGS_SHIFT)
	{e3}	long	@z_ex_sp_iy
	{e4}	long	@z_call_po	| (patch_imm16 << OP_FLAGS_SHIFT)
	{e5}	long	@z_push_iy
	{e6}	long	@z_and_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{e7}	long	@z_rst_20	| (terminate << OP_FLAGS_SHIFT)
	{e8}	long	@z_ret_pe
	{e9}	long	@z_jp_iy	| (terminate << OP_FLAGS_SHIFT)
	{ea}	long	@z_jp_pe	| (patch_imm16 << OP_FLAGS_SHIFT)
	{eb}	long	@z_ex_de_hl
	{ec}	long	@z_call_pe	| (patch_imm16 << OP_FLAGS_SHIFT)
	{ed}	long	@prefix_ed_xx	| (jump_to << OP_FLAGS_SHIFT)
	{ee}	long	@z_xor_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{ef}	long	@z_rst_28	| (terminate << OP_FLAGS_SHIFT)
	{f0}	long	@z_ret_p
	{f1}	long	@z_pop_af
	{f2}	long	@z_jp_p		| (patch_imm16 << OP_FLAGS_SHIFT)
	{f3}	long	@z_di
	{f4}	long	@z_call_p	| (patch_imm16 << OP_FLAGS_SHIFT)
	{f5}	long	@z_push_af
	{f6}	long	@z_or_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{f7}	long	@z_rst_30	| (terminate << OP_FLAGS_SHIFT)
	{f8}	long	@z_ret_m
	{f9}	long	@z_ld_sp_iy
	{fa}	long	@z_jp_m		| (patch_imm16 << OP_FLAGS_SHIFT)
	{fb}	long	@z_ei
	{fc}	long	@z_call_m	| (patch_imm16 << OP_FLAGS_SHIFT)
	{fd}	long	@prefix_fd_xx	| (jump_to << OP_FLAGS_SHIFT)
	{fe}	long	@z_cp_imm8	| (patch_imm8 << OP_FLAGS_SHIFT)
	{ff}	long	@z_rst_38	| (terminate << OP_FLAGS_SHIFT)
